////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VMOutput.h: interface for the CVMOutput class.
// vmdecl: __cdecl for Win & empty for Lunux
//
//////////////////////////////////////////////////////////////////////

#if !defined(CVMGen_H_INCLUDED_)
#define CVMGen_H_INCLUDED_
//	Pre-includes: <math.h>,<iostream>

#if defined WIN32
#define vmdecl	__cdecl
#elif defined WIN64
#define vmdecl	__cdecl
#else
#define vmdecl
#endif

#define CVMGen_NO_VEGAS
#define CVMGen_NO_FOAM
///////////////////////////////////// Rnd
class CVMRnd {
	int m_nSkip; //(def: 0) used to skip 'm_nSkip' numbers before next generated
public:
	CVMRnd();
	tReal Rnd();
	void RndArr(long n, tReal* arr);
	void SetFilter(int itNo, int itMask); //used to generate '(numNo % itMask)==itNo' sub-set of numbers
	virtual bool IsReady(); //'true' by default
protected:
	virtual tReal RndImpl()=0;

public:
	/*
	*	create RND-generator instance of 'default' type (depending on the 'tReal' type)
	*	Warning: always use GetRndRef() in Monte-Carlo generators!
	*/
	static CVMRnd* CreateDefaultRnd();

public:
	/*
	*	get reference to 'global' RND-object
	*	Note: 'global' RND-object to be used as 'the only' RND-object in project
	*/
	static CVMRnd* GetRndRef();
	/*
	*	create 'global' RND-object
	*	if(itMask>1)==> every '(n % itMask) == itNo' number is generated.
	*/
	static void ResetRndGlobal(int itNo=-1, int itMask=-1);
	static void ClearRndGlobal();
private:
	static CVMRnd* m_RndGlobal;
};

class CVMRndMZ : public CVMRnd { //G.Marsaglia, A.Zaman' 1987
	double m_U[97],m_C,m_CD,m_CM;
	int m_i,m_j;
public:
	CVMRndMZ();
	CVMRndMZ(long na1,long na2,long na3,long nb1);
	void GetSeed(double (*u)[97], double* c, double* cd, double* cm, int* i, int* j);
	void SetSeed(double (*u)[97], double c, double cd, double cm, int i, int j);
	void Test();
protected:
	virtual tReal RndImpl();
private:
	void Init(long na1,long na2,long na3,long nb1);
};

class CVMRndQD : public CVMRnd { //Uses RND-procedures from QD package
public:
	CVMRndQD();
	virtual bool IsReady(); //'true' by default
protected:
	virtual tReal RndImpl();
};

///////////////////////////////////// Gen
#define Decl_CVMGen

#define CVMGen_MAXDIM		20
//#define CVMGen_MAXDIM		8
#define CVMGen_MAXUSERVARS	100

class CVMGen {
public:
	class CVMGenFunc {
	public:
		virtual tReal Call(tReal* params)	=0;
		virtual CVMGenFunc* Clone() const	=0;
	};
	class CVMGenFuncImpl : public CVMGenFunc {
		tReal (vmdecl *m_pFunc)(tReal*); // function pointer
	public:
		CVMGenFuncImpl(tReal(vmdecl *func)(tReal*));
		virtual tReal Call(tReal* params);
		virtual CVMGenFunc* Clone() const;
	};
public:
	int m_nDim;

	//Integration result
	tReal m_Integral;
	tReal m_Error;
	tReal GetIntegral() { return m_Integral; }
	tReal GetIntegralError() { return m_Error; }

	//Event generated
	tReal m_GenValues[CVMGen_MAXDIM];	// generated event
	tReal m_Weight;			// for weighted events, +1 for unweighted
	tReal m_WgtError;		// error value for generated m_Weight value
	tReal m_WeightDebug;	// for debug only, in unw.-generation includes overweight for f>fMax (factor: f/fMax)

	//Event generation statistics (TODO)
	//long m_nEvtsGenerated;
	//long m_nEvtsFailed;
	//tReal m_WgtSumGenerated;
	//tReal m_WgtDebugSumGenerated;
	//void GenReset();
	//bool GenGetStatistics(long* nEvts, tReal* errStat, tReal* errSystFMax);

	//Statistics
	tReal m_Volume;
//protected:
	tReal m_VarMin[CVMGen_MAXDIM];	// limits
	tReal m_VarMax[CVMGen_MAXDIM];	// limits
	long m_nFuncCalls;
protected:
	CVMGenFunc* m_pFunc;			// function pointer
	bool m_bFuncOwner;				// or if 'm_pFunc' is an external link
	CVMRnd* m_rnd;
	bool m_bRndIsExternal;			// if 'm_rnd' is an external link
	//
	char* m_Title;					//Short text used in debug output only
	char* m_Metadata;				//Any user text
	char* m_UserVarsNames[CVMGen_MAXUSERVARS];	//Any user variables to be kept with grid (e.g. actual limits values)
	tReal m_UserVarsVals[CVMGen_MAXUSERVARS];	//Any user variables to be kept with grid (e.g. actual limits values)
public:
	CVMGen();
	~CVMGen();
	// Note: rndCpy will be deleted ==> call Init(....,new CVMRndSimple());
	bool Init(int nDim, tReal* _VarMin, tReal* _VarMax, tReal(vmdecl *func)(tReal*), CVMRnd* rndCpy=NULL);
	bool Init(int nDim, tReal* _VarMin, tReal* _VarMax, const CVMGenFunc* func, CVMRnd* rndCpy=NULL);
	bool Init_ExternalFunc(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* funcExternal, CVMRnd* rndCpy=NULL);
	void ReleaseExtFunction(); //used to avoid further use of 'external' function object
protected:
	virtual bool InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy=NULL);
public:
	virtual void SetExpectedIntegral(tReal valExpected); //To be called after 'Init' but before 'Integrate'
	virtual tReal Integrate(double fPrecision=0.01) = 0;
	virtual bool GenEvent();		//Generate unweighted event
	virtual bool GenEventWgt() = 0; //Generate weighted event (faster than GenEvent())
	virtual double GetGenEventEfficiency();	//Efficiency of previous 'GenEvent' calls
	virtual const char* ClassName();
	CVMRnd* GetRndRef();
	CVMGenFunc* GetFuncRef() { return m_pFunc; } // For external tests of function
	//
	void SetMetadata(const char* info);
	void SetTitle(const char* title);
	const char* GetMetadata();
	void AddUserVariable(const char* var, tReal val);
	bool GetUserVariable(const char* var, tReal* val);
	int GetUserVariablesNum();
protected:
	tReal CallFunction(tReal* params);
};

class CVMGenPlain : public CVMGen {
public:
	CVMGenPlain();
	virtual void SetExpectedIntegral(tReal valExpected); //To be called after 'Init' but before 'Integrate'
	virtual tReal Integrate(double fPrecision=0.01);
	virtual bool GenEventWgt();
};

/*
*	Features:
*	- Adaptive
*	- Numerical precision (digits overflow) checks
*/
class CVMGenLite : public CVMGen {
private:
	class CVMGenLiteCell { //Size: sizeof(tReal)*(2*MAXDIM(20)+4)+4*(7)->~400bytes[double],1.6k[qd]
	public:
		class CLocals {
			/*
			*	Size: sizeof(tReal)*(CVMGen_MAXDIM*(6*2+1))+CVMGen_MAXDIM*8 =|qd,Dmax=20|~8.5k
			*	+ (sizeof(tReal)+4)*(CVMGen_MAXDIM*10) ~7.2k
			*/
		public:
			// Sub-cell variables ([0]:left, [1]:right)
			tReal _FSum[CVMGen_MAXDIM][2];
			tReal _FSqr[CVMGen_MAXDIM][2];
			tReal _FMax[CVMGen_MAXDIM][2];
			tReal _FMinGt0[CVMGen_MAXDIM][2];
			tReal varMid[CVMGen_MAXDIM];
			long _nEv[CVMGen_MAXDIM][2];
			tReal _ISubVal[CVMGen_MAXDIM][2];
			tReal _SDSub[CVMGen_MAXDIM][2];
			// In-cell results variables (ALWAYS use local[LEVEL] variables - they must preserve after child calls)
			tReal _FSumCell;
			tReal _FSqrCell;
			tReal varMidSel;
			tReal selVal1;
			tReal selVal2;
			tReal selVariance1;
			tReal selVariance2;
			tReal _SubVol;
			tReal _FMax1;
			tReal _FMax2;
			//
			tReal var[CVMGen_MAXDIM]; //used in event generation cycle
			//
			tReal fAveBin[CVMGen_MAXDIM][10]; //Average F-values in 10 buns along X-axis
			long nAveBin[CVMGen_MAXDIM][10]; //Average F-values in 10 buns along X-axis
		};
		CVMGenLite* m_Gen;
		int m_nSubOrder;
		short m_nDimOrder[CVMGen_MAXDIM]; //4*20~80b
		tReal m_VarMin[CVMGen_MAXDIM];	//limits, sizeof(tReal)*20~640b[qd]
		tReal m_VarMax[CVMGen_MAXDIM];	//limits, sizeof(tReal)*20~640b[qd]
		tReal m_Volume;			// to cache value
		tReal m_Integral;
		long m_nEvts;			// Number of events sampled for integration
		tReal m_Variance;		// Sdt.Dev.^2/Nevents == error^2
		tReal m_FuncMax;		//used for unweighted generation
		CVMGenLiteCell* m_Sub1;
		CVMGenLiteCell* m_Sub2;
		int m_iSplitAxis;
		int m_iSplitAxisPrev;
		int m_iSplitAxisPrevTimes;
		tReal m_fWghtExpected;	// to estimate required precision (1: full integral, -1 if unused)
		//asdfg - check how 'm_fWghtExpected' is used!
		bool m_bIntegrationSkipped; //set to 'true' if integration is skipped & pre-defined value left
	public:
		CVMGenLiteCell(CVMGenLite* _Gen, int iSubOrder);
		~CVMGenLiteCell();
		bool IsPrecisionOK();
		bool Integrate(long nCashedCalls, tReal** arrCashedData);
		int SelectSplitAxis(CLocals* loc);
		void SaveCell(std::ostream& ofs);
		bool LoadCell(std::istream& ifs, int iVer, int nDig, int iTypeID);
		void UpdateResults(); //Updates integral/error values. To be called if child cells are changed.
		// Debug
		void ViewCellPicture(int id=-1); //id: to create unique file "cell_%ld.eps"
	};
public:
	CVMGenLiteCell* m_Cells;

	// Integration settings
	tReal m_fIntegralExpected;	//ToBe set AFTER Init()!, input parameter, affects rate of convergence only, don't set if unknown (default: 0)
	bool m_bPreIntegrationMode;	//Flag used to cancel some integration-specific checks in 'estimate' mode
	tReal m_fPrecisionReq;
	tReal m_fCLFactor;			//(def: ~3) Integral[sqrt(1/2pi)exp(-x2/2),x,-CLFactor,+CLFactor]=C.L.
	tReal m_RelVarianceMax;		//Max[(sigma^2/N)/Integral] allowed by 'm_fPrecisionReq' == m_fPrecisionReq^2*CLfactor
	int m_iAxisSelMode;			//0:(not next), 1:(lowest SD<->mid SD), //2: random
	int m_nSubOrderMin;
	int m_nSubOrderMax;
	long m_SubEvts;
	long m_nFirstOrders;		//number of 'First' orders (with separate 'm_SubEvts')
	long m_SubEvtsFirstOrders;	//num of sub-events at 'First' orders (order <= m_nFirstOrders)
	bool m_bUseEventsSmart;
	/*
	*	m_fMaxWgtFactor: restrict fMax/fAverage in cell by 'm_fMaxWgtFactor'
	*	-1: unused(default), <50: not recommended
	*	Not precise: weight may finally occur more than m_fMaxWgtFactor
	*/
	tReal m_fMaxWgtFactor;
	/*
	*	Restrict error in every cell (<0 if not applied (default))
	*	WARNING: this parameter is NOT currently saved to GRID-file.
	*/
	tReal m_fCellErrMaxSqr;
	bool m_bUpdateIntegrationNext; //Update previous integration during next Integrate()-call.
	bool m_bUsePreviousGrid;	//Use existing(e.g. loaded) grid for integration.
	bool m_bMergeSkippedCells;	// saves memory, may slow down unw-generation (TODO: check it!)

	// Generation settings
	bool m_bGenWeightErrors;	// slower generation due to sqrt() calls for every event
	bool m_bGenFixThreshold;	// fix 'rnd'-threshold for all iterations in GenEvent()
	/*
	*	Fmax-overlap processing modes:
	*		- no
	*		- update Fmax
	*		- generate with Max=Fmax*FMaxSafetyFactor
	*		- [wrong, debug only!] generate overlapped events with weight ~Fval/Fmax (using 'WeightDebug')
	*	Warning: don't mix these modes!
	*/
	bool m_bUpdateMaxValue;		// update 'max' value in cell ==> later unweighted-events may be much slower
	double m_fMaxSafetyFactor;	// (default: -1) generate with Max=Fmax*SafetyFactor (<0 if not applied)

	// Statistics
	long m_nCellsCnt;
	int m_nSOrderMaxActual;
	long m_nCellGenRepeated;	// Repeated generations if N[any half]==0
	long m_nMachinePrecFault;	// Number of cell failed with 'machine precision exceeded' fault.
	long m_nCellIntegrFault;	// Number of cell integration failed (by any reason)
	bool m_bLogCellsCnt;		//Log 'm_nCellsCnt' every 1000 cells (def.: true)

	//debug
	CVMGenLiteCell* m_MaxFinalVarianceCell;
	long m_nMaxFinalVariance;
	unsigned long m_nGenEvtFailed;
	unsigned long m_nGenEvtIterations[100];
	// buffers used
	tReal*** _Cache1;
	tReal*** _Cache2;
	int*** _iCache1;
	CVMGenLiteCell::CLocals* m_LocalsCache;
public:
	CVMGenLite();
	~CVMGenLite();
protected:
	virtual bool InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy=NULL);
public:
	virtual void SetExpectedIntegral(tReal valExpected); //To be called after 'Init' but before 'Integrate'
	virtual tReal Integrate(double fPrecision=0.01);
	virtual bool GenEvent();				//Generate unweighted event
	virtual bool GenEventWgt();				//Generate weighted event
	virtual bool GenEvtDebug();				//Generate event according to GRID only. No function calls, fast but incorrect.
private:
	unsigned long m_iGenEvtCalls;			// used to calculate efficiency of 'GenEvent' calls
	unsigned long m_iGenEvtLoops;			// used to calculate efficiency of 'GenEvent' calls
public:
	virtual double GetGenEventEfficiency();	//Efficiency of previous 'GenEvent' calls
	void SaveGrid(const char* fn);
	bool LoadGrid(const char* fn);
	/*
	*	MergeWithGrid(): combine current grid with other one.
	*	The other grid MUST have similar cell-by-cell structure.
	*	The usual way to obtain similar grids:
	*		- Integrate in 'm_bUsePreviousGrid && !m_bMergeSkippedCells' mode with different RND seeds.
	*
	*	The values are mixed in proportion to 'm_SubEvts', m_SubEvts is updated with other->m_SubEvts.
	*
	*	Warning: current GRID may be destroyed if operation failed.
	*/
	bool MergeWithGrid(const char* fn);
	/*
	*	ApplyGridFactor() is used to:
	*	1. Co-scale grid if function is scaled.
	*	2. Start integration from pre-created but empty grid (f==0.).
	*/
	void ApplyGridFactor(double f, bool bAffectIExpected=true);
	void ViewGrid(int dim1=0, int dim2=1);					//implemented if '#ifdef CVMHistLite_H_INCLUDED_'
	void ViewCellsLog(int iOrderMin=-1, int iOrderMax=-1);	//implemented if '#ifdef CVMHistLite_H_INCLUDED_'
	/*
	*	Create debug plots,
	*	id: 0(cells order), 1(fMax/fAve), 2(fMax/fAve*cellWeight),
	*	3: ave,max(fMax/fAve)[cellWghtS]+Ncells[cellWghtS]
	*	4: NCells/CellRelativeError + NCells/CellAbsError(scaled)
	*	[todo:](Ncells/m_Integral), (order-falls)
	*/
	void ViewCellsPlot(int id);								//implemented if '#ifdef CVMHistLite_H_INCLUDED_'
//#endif //CVMHistLite_H_INCLUDED_
	void DropOutCells(int iOrderMin, int iOrderMax=-1); //for debug only
	void DropOutCellsPrec(tReal fPrecision, bool bDropGoodCells); //for debug only
	void GetMaxSplittingOrders(long* iMaxOrders); //for debug only
	static const char* CLASS_NAME;
	virtual const char* ClassName();

public:
	//TODO: move next function to common library
	//nDig==0 means ==sizeof(tReal)
	//iTypeID==0 means unknown
	static bool LoadBinaryVal(std::istream& ifs, tReal& res, int nDig, int iTypeID);
};

#ifndef CVMGen_NO_VEGAS
#define CVMVegas_AXBINMAX	50	//Max. number of intervals per coordinate axis
#define Decl_CVMGenVegas
class CVMGenVegas : public CVMGen {
public:
	tReal m_Chi2It; //Chi^2 per iteration
	tReal m_StdDevIt; //Standard deviation per iteration //useless to keep?
//private:
	tReal m_SI,m_SI2,m_SWGT,m_SCHI;//Sub-results to allow 'bUseOldRes' option
	int m_nIt; //Iteration number
	tReal m_spGrid[CVMVegas_AXBINMAX][CVMGen_MAXDIM];//Actual subdivision of N-dimensional coordinate space
	long m_nAxisParts;//Number of intervals per coordinate axis
public:
	CVMGenVegas();
	virtual tReal Integrate(double fPrecision=0.01);
	tReal Integrate(double fPrecision, bool bUseOldGrid, bool bUseOldRes, long nCalls, int nItMax);
	virtual bool GenEventWgt();
};
#endif // #ifndef CVMGen_NO_VEGAS

#if !defined(CVMGen_NO_FOAM)
#define Decl_CVMGenFoam
class CVMGenFoam : public CVMGen {
public:
	class TFOAM_INTEGRAND{ // Abstract class representing n-dimensional real integrand function
	public:
		virtual double Density(int ndim, double*) = 0;
	};
	class TFDISTR: public TFOAM_INTEGRAND { // Collection of the testing functions
	private:
		int    m_Type;                      // Type of function
		double m_Pi;                        // constant pi
	public:
		TFDISTR(int t=0);                     // Constructor
		virtual ~TFDISTR();               // Destructor
		// methods
	public:
		double Density(int, double *);      // MainTesting functions
		double Sphere( int, double *);
		double Void(   int, double *);
		double Diagon( int, double *);
		double Camel(  int, double *);
		double Gauss(  int, double *);
		double Ridge(  int, double *);
	};
	class TFMAXWT{
	// Small auxiliary class for controling MC weight.
	// It provides certain measure of the maximum weight depending on small parameter epsilon.
	// It creates and uses 2 histograms of the TH1D class.
	// User defines no. of bins nBin,  nBin=1000 is  recommended wmax defines weight range (1,wmax), it is adjusted "manually"
	private:
		double  m_Nent;
		int     m_nBin;
		double  m_wmax;
	public:
		CVMHistLite  *m_WtHst1;      // histogram of wt
		CVMHistLite  *m_WtHst2;      // histogram of wt with wt
	public:
		TFMAXWT(const double, const int);   // Principal Constructor
		~TFMAXWT();                         // Destructor
		void Reset();                       // Reset
		TFMAXWT& operator =(TFMAXWT &);     // operator =
		void Fill(double);
		void Make(const double, double&);
		void GetMCeff(const double, double&, double&);  // get MC efficiency= <w>/wmax
	};
	class TFVECT { //Auxiliary class TFVECT of n-dimensional vector
	private:
		int     m_Dim;                     // Dimension
		double *m_Coords;                  // [m_Dim] Coordinates
		TFVECT  *m_Next;                   // pointer for tree construction
		TFVECT  *m_Prev;                   // pointer for tree construction
		const static int m_chat =0;        // chatability (for debug)
	public:
		TFVECT();                           // Constructor
		TFVECT(const int );                 // USER Constructor
		TFVECT(const TFVECT &);              // Copy constructor
		~TFVECT();                          // Destructor
		// Operators
		TFVECT& operator =(const TFVECT&);   // = operator; Substitution (const ?)
		double &operator[](int);           // [] provides POINTER to coordinate
		TFVECT& operator =(double []);      // LOAD IN entire double vector
		TFVECT& operator =(double);         // LOAD IN double nume
		// Other
		TFVECT& operator+=(const  TFVECT&);       // +=; add vector u+=v      (FAST)
		TFVECT& operator-=(const  TFVECT&);       // +=; add vector u+=v      (FAST)
		TFVECT& operator*=(const double&);       // *=; mult. by scalar v*=x (FAST)
		TFVECT  operator+( const  TFVECT&);       // +;  u=v+s, NEVER USE IT, SLOW!!!
		TFVECT  operator-( const  TFVECT&);       // -;  u=v-s, NEVER USE IT, SLOW!!!
		void Print(void);                  // Prints vector
		void PrintList(void);              // Prints vector and the following linked list
		const int &GetDim(void);           // Returns dimension VALUE
		double GetCoord(int i){return m_Coords[i];};   // Returns coordinat
	};
	class TFCELL {//Implements single cell of foam in k dimensions hyper-cubical subspace and n dimensions in simplical subspace.
		//   static, the same for all cells!
	private:
		short int     m_kDim;             // Dimension of h-cubical subspace
		short int     m_nDim;             // Dimension of simplical subspace
		short int     m_OptMCell;         // Option Mega-Cell
		short int     m_OptCu1st;         // m_OptCubFirst=1, Numbering of dims starts with h-cubic, =0 simpl.
		int           m_nVert;            // No. of vertices in the simplex = m_nDim+1
		//   MEMBERS
	private:
		//--- MC pool (not used) ---
		int    m_nPool;                   // No of events in the MC pool
		TFVECT *m_Pool;                   // [m_nPool] Linked list of MC wt=1 event pool
		//--- linked tree organization ---
		TFCELL **m_Cell0;                 //! Common root cell, for positioning of pointers in the tree
		TFVECT **m_Vert0;                 //! Common root cell, for positioning of pointers in the tree
		int    m_Serial;                  // Serial number (index in m_Cell0)
		int    m_Status;                  // Status (active, inactive)
		int    m_Parent;                  // Pointer to parent cell
		int    m_Daught0;                 // Pointer to daughter 1
		int    m_Daught1;                 // Pointer to daughter 2
		//--- M.C. sampling and choice of the best edge ---
	private:
		double m_Xdiv;                    // Factor for division
		int    m_Best;                    // Best Edge for division
		//--- Integrals of all kinds ---
		double m_Volume;                  // Cartesian Volume of cell
		double m_Integral;                // Integral over cell (estimate from exploration)
		double m_Drive;                   // Driver  integral, only for cell build-up
		double m_Primary;                 // Primary integral, only for MC generation
		//--- Geometry of the cell ---
		int   *m_Verts;                   // [m_nVert] Pointer to ARRAY of vertices in SIMPLEX subspace
		TFVECT *m_Posi;                   // Pointer to position vector,  in H-CUBIC subspace
		TFVECT *m_Size;                   // Pointer to size vector,      in H-CUBIC subspace
		// Methods  
	public:
		TFCELL();
		TFCELL(int, int, int, int);        // User Constructor
		TFCELL(TFCELL &);                  // Copy Constructor
		~TFCELL();                         // Destructor
		void  Fill(int, int, int, int, int*, TFVECT*,TFVECT*); // Assigns values of attributes
		TFCELL&  operator=(TFCELL&);       // Substitution operator (never used)
		TFVECT*& operator[](int);          // Pointer to one of vertices
		int  operator()(int);              // integer pointer to one of vertices
		//--------------- Geometry ----------------------------------
		double  GetXdiv(void){  return m_Xdiv;};          // Pointer to Xdiv
		int     GetBest(void){  return m_Best;};          // Pointer to Best
		void    SetBest(int    Best){ m_Best =Best;};     // Set Best edge candidate
		void    SetXdiv(double Xdiv){ m_Xdiv =Xdiv;};     // Set x-division for best edge cand.
		void    GetHcub(  TFVECT&, TFVECT&);   // Get position and size vectors (h-cubical subspace)
		void    GetHSize( TFVECT& );           // Get size only of cell vector  (h-cubical subspace)
		void    GetXSimp(TFVECT &, TFVECT & , int );
		//--------------- Integrals/Volumes -------------------------
		void    CalcVolume(void);                         // Calculates volume of cell
		double  GetVolume(void){ return m_Volume;};       // Volume of cell
		double  GetIntg(void){  return m_Integral;};      // Get Integral
		double  GetDriv(void){  return m_Drive;};         // Get Drive
		double  GetPrim(void){  return m_Primary;};       // Get Primary
		void    SetIntg(double Intg){ m_Integral=Intg;};  // Set true integral
		void    SetDriv(double Driv){ m_Drive   =Driv;};  // Set driver integral
		void    SetPrim(double Prim){ m_Primary =Prim;};  // Set primary integral
		//--------------- linked tree organization ------------------
		int     GetStat(void){ return m_Status;};         // Get Status
		void    SetStat(int Stat){ m_Status=Stat;};       // Set Status
		int     GetnVert(void){ return m_nVert;};         // Get No. of vrtices in the list
		TFCELL* GetPare(void){if(m_Parent<0){return NULL;}else{return m_Cell0[m_Parent];}}; // Get Pointer to pointer of parent vertex
		TFCELL* GetDau0(void){if(m_Daught0<0){return NULL;}else{return m_Cell0[m_Daught0];}};// Get Pointer to 1-st daughter vertex
		TFCELL* GetDau1(void){if(m_Daught1<0){return NULL;}else{return m_Cell0[m_Daught1];}};// Get Pointer to 2-nd daughter vertex
		void   SetDau0(int Daug){ m_Daught0 = Daug;};    // Set pointer to 1-st daughter
		void   SetDau1(int Daug){ m_Daught1 = Daug;};    // Set pointer to 2-nd daughter
		void   SetSerial(int Serial){ m_Serial=Serial;}; // Set serial number
		int    GetSerial(void){ return m_Serial;};       // Get serial number
		//--- other ---
		void   SetCell0(TFCELL **Cell0){m_Cell0 =Cell0;}; // Set root cell pointer
		void   SetVert0(TFVECT **Vert0){m_Vert0 =Vert0;}; // Set root vertex pointer
		void   Print(void);               // Prints cell content
	};
	class TFMATRIX{ //Auxiliary class TFMATRIX of  square matrices
		//Needed essentialy for calculating volume of the simplex
	private:
		int Dim;                                // Dimension  
		int Dim2;                               // Dimension  
		double *Matrix;                         // [dim2] Table of elements
	public:
		TFMATRIX();
		TFMATRIX(int dim );
		TFMATRIX(TFMATRIX &From);
		~TFMATRIX();
		TFMATRIX& operator =(TFMATRIX &From);
		double& operator()(int i, int j);
		double Determinant();
		void Print();
	};
	class TFPARTITION{ //Auxiliary class for organizing loop over partitions
	private:
		int m_len;                  // lenght=dimension
		int *m_digits;              // [m_len] Partition
	public:
		TFPARTITION(int len);
		~TFPARTITION();
		void Reset();
		int Next();
		const int& Digit(int i);
	};
	class TFOAM{ //Class TFOAM
	private:
		//-------------- Input parameters
		char  m_Name[128];       // Name of a give instance of the FOAM class
		char m_Version[10];      // Actual VERSION of the Foam like (2.xx)
		char  m_Date[40];        // Release DATE   of the Foam
		int m_nDim;              // Dimension of the simplical subspace
		int m_kDim;              // Dimension of the hyper-cubical subspace
		int m_TotDim;            // Total dimension = m_nDim+m_kDim
		int m_nCells;            // Maximum number of cells (from input)
		int m_vMax;              // Maximum number of vertices (calculated)
		int m_LastVe;            // index of the last vertex
		int m_RNmax;             // maximum r.n. requested at once
		//-------------------
		int m_OptDrive;          // type of Drive =0,1,2 for TrueVol,Sigma,WtMax
		int m_OptEdge;           // decides whether vertices are included in the sampling
		int m_OptPeek;           // type of Peek =0,1,2 for maximum, random, random (new)
		int m_OptOrd;            // Single root simplex for OptOrd=1, otherwise root h-cubic
		int m_OptMCell;          // MegaCell option, slim memory for h-cubes
		int m_Chat;              // Chat=0,1,2 chat level in output, Chat=1 normal level
		int m_OptDebug;          // m_OptDebug=1, additional histogram, SetDirectory(1)      DIPSWITCH
		int m_OptCu1st;          // m_OptCubFirst=1,0, Numbering of dims starts with h-cubic DIPSWITCH
		int m_OptVert;           // m_OptVert=0,1 (Vertices of simplex are stored=0 or not=1)
		int m_OptRej;            // OptRej=0,1, rejection in MC off, on (wt=1 events).
		//-------------------
		int  m_nBin;             // binning of edge-histogram in cell exploration
		int  m_nProj;            // number of projection edges
		int  m_nSampl;           // No of sampling when dividing (exploring) cell
		int  m_EvPerBin;         // maximum number of EFFECTIVE event per bin
		//-------------------
		int  m_N0Si;             // Start of numbering dimensions  for simplex hyperspace (see m_OptCu1st)
		int  m_N0Cu;             // Start of numbering dimensions  for hypercubic  hyperspace
		int  m_P0Si;             // Start of numbering projections for simplex hyperspace
		int  m_P0Cu;             // Start of numbering projections for hypercubic  hyperspace
		//-------------------  MULTI-BRANCHING ---------------------
		int    *m_MaskDiv;       //! [m_nProj] Dynamic Mask for  cell division, h-cubic + simplical
		int    *m_InhiDiv;       //! [m_kDim]  Flags alowing to inhibit cell division, h-cubic projection/edge
		int     m_OptPRD;        // General Option switch for PRedefined Division, for quick check
		TFVECT **m_XdivPRD;      //! Lists of division values encoded in one vector per direction
		//-------------------  GEOMETRY ----------------------------
		int m_NoAct;             // number of active cells
		int m_LastCe;            // index of the last cell
		TFCELL **m_Cells;        // [m_nCells] array of ALL cells
		TFVECT **m_VerX;         // [m_vMax] array of pointers to vertex vectors
		//------------------ M.C. generation----------------------------
		TFMAXWT *m_MCMonit;      // Monitor of MC efficiency
		double m_MaxWtRej;       // Maximum weight in rejection for getting wt=1 events
		TFCELL **m_CellsAct;     //! Array of pointers to active cells, constructed at the end of build-up
		double *m_PrimAcu;       //! Array of Cumulative Primary of active cells (for fast MC generation!)
		//
		CVMHistLite **m_HistEdg;    // Array of pointers to histograms
		CVMHistLite *m_HistWt;     // histograms of MC wt
		//
		double *m_MCvect;        // [m_TotDim] Generated MC vector for the outside user
		double  m_MCwt;          // MC weight
		tReal *m_Rvec;          // [m_RNmax] random number vector from r.n. generator m_TotDim+1 maximum elements
		//----------- Procedures 
		TFOAM_INTEGRAND *m_Rho;  // pointer to abstract class providing function to integrate
		CVMRnd *m_PseRan;        // volatile ref. to generator of pseudorandom numbers (VM: do not delete!)
		//----------- Statistics and MC results
		long m_nCalls;           // No of function calls
		long m_nEffev;           // Total no of effective events in build-up
		double m_SumWt, m_SumWt2;// Sum of wt and wt^2
		double m_SumOve;         // Sum of overveighted events
		double m_NevGen;         // No of MC events
		double m_WtMax, m_WtMin; // Max/Min wt
		double m_Prime;          // Primary integral Iprim (Itrue=Iprim<wt>)
		double m_MCresult;       // True Integral Itrue from MC series
		double m_MCerror;        // and its error
		//----------  working space for CELL exploration -------------
		double *m_Lambda;        // [m_nDim] Internal params of the simplex l_0+l_1+...+l_{nDim-1}<1
		double *m_Alpha;         // [m_kDim] Internal params of the hyp-cubic 0<a_i<1
		//////////////////////////////////////////////////////////////////////////////////////////////
		//                                     METHODS                                              //
		//////////////////////////////////////////////////////////////////////////////////////////////
	public:
		TFOAM();                          // Constructor
		TFOAM(const char*);               // User constructor
		~TFOAM();                         // Destructor
		TFOAM(const TFOAM&);              // Copy Constructor  NOT USED
		TFOAM& operator=(const TFOAM& );  // substitution      NOT USED 
		// Initialization
		void Initialize(CVMRnd*, TFOAM_INTEGRAND*); // Initialization of the FOAM (grid, cells, etc).
		void InitVertices(void);          // Initializes first vertices of the basic cube
		void InitCells(void);             // Initializes first n-factorial cells inside original cube
		int  CellFill(int, TFCELL*, int*,TFVECT*,TFVECT*); // fill next cell and return its index
		void LinkCells(void);             // lift up cells after re-read from disk
		void Explore(TFCELL *Cell);       // Exploration of new cell, determine <wt>, wtMax etc.
		void Carver(int&,double&,double&);// Determine the best edge, wtmax   reduction
		void Varedu(double [], int&, double&,double&); // Determine the best edge, variace reduction
		void MakeLambda(void);            // provides random point inside simplex
		void MakeAlpha(void);             // provides random point inside hypercubic
		void Grow(void);                  // Adds new cells to FOAM until buffer is full
		long PeekMax(void);               // choose one active cell, used by Grow and also in MC generation
		TFCELL* PeekRan(void);            // choose randomly one active cell, used only by Grow
		int  Divide(TFCELL *);            // Divide iCell into two daughters; iCell retained, taged as inactive
		void MakeActiveList(void);        // Creates table of active cells used by GenerCel2
		void GenerCell(TFCELL *&);        // Chose an active cell with probability ~ Primary integral
		void GenerCel2(TFCELL *&);        // Chose an active cell with probability ~ Primary integral
		// Generation
		void MakeEvent(void);           // Make one MC event
		void GetMCvect(double *);       // Provides random MC vector;
		void GetMCvectR(tReal *);       // Provides random MC vector;
		void GetMCwt(double &);         // Provides  obtained MC weight
		double GetMCwt(void);             // Provides  obtained MC weight
		double MCgenerate(tReal *MCvect);// All three above function in one
		// Finalization
		void GetIntegMC(double&, double&);// Provides Inegrand and abs. error from MC run
		void GetIntNorm(double&, double&);// Provides normalization Inegrand
		void GetWtParams(const double, double&, double&, double&);// Provides MC weight parameters
		void Finalize(  double&, double&);// Prints summary of MC integration
		TFOAM_INTEGRAND *GetRho(){return m_Rho;};    // Get pointer of the distribut. (after restoring from disk)
		CVMRnd *GetPseRan(){return m_PseRan;}; // Get pointer of r.n. generator (after restoring from disk)
		void SetRho(TFOAM_INTEGRAND *Rho){delete m_Rho;   m_Rho=Rho;};       // Set new integrand distr.
		void SetPseRan(CVMRnd  *PseRanRef){m_PseRan=PseRanRef;}; // Set new r.n. generator
		// Getters and Setters
		void SetnDim(int nDim){m_nDim = nDim;};         // Sets dimension of simplical subspace
		void SetkDim(int kDim){m_kDim = kDim;};         // Sets dimension of hyper-cubical subspace
		void SetnCells(long nCells){m_nCells =nCells;}; // Sets maximum number of cells
		void SetnSampl(long nSampl){m_nSampl =nSampl;}; // Sets no of MC events in cell exploration
		void SetnBin(int nBin){m_nBin = nBin;};         // Sets no of bins in histogs in cell exploration
		void SetChat(int Chat){m_Chat = Chat;};         // Sets option Chat, chat level
		void SetOptRej(int OptRej){m_OptRej =OptRej;};  // Sets option for MC rejection 
		void SetOptDrive(int OptDrive){m_OptDrive =OptDrive;}; // Sets option Drive, type of driver integrand
		void SetOptPeek( int OptPeek){ m_OptPeek  =OptPeek;};  // Sets option Peek, type of choice of cell in build-up
		void SetOptEdge( int OptEdge){ m_OptEdge  =OptEdge;};  // Sets option Edge, include or not vertices
		void SetOptOrd(  int OptOrd){  m_OptOrd   =OptOrd;};   // Sets option Ord, single root simplex or h-cub.
		void SetOptMCell(int OptMCell){m_OptMCell =OptMCell;}; // Sets MegaCell option, slim memory for h-cubes
		void SetOptVert(int OptVert){m_OptVert =OptVert;}      //Sets OptVert option
		void SetEvPerBin(int EvPerBin){m_EvPerBin =EvPerBin;}; // Sets max. no. of eff. events per bin in exploration
		void SetMaxWtRej(const double MaxWtRej){m_MaxWtRej=MaxWtRej;}; // Sets max. weight for rejection (wt=1 evts)
		void SetInhiDiv(int, int );          // Set inhibition of cell division along certain edge
		void SetXdivPRD(int, int, double[]); // Set predefined division points
		// Getters and Setters
		char *GetVersion(void ){return m_Version;};      // Get version of the foam
		int GetTotDim(void ){ return m_TotDim;};       // Get total dimension
		double GetPrimary(void ){return m_Prime;};        // Get value of primary integral
		void GetPrimary(double &Prime){Prime = m_Prime;}; // Get value of primary integral
		long GetnCalls(void){return m_nCalls;};           // Get total no of function calls
		long GetnEffev(void){return m_nEffev;};           // Get total no of effective wt=1 events
		// Debug
		void CheckAll(const int);   // Checks corectness of FOAM structure
		void PrintCells(void);      // Prints all cells
		void PrintVertices(void);   // Prints all Vertices
		void LaTexPlot2dim(char*);  // Generating Latex document drawing FOAM
		void RootPlot2dim(char*);   // Generating c++ code for drawing FOAM
		// Inline
	private:
		double sqr( const double x ){ return x*x;};
		double dmax(double x, double y) {  if(x>y) return x; else return y; }
		double dmin(double x, double y) {  if(x>y) return y; else return x; }
		long silnia(int n){ long s=n; for(int i=n-1; i>1; i--) s*=i; if(n==0) s=1; return s;} //factorial
		void StopM(char* message){ std::cout <<"++++ Exit in TFOAM: "<< message << std::endl; exit(0);}    //Error message
	};
	class TVMLIB_INTEGRAND: public TFOAM_INTEGRAND { // Collection of the testing functions
	private:
		CVMGenFunc* m_pFunc;
		int    m_n;
		tReal* m_VarMinRef;
		tReal* m_VarMaxRef;
		tReal m_LocalVars[CVMGen_MAXDIM];
	public:
		TVMLIB_INTEGRAND(int n, CVMGenFunc* func, tReal* _VarMinRef, tReal* _VarMaxRef);
		~TVMLIB_INTEGRAND();
		double Density(int, double *);
	};
public:
	TFOAM* m_pFoam;
	long m_nEvtsGen;
	TVMLIB_INTEGRAND* m_FoamFunc;
	bool m_bLog;
public:
	CVMGenFoam();
	virtual ~CVMGenFoam();
	virtual const char* ClassName();
	void SelfTest();
protected:
	virtual bool InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy=NULL);
public:
	virtual tReal Integrate(double fPrecision=0.01); // To be called after event generation
	virtual bool GenEventWgt();	//Generate weighted event
};
#endif // !defined(CVMGen_NO_FOAM)
#endif // !defined(CVMGen_H_INCLUDED_)

///////////////////////////////////// CVMGenTest
//	test class for generators
//
//	Note: 'CVMGenLite' is used no other generator is set
//
class CVMGenTest {
private:
	class CIntegrateCircleImpl : public CVMGen::CVMGenFunc {
	public:
		int m_nDim;
		CIntegrateCircleImpl(int nDim);
		virtual tReal Call(tReal* params);
		virtual CVMGen::CVMGenFunc* Clone() const;
	};
public:
	static bool IntegrateCircle(int nDim=2, tReal prec=0.01, CVMGen* gen=NULL);
};
