////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////

// VMHistLite.h: interface for the CVMHistLite class.
//
//////////////////////////////////////////////////////////////////////
//[not here!]#ifdef VM_HEP_LIB_FULL
#if !defined(CVMHistLite_H_INCLUDED_)
#define CVMHistLite_H_INCLUDED_

#define CVMHistLiteVER	1

/*
*	Pre-includes:
*	#include <iostream>
*	#include <fstream>
*	#include "VMOutput.h"
*/

class IVMPlotData2D {
public:
	static const int PL_HIST;
	static const int PL_SCAT;
	static const int PL_FREEDRAW;
protected:
	char* m_name;
public:
	IVMPlotData2D(const char* name);
	~IVMPlotData2D();
	virtual int GetClassId()=0;
	virtual double GetMaxXValue()=0;
	virtual double GetMinXValue()=0;
	virtual double GetMaxYValue()=0;
	virtual double GetMinYValue()=0;
	virtual const char* GetName();
	virtual const char* GetLegendTxt();
	virtual bool IsEmpty()=0;
	static char* GetStrCopy(const char* s); // to allow stand-alone class usage & possibly filter symbols
	void SetName(const char* name);			// change plot name
};

#define Decl_CVMHistLite
class CVMHistLite : public IVMPlotData2D {
public:
	CVMHistLite();
	CVMHistLite(const char* name, unsigned nBins, double xMin, double xMax, bool bLogX=false);
	CVMHistLite(const char* name, unsigned nBins, double* xBinMargins); //size[xBinMargins]=nBins+1
	CVMHistLite(const CVMHistLite& hist);
	virtual ~CVMHistLite();
	virtual int GetClassId();
	void Init(const char* name, unsigned nBins, double xMin, double xMax, bool bLogX=false);
	//Note on Auto-X mode: histogram seems !inited unless all test events are filled
	void InitAutoX(const char* name, unsigned nBins, unsigned nTestEvts, bool bLogX=false);
	void SetLegendText(const char* txt);	// histogram name is used by default
	virtual void Reset();
private:
	int GetBinNoInternal(double value);		// iBin==0 is underflow, iBin==m_nBins+1 is overflow
public:
	void Fill(double val, double wgt=1., double wgtErr=0.);
	void FillMax(double val, double wgt); //replace previous weight with 'wgt' if newest is greather
	double GetValue(unsigned i, double* err=NULL);
	double GetValueOrOverflow(unsigned i);	// 0<=i<=(nDim+1), first,last-->outmargins
	double operator()(unsigned i);
	unsigned GetNBins();
	unsigned long GetNumEvents();
	unsigned long GetNumEventsBin(unsigned i);
	unsigned long GetNumOutmargins();
	virtual double GetMaxXValue();
	virtual double GetMinXValue();
	virtual double GetMaxYValue();
	virtual double GetMinYValue();
	double GetBinXMargin(unsigned i);		// 0<=i<=nBins
	double GetBinXLeft(unsigned i) { return GetBinXMargin(i); }		// 0<=i<nBins
	double GetBinXRight(unsigned i) { return GetBinXMargin(i+1); }	// 0<=i<nBins
	double GetBinXCenter(unsigned i);		// 0<=i<nBins
	double GetBinXWidth(unsigned i);		// 0<=i<nBins
	bool FitXRegion(double x);				// ((x>=xMin)&&(x<=xMax)
	//double GetSumValue() { return m_SumValue; }
	double GetIntegral();					//[excl. outmargins]
	virtual const char* GetLegendTxt();
	virtual bool IsEmpty();
	//Data access

	//Normalization operations
	/*
	*	Normalize: Set integral value (as global factor).
	*	Doesn't modify bins ==> may be called BEFORE filling.
	*	Normalize(-1) removes previously set value.
	*/
	void Normalize(double _Integral=1.); // value*=_NormValue/m_SumValue{excl.outmargins};
	/*
	*	NormalizeAdv: Scale to new integral value (scaling all bins).
	*	Modifies bins ==> may be called AFTER filling only.
	*/
	void NormalizeAdv(double _Integral=1.); //to df/dx:  BinValue*=norm/NEvents{including outmargins}/dx
	/*
	*	NormalizeToBinWidth: Convert bin values to 'df/dx'. Integral 'f'=Sum{event weights filled}.
	*	May not be followed by filling, averaging or re-normalization.
	*/
	void NormalizeToBinWidth(); //to df/dx: BinValue/=dx
	/*
	*	NormalizeByDiffBins: Restore integral value after 'df/dx' is filled in every bin (1 event/bin).
	*/
	void NormalizeByDiffBins();
	/*
	*	Factorize all values in histogram by f (multiply previously filled weights: 'wgt'->'wgt*f').
	*/
	void Factor(double f);
	void AverageBinValues(); //Unsafe! BinValue=BinValue/NEventsInBin, m_yMaxValue updated but Sum/Outmargins are retained

	//Histogram operations
	void AddHisto(CVMHistLite* h, double hFactor=1.); //hFactor==-1 ==> 'minus h'
	void DivideHisto(CVMHistLite* h);

	//Save
	bool SaveAsTable(const char* fn, bool bAddColsNumber=false);
	bool SaveAsObject(CVMFileOut* f);
	bool LoadAsObject(CVMFileIn* f);
private:
	//TODO: keep outmargins separatelly (currently kept in m_values/m_nBinEvents)
	bool m_Ready; // used to track empty constructor usage + Init()
	unsigned m_nBins;
	char* m_txtLegend;
	double* m_values;
	double* m_valsSqr;
	double* m_valSdSqr;
	double* m_margins;
	double m_SumValue;			//Integral, Sum of weights filled (excluding outmargins)
	double m_SumValueAll;		//Integral, Sum of weights filled (including outmargins)
	double m_NormalizeIntegral;	//<0 is not normalized with global factor (excl. outmargins!)
	double m_yMaxValue;
	unsigned long* m_nBinEvents;
	unsigned long m_nEvents;	// including outmargins (!)
	//
	// Variables used in Auto-X mode only
	bool m_bAutoXMode;			// 'true' in test period only
	unsigned m_nAutoXTestSize;	// number of events used to estimate X-margins in AutoX-mode
	unsigned m_nAutoXTestEvt;	// number of last event used to estimate X-margins in AutoX-mode
	double* m_bufAutoXFills;	// buffer of Fill() values, size==m_nAutoXTestEvts*3
	bool m_bLogXAfterAutoMode;
};

//defined in .cpp: #define VMScat_MAXEVTS	10000
class CVMScatLite : public IVMPlotData2D {
public:
	CVMScatLite();
	CVMScatLite(const char* name, double xMin, double xMax, double yMin, double yMax, bool bAutoAverage=false);
	CVMScatLite(const char* name, bool bAutoAverage=false);
	virtual ~CVMScatLite();
	virtual int GetClassId();
	void Fill(double x, double y, double wgt=1.);
	bool GetValue(unsigned long i, double *x, double *y, double *wgt);
	void SetAutoAverage(bool bAverage, int nx=30, int ny=30);
	unsigned long GetNumEvents();
	virtual double GetMaxXValue();
	virtual double GetMinXValue();
	virtual double GetMaxYValue();
	virtual double GetMinYValue();
	double GetMaxWgt();
	double GetMinWgt();
	virtual bool IsEmpty();
	bool IsOverfilled();
	void Average(unsigned nx=30, unsigned ny=30); // leave nx*ny points to reduce output size
private:
	unsigned long m_nEvents;
	double* m_valX;
	double* m_valY;
	double* m_valWgt;
	bool m_bExpandMargins;
	double m_xMin;
	double m_xMax;
	double m_yMin;
	double m_yMax;
	double m_wgtMin;
	double m_wgtMax;
	bool m_bOverfilled;
	bool m_bAutoAverage;
	int m_iAutoAverageNx;
	int m_iAutoAverageNy;
};

//defined in .cpp: #define CVMFreeDr_MAXEVTS	20000
class CVMFreeDrawing : public IVMPlotData2D {
public:
	CVMFreeDrawing(const char* name);
	virtual ~CVMFreeDrawing();
	virtual int GetClassId();
	void MoveTo(double x, double y);
	void LineTo(double x, double y);
	void AddAction(int type, double x, double y); //1(moveto),2(lineto)
	int GetAction(unsigned long i, double *x, double *y); //1(moveto),2(lineto)
	unsigned long GetNumEvents();
	virtual double GetMaxXValue();
	virtual double GetMinXValue();
	virtual double GetMaxYValue();
	virtual double GetMinYValue();
	virtual bool IsEmpty();
	bool IsOverfilled();
private:
	unsigned long m_nEvents;
	double* m_valX;
	double* m_valY;
	int* m_valType;
	double m_xMin;
	double m_xMax;
	double m_yMin;
	double m_yMax;
	bool m_bOverfilled;
};

#define VMHistLiteViewHMAX		100
#define VMHistLiteViewFileExt	"view"
class CVMHistLiteViewDefaults {
public:
	//[options copied to every instance of 'view']
	static bool m_bDrawAxisBox;			//def: true
	static bool m_bDrawLegend;			//def: true
	static bool m_bPlotBins;			//def: false
	static bool m_bDrawErrorMarks;		//def: false
	static bool m_bPrintNEvts;			//def: false
	static bool m_bPrintIntegral;		//def: false
	static bool m_bLegendOverlapped;	//def: false (draw legend in overlapped box)
	static bool m_bThickGridLines;		//def: true
	static bool m_bLablesAtCenter;		//def: false;

	//[options used without copying to view instance]
	static bool m_bSaveViewWithEPS;		//def: false (save draw 'CVMHistLiteView'-object together with SaveAsEPS() call)
};
class CVMHistLiteView {
	IVMPlotData2D *m_plots[VMHistLiteViewHMAX]; // volatile or persistent links
	bool m_bPlotIsExternal[VMHistLiteViewHMAX]; // ownership rules for plots
	int m_nPlots;

	// styles
	CVMRGB m_clrPlot[VMHistLiteViewHMAX];
	int m_styPlot[VMHistLiteViewHMAX]; // 0(line),1(dashed),2(dotted)
	float m_LineWidth[VMHistLiteViewHMAX];
	CVMRGB m_clrBcgr;
public:
	// axis settings
	bool m_bLogAxisY;
	bool m_bLogAxisX;
	bool m_bDrawVertGrid;
	bool m_bDrawHorGrid;
	bool m_bDrawAxisBox;
	bool m_bDrawLegend;
	bool m_bThickGridLines;
	bool m_bLablesAtCenter;
	/*
	*	m_iLegendPos: 1: RightTop, 2: RightBottom, 3: LeftTop, 4:LeftBottom
	*/
	int m_iLegendPos;
	bool m_bLegendOverlapped;		//draw legend in overlapped box
	bool m_bAxisXFactorInsideBox;	//def: false
	bool m_bAxisYFactorInsideBox;	//def: false
	void SetYMargins(double yMin, double yMax) { m_RequestedYMargins[0]=yMin; m_RequestedYMargins[1]=yMax; }
	void SetXMargins(double xMin, double xMax) { m_RequestedXMargins[0]=xMin; m_RequestedXMargins[1]=xMax; }
private:
	double m_RequestedYMargins[2];
	double m_RequestedXMargins[2];

public:
	// histogram-specific view options
	//[Old] bool m_bPlotBins; //use 'CVMHistLiteViewDefaults::m_bPlotBins', 'SetPlotBins()' or 'SetHistStyle()'!
	bool m_bDrawErrorMarks;
	bool m_bHideEmptyHist;
	bool m_bPrintNEvts;
	bool m_bPrintIntegral;

	//scatter-plot-specific view options
	bool m_bShiftMergedPoints; // to separate several scatter-plots

	// additional drawings
	double m_xLevelLineAt; // draw if != 0.
	double m_yLevelLineAt; // draw if != 0.

	CVMHistLiteView(IVMPlotData2D* plot=NULL);
	virtual ~CVMHistLiteView();
	void Clear();
	void AddPlotToView(IVMPlotData2D* plot, bool bKeepHistoInView=false);
	void RemovePlotFromView(int i);
	IVMPlotData2D* GetPlotRef(int i) { return m_plots[i]; }
	int GetNumPlots() { return m_nPlots; }

	static const int LN_DENSE_DASH_FLAG;	// to be combined with CVMOutput::LINE_XXX
	static const int LN_MASK_BASIC_STYLE;	// includes all CVMOutput::LINE_XXX but not local flags
	static const int LN_MASK_DASH_INFO;		// (for local use only) includes all dash info
	static const int LN_DRAW_BINS;
	void GetHistStyle(int i, CVMRGB* clr, int* iLineType, float* fLineWidth=NULL);
	void SetHistStyle(int i, CVMRGB clr, int iLineType, float fLineWidth=-1.f);
	void SetBlackAndWhiteStyle();
	bool GetPlotBinsFlag(int idHist);
	void SetPlotBinsFlag(int idHist, bool bPlotBins);
	void SetHistLineWidth(int i, float line_width);
	void SetPlotBins(bool bPlotBins);
	bool ContainBinPlots(); // if any plot is marked as 'bins-drawed'
	void SetLogX(bool bLog=true); // set automatically if bins are logarithmic
	void SetLogY(bool bLog=true);
	void SetXAxisLabel(const char* xLabel);
	void SetYAxisLabel(const char* yLabel);
	void SetPlotLabel(const char* sPlotLabel);

	bool SaveAsEPS(const char* fn);
	bool SaveAsNB(const char* fn);
	bool SaveAsObject(const char* fn);
	bool LoadAsObject(const char* fn);
private:
	int LoadAsObjectImpl(CVMFileIn& f);

private:
	double m_xAxis;
	double m_yAxis;
	double m_xPosMax;
	double m_yPosMax;
	double m_xMinVal;
	double m_xMaxVal;
	double m_yMinVal;
	double m_yMaxVal;
	char* m_sXAxisLabel;			//TODO: use hist-specific labels instead of this
	char* m_sYAxisLabel;			//TODO: use hist-specific labels instead of this
public:
	double m_fYLabelYShift;			//in fractions of letter height (for manual adjustment)
	double m_fXLabelYShift;			//in fractions of letter height (for manual adjustment)
private:
	char* m_sPlotLabel;				//like 'A/B' in the 'm_iLegendPos' corner
	int UtilGetPower(double val, double* mantissa=NULL); // format: n.mmm*10^pow, n[1,9]
	double GetRoundMark(double val_max, bool bAllow52, bool bGetUpper=false);
	long GetUpperRoundLong(long val, bool bOnly125=true); //6..9->10, (!bOnly125)11..14->15, 16..19->20, ...
	double GetTruncToStep(double val, double round_step); // abs(result)<=abs(val), step presumed to be round
	double GetXPos(double xVal);
	double GetYPos(double yVal);
	bool write_plot(CVMOutput* out);
	void write_plot_points(CVMOutput* out, CVMHistLite* hist, bool bPlotBins);
	void write_scat_points(CVMOutput* out, CVMScatLite* scat, int iOrder=0); // iOrder used for shift of view
	void write_fdraw_points(CVMOutput* out, CVMFreeDrawing* fdraw);
	double m_TmpMarks[500];
	double m_TmpMarkMantissa[500];
	int m_TmpMarkPows[500];
	int m_TmpMarkType[500];
	int m_TmpMarkLevel[500];
	bool calc_axis_log(double _MinVal, double _MaxVal, int nMarksMax,
		int nArrAlloc, int *_nMarks, double *_ArrMarks, double *_ArrMarkMantissa, int *_iArrMarkPow,
		int *_iArrMarkType, int *_iArrMarkLevel,
		double *_fComFactor, int *_iComFactorPow, bool *_bShowValuesAsExp);
	bool calc_axis_lin(double _MinVal, double _MaxVal, int nMarksMin,
		int nArrAlloc, int *_nMarks, double *_ArrMarks, double *_ArrMarkMantissa, int *_iArrMarkPow,
		int *_iArrMarkType, int *_iArrMarkLevel,
		double *_fComFactor, int *_iComFactorPow, bool *_bShowValuesAsExp);
	void write_axis(CVMOutput* out);
};

#endif // !defined(CVMHistLite_H_INCLUDED_)
//[not here!]#endif //#ifdef VM_HEP_LIB_FULL
