////////////////////////////////////////////////////////
//	This file is a part of ALHEP project
//	Author: V.Makarenko (makarenko@hep.by)
//	See details at http://www.hep.by/alhep
////////////////////////////////////////////////////////
// VMGen.cpp: implementation of the CVMGen class.
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include <iostream> // for stdlib,stdio ect
#include <fstream>
#include <iomanip>

#include "VMLib/include/vmlib.h"

/////////////////////////////////////////////////////////////////////// CVMRnd
CVMRnd::CVMRnd() { m_nSkip=0; }
tReal CVMRnd::Rnd() { if(m_nSkip>0){for(int i=0;i<m_nSkip;i++)RndImpl();} return RndImpl(); }
/*
*	used to generate '(numNo % itMask)==itNo' sub-set of numbers
*	Note: before the first number 'itNo+(itMask-1)' will be actually skipped!
*/
void CVMRnd::SetFilter(int itNo, int itMask) {
	if(itMask>1){ if(itNo>=itMask)itNo=itNo%itMask; for(int i=0;i<itNo;i++)RndImpl(); m_nSkip=itMask-1; }else{ m_nSkip=0; }
}
void CVMRnd::RndArr(long n, tReal* arr) { if(arr!=NULL){ for(long i=0;i<n;i++){arr[i]=Rnd();} } }
bool CVMRnd::IsReady() { return true; } //'true' by default
//
CVMRnd* CVMRnd::CreateDefaultRnd() { //return 'default' RND-generator instance
#if defined VM_USE_QD_REAL
#ifdef VMLIB_WIN
	return new CVMRndMZ(); //Note: 'qd_real::rand()' produces erroneous results under Windows ==> Use CVMRndMZ anyway.
#else
	return new CVMRndQD();
#endif //VMLIB_WIN
#elif defined VM_USE_DD_REAL
#ifdef VMLIB_WIN
	return new CVMRndMZ(); //Note: 'dd_real::rand()' produces erroneous results under Windows ==> Use CVMRndMZ anyway.
#else
	return new CVMRndQD();
#endif //VMLIB_WIN
#else
	return new CVMRndMZ();
#endif //VM_USE_QD_REAL,VM_USE_DD_REAL
}

//Global RND-object (to be used as 'the only' RND-object in project)
CVMRnd* CVMRnd::m_RndGlobal = NULL;
/*
*	get reference to 'global' RND-object
*/
CVMRnd* CVMRnd::GetRndRef() { if(m_RndGlobal==NULL)ResetRndGlobal(); return m_RndGlobal; }
/*
*	create 'global' RND-object
*	if(itMask>1)==> every '(n % itMask) == itNo' number is generated.
*/
void CVMRnd::ResetRndGlobal(int itNo, int itMask) {
	if(m_RndGlobal!=NULL){ ClearRndGlobal(); std::cout<<"WARNING: global RND-object is changed! The previous links (is exist) are erroneous now."<<std::endl; }
	m_RndGlobal=CreateDefaultRnd();
	if(m_RndGlobal==NULL){ std::cout<<"ERROR creating RND-object"<<std::endl; exit(0); /*No way to continue*/ }
	if(itMask>1){ m_RndGlobal->SetFilter(itNo,itMask); }
}
void CVMRnd::ClearRndGlobal() { if(m_RndGlobal!=NULL){ CVMRnd* ref=m_RndGlobal; m_RndGlobal=NULL; delete ref; } }

/////////////////////////////////////////////////////////////////////// CVMRndMZ
/*
*	Ref: 'TOWARD A UNVERSAL RANDOM NUMBER GENERATOR'/ G.MARSAGLIA, A.ZAMAN /FLORIDA STATE UNIV. PREPRINT FSU-SCRI-87-??
*	(Seen in Djangoh code by H.Spiesberger)
*	Req: 24 bits minimum for real value. Period: ~2**144
*/
CVMRndMZ::CVMRndMZ() : CVMRnd() { Init(12,34,56,78); }
CVMRndMZ::CVMRndMZ(long na1, long na2, long na3, long nb1) : CVMRnd() { Init(na1,na2,na3,nb1); }
/*
*	Initialization values
*	na1,na2,na3 must be in 1..178 and not all ==1 (standard values: 12,34,56)
*	nb1 must be in 1..168 (standard values: 78)
*/
tReal CVMRndMZ::RndImpl() {
	double res=m_U[m_i]-m_U[m_j]; if(res<0)res+=1.;
	m_U[m_i]=res; m_i--;if(m_i<0)m_i=96; m_j--;if(m_j<0)m_j=96; m_C-=m_CD;if(m_C<0.)m_C+=m_CM;
	res-=m_C; if(res<0)res+=1.;
	return (tReal)res;
}
void CVMRndMZ::Init(long na1,long na2,long na3,long nb1) {
	if ((na1<1)||(na1>178)||(na2<1)||(na2>178)||(na3<1)||(na3>178)||(nb1<1)||(nb1>168)||((na1==1)&&(na2==1)&&(na3==1))) {
		na1=12; na2=34; na3=56; nb1=78; // reset defaults
	}
	long ma1=na1; long ma2=na2; long ma3=na3; long mb1=nb1; m_i=96; m_j=32;
	for(int ii2=0; ii2<97; ii2++) {
		double s=0.; double t=0.5;
		for(int ii1=1; ii1<=24; ii1++) {
			long mat=(((ma1*ma2)%179)*ma3)%179; ma1=ma2; ma2=ma3; ma3=mat; mb1=(53*mb1+1)%169;
			if(((mb1*mat)%64)>=32) s+=t; // same to: if((mb1*mat)&0x20) s+=t;
			t*=0.5;
		}
		m_U[ii2]=s;
	}
	m_C = 362436.0/16777216.0; m_CD = 7654321.0/16777216.0; m_CM = 16777213.0/16777216.0;
}
void CVMRndMZ::GetSeed(double (*u)[97], double* c, double* cd, double* cm, int* i, int* j) {
	if(u!=NULL){ for(int k=0; k<97; k++){(*u)[k]=m_U[k];} }
	if(c!=NULL)*c=m_C; if(cd!=NULL)*cd=m_CD; if(cm!=NULL)*cm=m_CM; if(i!=NULL)*i=m_i; if(j!=NULL)*j=m_j;
}
void CVMRndMZ::SetSeed(double (*u)[97], double c, double cd, double cm, int i, int j) {
	if (u!=NULL) { for(int k=0; k<97; k++){m_U[k]=(*u)[k];} m_C=c; m_CD=cd; m_CM=cm; m_i=i; m_j=j; }
}
void CVMRndMZ::Test() { // Test function from Djangoh code by H.Spiesberger
	double u[97],c,cd,cm; int i,j; // to keep seed
	double v[6]={6533892.0,14220222.0,7275067.0,6172232.0,8354498.0,10633180.0}; double x[6],d[6];
	GetSeed(&u,&c,&cd,&cm,&i,&j);
	Init(12,34,56,78);
	int k; for(k=0;k<20000;k++) { Rnd(); }
	double sd=0.; for(k=0;k<6;k++){ x[k]=4096.*(4096.*DBLtR(Rnd())); d[k]=x[k]-v[k]; sd+=d[k]; }
	SetSeed(&u,c,cd,cm,i,j);
	//if (sd!=0.) {
		std::cout << "=== Test of RND-generator ===" << std::endl;
		std::cout << "expected   calculated   difference" << std::endl;
		for(k=0; k<6; k++) { std::cout<<v[k]<<'\t'<<x[k]<<'\t'<<d[k]<<std::endl; }
	//}
}

/////////////////////////////////////////////////////////////////////// CVMRndQD
//Uses RND-procedures from QD package
CVMRndQD::CVMRndQD() : CVMRnd() {}
tReal CVMRndQD::RndImpl() {
#if defined VM_USE_QD_REAL
	return qd_real::rand();
#elif defined VM_USE_DD_REAL
	return dd_real::rand();
#else
	return 0.5; //Error
#endif //VM_USE_QD_REAL,VM_USE_DD_REAL
}
bool CVMRndQD::IsReady() {
#if defined VM_USE_QD_REAL
	return true;
#elif defined VM_USE_DD_REAL
	return true;
#else
	return false;
#endif //VM_USE_QD_REAL,VM_USE_DD_REAL
}

//
#define MC_CL_fact_99	2.5758294
#define MC_CL_fact_999	3.290527
#define MC_CL_fact_used	MC_CL_fact_999
//
/////////////////////////////////////////////////////////////////////// CVMGen
CVMGen::CVMGenFuncImpl::CVMGenFuncImpl(tReal(vmdecl *func)(tReal*)) : CVMGenFunc() { m_pFunc=func; }
tReal CVMGen::CVMGenFuncImpl::Call(tReal* params) { return (*m_pFunc)(params); }
CVMGen::CVMGenFunc* CVMGen::CVMGenFuncImpl::Clone() const { return new CVMGenFuncImpl(m_pFunc); }
CVMGen::CVMGen() {
	m_nDim=-1; //not-inited flag
	int i;
	for(i=0; i<CVMGen_MAXDIM; i++) { m_GenValues[i]=0.; m_VarMin[i]=0.; m_VarMax[i]=1.; }
	m_Weight=0.; m_WeightDebug=0.; m_WgtError=0.; m_Integral=0.; m_Error=0.; m_pFunc=NULL; m_bFuncOwner=false; m_nFuncCalls=0; m_rnd=NULL; m_bRndIsExternal=true; m_Volume=-1.;
	m_Metadata=NULL; m_Title=NULL;
	for(i=0; i<CVMGen_MAXUSERVARS; i++) { m_UserVarsNames[i]=NULL; m_UserVarsVals[i]=0.; }
}
CVMGen::~CVMGen() {
	if(m_pFunc!=NULL) { if(m_bFuncOwner){ delete m_pFunc; } m_pFunc=NULL; }
	if(m_Metadata!=NULL) { delete[] m_Metadata; m_Metadata=NULL; }
	if(m_Title!=NULL) { delete[] m_Title; m_Title=NULL; }
	if(m_rnd!=NULL) { if(!m_bRndIsExternal)delete m_rnd; m_rnd=NULL; m_bRndIsExternal=true; }
	for(int i=0;i<CVMGen_MAXUSERVARS;i++){ if(m_UserVarsNames[i]!=NULL){ delete[] m_UserVarsNames[i]; } }
}
bool CVMGen::Init(int nDim, tReal* _VarMin, tReal* _VarMax, tReal(vmdecl *func)(tReal*), CVMRnd* rndCpy) {
	if(func==NULL)return false;
	CVMGenFunc* f=new CVMGenFuncImpl(func);
	return InitImpl(nDim,_VarMin,_VarMax,f,true,rndCpy);
}
bool CVMGen::Init(int nDim, tReal* _VarMin, tReal* _VarMax, const CVMGenFunc* func, CVMRnd* rndCpy) {
	if(func==NULL)return false;
	CVMGenFunc* f=((func!=NULL)?func->Clone():NULL);
	return InitImpl(nDim,_VarMin,_VarMax,f,true,rndCpy);
}
bool CVMGen::Init_ExternalFunc(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* funcExternal, CVMRnd* rndCpy) {
	if(funcExternal==NULL)return false;
	return InitImpl(nDim,_VarMin,_VarMax,funcExternal,false,rndCpy);
}
void CVMGen::ReleaseExtFunction() { //used to avoid further use of 'external' function object
	if ((m_pFunc!=NULL) && (!m_bFuncOwner)) { m_pFunc=NULL; }
	//Note: 'no-more-use' is enshured by 'm_pFunc==NULL' checks
}
bool CVMGen::InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy) {
	if((m_nDim>=0) || (m_pFunc!=NULL)) { // check not-inited flag
		std::cout<<"CVMGen::Init() ERROR: Trying to re-init generator"<<std::endl; return false;
	}
	if ((nDim<1) || (nDim>CVMGen_MAXDIM) || (_VarMin==NULL) || (_VarMax==NULL) || (func==NULL)) return false;
	m_Volume=1.;
	for(int i=0; i<nDim; i++) {
		m_VarMin[i]=_VarMin[i]; m_VarMax[i]=_VarMax[i]; if(m_VarMin[i]>=m_VarMax[i]) return false;
		m_Volume*=(m_VarMax[i]-m_VarMin[i]);
	}
	if(m_Volume<=0.) { std::cout<<"CVMGen::Init() ERROR: Invalid integration volume"<<std::endl; return false; }
	m_nDim=nDim; // fill if init-OK
	m_pFunc=func; m_bFuncOwner=bFuncOwner;
	m_nFuncCalls=0;
	m_Integral=0.;
	if(rndCpy==NULL){
		//Use 'global' RND-object (or previously defined)
		if(m_rnd==NULL){ m_rnd=CVMRnd::GetRndRef(); m_bRndIsExternal=true; }
	}else if(m_rnd!=rndCpy) {
		if(m_rnd!=NULL){ if(!m_bRndIsExternal)delete m_rnd; m_rnd=NULL; m_bRndIsExternal=true; }
		m_rnd=rndCpy; m_bRndIsExternal=false; //'rndCpy' will be deleted
	}
	return true;
}
void CVMGen::SetExpectedIntegral(tReal valExpected) {}
const char* CVMGen::ClassName() { return "Abstract CVMGen"; }
bool CVMGen::GenEvent() { return false; } //If unweighted events are not implemented
double CVMGen::GetGenEventEfficiency() { return 0.; } //Efficiency of previous 'GenEvent' calls
CVMRnd* CVMGen::GetRndRef() { return m_rnd; }
tReal CVMGen::CallFunction(tReal* params) {
	if(m_nFuncCalls<LONG_MAX)m_nFuncCalls++;
	return ((m_pFunc!=NULL)?m_pFunc->Call(params):0.);
}
#define GEN_META_MAX_SIZE	100000
void CVMGen::SetMetadata(const char* info) {
	if(m_Metadata!=NULL) { delete[] m_Metadata; m_Metadata=NULL; }
	if(info!=NULL){ //TODO: use 'memcpy/memmove(trg, s, (nSymsToCpy)*sizeof(char))' from '#include <memory>'
		long l=0; for(;l<GEN_META_MAX_SIZE;l++){ if(*(info+l)=='\0')break; }
		m_Metadata=new char[l+2]; for(long i=0;i<l;i++){ *(m_Metadata+i)=*(info+i); } *(m_Metadata+l)='\0';
	}
}
void CVMGen::SetTitle(const char* title) {
	if(m_Title!=NULL) { delete[] m_Title; m_Title=NULL; }
	if(title!=NULL){ //TODO: use 'memcpy/memmove(trg, s, (nSymsToCpy)*sizeof(char))' from '#include <memory>'
		long l=0; for(;l<100;l++){ if(*(title+l)=='\0')break; }
		m_Title=new char[l+2]; for(long i=0;i<l;i++){ *(m_Title+i)=*(title+i); } *(m_Title+l)='\0';
	}
}
const char* CVMGen::GetMetadata() { return m_Metadata; }
void CVMGen::AddUserVariable(const char* var, tReal val) {
	if(var==NULL)return;
	bool bAddOK=false;
	for(int i=0; i<CVMGen_MAXUSERVARS; i++) {
		if(m_UserVarsNames[i]!=NULL){
			if(CVMUtil::strEqu(m_UserVarsNames[i],var)) {
				if(m_UserVarsVals[i] != val){
					std::cout<<"[WARNING in CVMGen::AddUserVariable]: User variable ("<<var<<") updated (Prev: "<<m_UserVarsVals[i]<<", new: "<<val<<")"<<std::endl;
					m_UserVarsVals[i]=val;
				}
				bAddOK=true; break;
			}
		}else{
			m_UserVarsNames[i]=CVMUtil::strCpy(var); m_UserVarsVals[i]=val; bAddOK=true; break;
		}
	}
	if(!bAddOK){ std::cout<<"[ERROR in CVMGen::AddUserVariable]: Can't save user variable. Limit exceeded"<<std::endl; }
}
bool CVMGen::GetUserVariable(const char* var, tReal* val) {
	if(var==NULL)return false;
	bool bRes=false;
	for(int i=0; i<CVMGen_MAXUSERVARS; i++) {
		if(m_UserVarsNames[i]!=NULL){
			if(CVMUtil::strEqu(m_UserVarsNames[i],var)) { if(val!=NULL){ *val=m_UserVarsVals[i]; } bRes=true; break; }
		}else{ break; }
	}
	return bRes;
}
int CVMGen::GetUserVariablesNum() {
	int n=0; for(int i=0;i<CVMGen_MAXUSERVARS;i++){ if(m_UserVarsNames[i]!=NULL){ n++; }else{ break; } } return n;
}

/////////////////////////////////////////////////////////////////////// CVMGenPlain
CVMGenPlain::CVMGenPlain() : CVMGen() {}
void CVMGenPlain::SetExpectedIntegral(tReal valExpected) {}
tReal CVMGenPlain::Integrate(double fPrecision) { // simple integration without correct error estimation
	if ((m_nDim<=0) || (m_pFunc==NULL)) return m_Integral; // not inited
	if ((fPrecision<=0) || (fPrecision>1)) fPrecision=0.01; //default

	m_nFuncCalls=0; //internal init
	long NMin=10000; long NMax=100000000; // stop & not-stop parameters
	long N=0; tReal fSum=0.; tReal fSqrSum=0.;
	for(;N<NMax;){
		if(!GenEventWgt()){ break; }//error
		N++;
		fSum+=m_Weight; fSqrSum+=m_Weight*m_Weight;
		if(N>NMin){
			//sd2:
			tReal fRes=fSum/N; tReal fRes2=fRes*fRes;
			tReal fErr2=(fSqrSum/N-fRes2)/N; //TODO: insert Student's factor here!
			if(fabs(fRes2*fPrecision*fPrecision) < fabs(fErr2)) {
				break;
			}
		}
	}
	if(N>0){
		m_Integral=fSum/N;
		m_Error=sqrt(fabs((fSqrSum/N-m_Integral*m_Integral)/N)); //TODO: insert Student's factor here!
	}else{
		m_Integral=-1.;
		m_Error=-1.;
	}
	return m_Integral;
}
bool CVMGenPlain::GenEventWgt() {
	if (m_nDim<=0) return false;
	for(int i=0; i<m_nDim; i++) { m_GenValues[i]=m_VarMin[i]+(m_VarMax[i]-m_VarMin[i])*m_rnd->Rnd(); }
	m_Weight=CallFunction(m_GenValues)*m_Volume; m_WeightDebug=m_Weight;
	return true;
}

/////////////////////////////////////////////////////////////////////// CVMGenLite
#define CVMGenLite_MAXSUBEVTS	10000
CVMGenLite::CVMGenLiteCell::CVMGenLiteCell(CVMGenLite* _Gen, int iSubOrder) {
	m_Gen=_Gen; m_nSubOrder=iSubOrder;
	if(m_Gen!=NULL)m_Gen->m_nCellsCnt++;
	//if(m_Gen->m_bLogCellsCnt && ((m_Gen->m_nCellsCnt % 10000)==0)) { std::cout<<"Cell #"<<m_Gen->m_nCellsCnt<<" at level "<<m_nSubOrder<<std::endl; }
	//if(m_Gen->m_bLogCellsCnt && ((m_Gen->m_nCellsCnt % 100)==0)) { std::cout<<"Cell #"<<m_Gen->m_nCellsCnt<<" at level "<<m_nSubOrder<<std::endl; }
	for(int i=0; i<CVMGen_MAXDIM; i++){ m_VarMin[i]=0.; m_VarMax[i]=0.; m_nDimOrder[i]=0; } m_Volume=-1;
	if(m_Gen->m_nSOrderMaxActual<m_nSubOrder)m_Gen->m_nSOrderMaxActual=m_nSubOrder;
	m_Integral=0.;
	m_nEvts=0;
	m_FuncMax=0.;
	m_Variance=-1.;
	m_fWghtExpected=-1.;
	m_Sub1=NULL; m_Sub2=NULL; m_iSplitAxis=-1;
	m_iSplitAxisPrev=-1; m_iSplitAxisPrevTimes=0;
	m_bIntegrationSkipped=false; //set to 'true' if integration is skipped & pre-defined value left
}
CVMGenLite::CVMGenLiteCell::~CVMGenLiteCell() {
	if(m_Sub1!=NULL){ delete m_Sub1; m_Sub1=NULL; }
	if(m_Sub2!=NULL){ delete m_Sub2; m_Sub2=NULL; }
	if(m_Gen!=NULL)m_Gen->m_nCellsCnt--;
}
/*
*	Cell::IsPrecisionOK(): check if in-cell-error is enough to satisfy the required precision.
*	Note: 'm_Integral' and 'm_Variance' may be:
*		- just integrated,
*		- assigned by parent cell,
*		- integrated at previous iteration.
*/
bool CVMGenLite::CVMGenLiteCell::IsPrecisionOK() { //Precision check
	bool bPrecOK=false;
	bool bPrecOKStrict=false;

	tReal _PrecReq=m_Gen->m_fPrecisionReq; //TODO: keep m_fPrecisionReq^2 for faster check?
	tReal _fCLfactSqr=m_Gen->m_fCLFactor*m_Gen->m_fCLFactor;

	tReal _IExpected=m_Gen->m_fIntegralExpected;
	if(_IExpected<=0.)_IExpected=m_Gen->m_Integral;
	if((_IExpected<=0.) && (m_fWghtExpected>0.))_IExpected=m_Integral/m_fWghtExpected;
	if(_IExpected<=0.){
		std::cout<<"[GenLite][Internal ERROR in CVMGenLiteCell::IsPrecisionOK] Can't estimate expected integral value."<<std::endl; bPrecOK=false; exit(0); /*Error*/
	}

	//Mode: iPrecEstimateMode
	//Mode: bAllowStop
	//Nt: _IExpected is >0. below

	int iPrecEstimateMode=3;	//(def: 2) 1:(equal error per volume (old 'bEquVol')), 2:(equal relarive error)
	bool bAllowStop=false;		//asdfg
	double fErrRestrictSqr=DBLtR(m_Gen->m_fCellErrMaxSqr); //=-1.; //=0.25;
	switch(iPrecEstimateMode){
	case 1: {
		/*
		*	phi_i=Vol_i/Vol
		*/
		tReal vol_i=m_Volume/m_Gen->m_Volume;
		tReal errMax=_PrecReq*_IExpected; if(m_Variance*_fCLfactSqr<=vol_i*errMax*errMax){ bPrecOK=true; bPrecOKStrict=true; }

		//Step-function stop check
		if(!bPrecOK && bAllowStop) {
			tReal safeFactor=m_Volume/m_Gen->m_Volume; /*=0.1;*/ if (m_Integral<_PrecReq*safeFactor*_IExpected)bPrecOK=true;
		}
			} break;
	case 2: {
		/*
		*	phi_i=wght_i
		*/
		if(m_Variance*_fCLfactSqr<=m_Integral*_IExpected*_PrecReq*_PrecReq){ bPrecOK=true; bPrecOKStrict=true; }

		//Step-function stop check
		if(!bPrecOK && bAllowStop) {
			tReal safeFactor=m_Volume/m_Gen->m_Volume; /*=0.1;*/ if (m_Integral<_PrecReq*safeFactor*_IExpected)bPrecOK=true;
		}
			} break;
	case 3: {
		/*
		*	phi_i=wght_i + forces
		*/
		if(m_Variance*_fCLfactSqr<=m_Integral*_IExpected*_PrecReq*_PrecReq){ bPrecOK=true; bPrecOKStrict=true; }

		//Force split
		if(bPrecOK && (m_Gen->m_fMaxWgtFactor>0.) && (m_Volume>0.) && (m_FuncMax>0.)){
			tReal fAve2Max=(m_Integral/m_Volume)/m_FuncMax;
			//tReal fAve2MaxMax=0.02; /*Parameter*/
			//Note: fAve2MaxMax=0.05; --> divergence is possible!
			//tReal fAve2MaxMax=0.01; /*Parameter*/
			if(fAve2Max<(1./m_Gen->m_fMaxWgtFactor)){ bPrecOK=false; bPrecOKStrict=false; }
		}

		//Error-in-cell restriction
		if((fErrRestrictSqr>0.) && bPrecOK && (m_Integral>0.) &&
			(m_Variance*_fCLfactSqr >= fErrRestrictSqr*m_Integral*m_Integral)) {
				bPrecOK=false; bPrecOKStrict=false;
		}

		//Step-function stop check
		if(!bPrecOK && bAllowStop) {
		//if(!bPrecOK) {
			tReal safeFactor=m_Volume/m_Gen->m_Volume; /*=0.1;*/ if (m_Integral<_PrecReq*safeFactor*_IExpected)bPrecOK=true;
		}
			} break;
	}
	

	//Old checks...
	//Old check impossible here: re-integrate cell if adjacent cell it too different
	//tReal parFMaxRatio2 = 3.; //parameter
	//if(bPrecOK && (loc->_FMax[m_iSplitAxis][1]>0.) && (loc->_FMax[m_iSplitAxis][0]>parFMaxRatio2*loc->_FMax[m_iSplitAxis][1])) bPrecOK=false;

	// Force repeat if FMax>>FAve
	//if(bPrecOKStrict){ //==>to 150+ orders
	//if(bPrecOK && !bPrecOKStrict){ //+20% cells, no integral changes
	//	if(m_FuncMax>200.*m_Integral/m_Volume)bPrecOK=false;
	//}

	//if (m_fMaxWgtFactor>0.) { //Force repeat if FMax>>FMin
	//	//TODO: fix here!
	//	//tReal _WgtMaxFactor=m_Gen->m_fMaxWgtFactor;
	//	//Old (erroneous at cut-edges)
	//	//if ((bEmptySub1) && (loc->_FMax[m_iSplitAxis][0]>_WgtMaxFactor*loc->_FSum[m_iSplitAxis][0]/loc->_nEv[m_iSplitAxis][0]))bEmptySub1=false;
	//	//New --> keep '_FMinGt0' (min but > 0)
	//	//if ((bPrecOK) && (_FMinGt0>0.) && (m_FuncMax>_WgtMaxFactor*_FMinGt0))bPrecOK=false;
	//}
	return bPrecOK;
}
bool CVMGenLite::CVMGenLiteCell::Integrate(long nCashedCalls, tReal** arrCashedData) {
	//
	//	Notes.
	//	1. If cell is already splitted ==> intgrate sub-cells only.
	//	   This mode is used to 'continue integration' with higher precision.
	//	2. The three conditions MUST always be equal: (m_iSplitAxis>=0)==(m_Sub1!=NULL)==(m_Sub2!=NULL)
	//

	if(m_iSplitAxis>=0) {
		//Continue/Update integration
		if((m_Sub1==NULL) || (m_Sub2==NULL)) { std::cout<<"[GenLite][ERROR] Unexpected NULL sub-cell. Integration HALTED."<<std::endl; exit(0); }

		tReal subVal1=m_Sub1->m_Integral; tReal subVar1=m_Sub1->m_Variance; //previous values
		if(!m_Sub1->Integrate(0,NULL)){
			m_Gen->m_nCellIntegrFault++; m_Sub1->m_Integral=subVal1; m_Sub1->m_Variance=subVar1; //restore previous
		}else{ if(m_FuncMax<m_Sub1->m_FuncMax)m_FuncMax=m_Sub1->m_FuncMax; }

		tReal subVal2=m_Sub2->m_Integral; tReal subVar2=m_Sub2->m_Variance; //previous values
		if(!m_Sub2->Integrate(0,NULL)){
			m_Gen->m_nCellIntegrFault++; m_Sub2->m_Integral=subVal2; m_Sub2->m_Variance=subVar2; //restore previous
		}else{ if(m_FuncMax<m_Sub2->m_FuncMax)m_FuncMax=m_Sub2->m_FuncMax; }

		m_Integral = m_Sub1->m_Integral + m_Sub2->m_Integral;
		m_Variance = m_Sub1->m_Variance + m_Sub2->m_Variance; //s^2/N = s1^2/N1 + s2^2/N2
		return true;
	}
	if((m_Sub1!=NULL) || (m_Sub2!=NULL)) {
		std::cout<<"[GenLite][ERROR] Existing sub-cells found with no splitting information."<<std::endl;
		if(m_Sub1!=NULL){delete m_Sub1;m_Sub1=NULL;} if(m_Sub2!=NULL){delete m_Sub2;m_Sub2=NULL;}
	}

	//
	long i,j,n;
	long nDim=m_Gen->m_nDim; if(nDim<=0){ std::cout<<"ERROR: Invalid dimension"<<std::endl; return false; }

	//'Before-any-check' initialization (spare check! done in constructors)
	if (m_Volume<0.) { m_Volume=1.; for(i=0;i<nDim;i++)m_Volume*=(m_VarMax[i]-m_VarMin[i]); }

	// Continue integration or leave previous value?
	// Note: 'm_Integral' and 'm_Variance' may be:
	//		- assigned by parent cell,
	//		- integrated at previous iteration.
	bool bNoRepeat=false; bool bCheckFixed=false;
	if (m_nSubOrder <= (m_Gen->m_nSubOrderMin)) { bNoRepeat=false; bCheckFixed=true; } //Cell order check
	if (m_nSubOrder >= (m_Gen->m_nSubOrderMax-1)) { bNoRepeat=true; bCheckFixed=true; } //Cell order check
	if (m_Gen->m_bUsePreviousGrid) { bNoRepeat=false; bCheckFixed=true; } //The cell is empty but must be repeated
	if(!bCheckFixed){
		bNoRepeat=IsPrecisionOK();	//Precision check
	}
	if(bNoRepeat){ m_bIntegrationSkipped=true; return true; } m_bIntegrationSkipped=false;

	// Something like scroll (Nt: m_nCellsCnt here is 1,3,5...999,1001...)
#ifdef VM_DEBUG_FULL
	if(m_Gen->m_bLogCellsCnt) { std::cout<<"Cells #"<<m_Gen->m_nCellsCnt<<". Integration at level "<<m_nSubOrder<<std::endl; }
	CAlHepLib::ViewStackUsed();
#else
	if(m_Gen->m_bLogCellsCnt && ((m_Gen->m_nCellsCnt % 1000)==1)) { std::cout<<"Cells #"<<m_Gen->m_nCellsCnt<<". Integration at level "<<m_nSubOrder<<std::endl; }
	//std::cout<<"Cells #"<<m_Gen->m_nCellsCnt<<". Integration at level "<<m_nSubOrder<<std::endl;
#endif //VM_DEBUG_FULL

	// Local variables & self-checks
	// Warning: don't use too many local variables here --> stack overflow ptoblem is possible
	if(m_nSubOrder>=m_Gen->m_nSubOrderMax) { //Cache allocation fails elsewhere
		std::cout<<"ERROR: Invalid cell order="<<m_nSubOrder<<std::endl; exit(0);
		return false;
	}
	// nEvt
	m_nEvts=m_Gen->m_SubEvts; if((m_Gen->m_nFirstOrders>=0)&&(m_nSubOrder<=m_Gen->m_nFirstOrders))m_nEvts=m_Gen->m_SubEvtsFirstOrders;
	if(m_nEvts>CVMGenLite_MAXSUBEVTS)m_nEvts=CVMGenLite_MAXSUBEVTS;
	//
	bool bResOK=true;
	bool bUseCache=true; tReal** _CacheAll=NULL; tReal** _CachePart=NULL; int** _CacheOrd=NULL;
	bool bLogCell=false; //Debug
	long dbgCellID=m_Gen->m_nCellsCnt;//TODO-NODEBUG
	CLocals* loc=&m_Gen->m_LocalsCache[m_nSubOrder]; //locals
	//Nt: SOME variables in 'loc' MAY be used as 'm_LocalsCache[0]' - they are NOT used after recursive Integrate() calls.
	// ==> they may be separated to something like 'CLocalsOnes' struct (save "(N-1)*sizeof(CLocals)" bytes)
	//But SOME variables we must use as 'm_LocalsCache[LEVEL]' - they are used after sub-integrate calls.
	//[Letest moves to 'loc':]tReal _FSumCell,_FSqrCell,varMidSel,selVal1,selVal2,selVariance1,selVariance2,_SubVol,_FMax1,_FMax2;// 0.3kb at qd-precision
	//Also add to 'CLocals': percent of zero events, minimal F-value

	{ //Separate local variables (TODO: check why inner-variables are NOT released after '}' -- before recursive call?)
		CVMRnd* _Rnd=m_Gen->GetRndRef();

		// Initialize & checks
		if (m_Volume<=0.) { m_Integral=0.; m_Variance=0.; std::cout<<"ERROR: Empty volume"<<std::endl; return true; }
		for(i=0; i<nDim; i++){
			loc->varMid[i]=(m_VarMin[i]+m_VarMax[i])/2.;
			for(j=0;j<2;j++){loc->_FSum[i][j]=0.;loc->_FSqr[i][j]=0.;loc->_nEv[i][j]=0;loc->_FMax[i][j]=0.;loc->_FMinGt0[i][j]=0.;}
			for(j=0;j<10;j++){loc->fAveBin[i][j]=0.;loc->nAveBin[i][j]=0;}
		}

		// Machine number precision test
		for(i=0; i<nDim; i++) {
			if((loc->varMid[i]==m_VarMin[i])||(loc->varMid[i]==m_VarMax[i])) { m_Integral=0.; m_Variance=0.; m_Gen->m_nMachinePrecFault++; return false; }
		}

		// Warning: Do NOT return after next lines!
		// Cache accessing or allocation
		//Note: _CacheAll, _CachePart are allocated to [nDim+1][m_nEvts]; _CacheOrd to [nDim][m_nEvts];
		if(bUseCache){ _CacheAll=m_Gen->_Cache1[m_nSubOrder]; _CachePart=m_Gen->_Cache2[m_nSubOrder]; _CacheOrd=m_Gen->_iCache1[m_nSubOrder]; }
		if ((_CacheAll==NULL) || (_CachePart==NULL) || (_CacheOrd==NULL)) {
			if(bUseCache){ std::cout<<"WARNING: No cache pre-allocated for order "<<m_nSubOrder<<std::endl; bUseCache=false; }
			// [Allocation size for sizeof(tReal)==8] _CacheAll,_CachePart: 1000*21*8=170kb (x2), _CacheOrd: 1000*20*4=80kb
			_CacheAll=new tReal*[nDim+1];for(i=0;i<=nDim;i++)_CacheAll[i]=new tReal[m_nEvts];
			_CachePart=new tReal*[nDim+1];for(i=0;i<=nDim;i++)_CachePart[i]=new tReal[m_nEvts];
			_CacheOrd=new int*[nDim];for(i=0;i<nDim;i++)_CacheOrd[i]=new int[m_nEvts];
		}

		// Adjust number of cached events used: don't use nore than 75% of precalculated events
		if(nCashedCalls > m_nEvts*3/4){ nCashedCalls=m_nEvts*3/4; }

		// Generate 'm_nEvts' events & fill sub-cell variables
		for(int nEvtGenAttempt=0;;nEvtGenAttempt++) { //Generation fails if too small events found in any sub-cell
			tReal* var=loc->var; tReal val;
			for(n=0; n<m_nEvts; n++) {
				if ((n<nCashedCalls) && (nEvtGenAttempt==0)) { //don't use cache at re-calculation
					for(i=0; i<nDim; i++){ var[i]=arrCashedData[i][n]; } val=arrCashedData[nDim][n];
				} else {
					for(i=0; i<nDim; i++){ var[i]=m_VarMin[i]+(m_VarMax[i]-m_VarMin[i])*_Rnd->Rnd(); } val=m_Gen->CallFunction(var);
				}
				for(i=0; i<nDim; i++){_CacheAll[i][n]=var[i];} _CacheAll[nDim][n]=val;
				for(i=0; i<nDim; i++) {
					int iSub=((var[i]<loc->varMid[i])?0:1); // 'left' or 'right' sub-cell
					_CacheOrd[i][n]=iSub; loc->_FSum[i][iSub]+=val; loc->_FSqr[i][iSub]+=val*val; loc->_nEv[i][iSub]++;
					if(loc->_FMax[i][iSub]<val)loc->_FMax[i][iSub]=val; if(val>0.){ if((loc->_FMinGt0[i][iSub]>val)||(loc->_FMinGt0[i][iSub]<=0.))loc->_FMinGt0[i][iSub]=val; }
					//
					int iBin=(int)(DBLtR((var[i]-m_VarMin[i])*10./(m_VarMax[i]-m_VarMin[i]))); if(iBin>=10)iBin=9; if(iBin<0)iBin=0;
					loc->fAveBin[i][iBin]+=val; loc->nAveBin[i][iBin]++;
				}
				if(m_FuncMax<val)m_FuncMax=val; //ambigous?
			}

			// Check if generation is successfull
			bool bGenOK=true;
			int nMinSubEvts=50; //min. number of events in sub-volume
			for(i=0; i<nDim; i++) { if((loc->_nEv[i][0]<nMinSubEvts) || (loc->_nEv[i][1]<nMinSubEvts)){ bGenOK=false; break; } }
			if(bGenOK)break;

			// Note: in 'm_Gen->m_bUsePreviousGrid' mode we don't need correct sub-cells: no splitting expected
			if(m_Gen->m_bUsePreviousGrid)break;

			// Check for too many generation attempts
			m_Gen->m_nCellGenRepeated++;
			if (nEvtGenAttempt>100) {
				std::cout<<"========== Cell integration FAILED ==========="<<std::endl;
				std::cout<<"Cell order="<<m_nSubOrder<<std::endl;
				std::cout<<"Num events= "<<m_nEvts<<" (cached= "<<nCashedCalls<<")"<<std::endl;
				for(i=0; i<nDim; i++){ std::cout<<"Dim "<<i<<". nLeft="<<loc->_nEv[i][0]<<", nRight="<<loc->_nEv[i][1]<<std::endl; }
				std::cout<<"Margins:"<<std::endl; int pOld=std::cout.precision(20);
				for(i=0; i<nDim; i++){ std::cout<<"Dim "<<i<<". min="<<m_VarMin[i]<<", mid="<<loc->varMid[i]<<", max="<<m_VarMax[i]<<std::endl; }
				std::cout.precision(pOld); bResOK=false; break;
			}

			// Clear generation data & repeat generation
			for(i=0;i<nDim;i++){ for(j=0;j<2;j++){ loc->_nEv[i][j]=0;loc->_FSum[i][j]=0.;loc->_FSqr[i][j]=0.;loc->_FMinGt0[i][j]=0.;loc->_FMax[i][j]=0.;} for(j=0;j<10;j++){loc->fAveBin[i][j]=0.;loc->nAveBin[i][j]=0;} }
		}

		//Debug: if((nDim>=2) && (m_VarMin[1]>=0.) && (m_VarMax[1]<=0.04) && (m_VarMin[0]>=0.87) && (m_VarMax[0]<=0.95))bLogCell=true;

		// Debug
		if((bLogCell)&&(nDim>=2)){for(int i2=0;i2<m_nSubOrder;i2++)std::cout<<".";std::cout<<"x["<<m_VarMin[0]<<":"<<m_VarMax[0]<<"], y["<<m_VarMin[1]<<":"<<m_VarMax[1]<<"]"<<std::endl; }
		//Log events: if((bLogCell)&&(m_nSubOrder>=14)) { for(n=0; n<m_nEvts; n++) { if(_CacheOrd[0][n]==0) { std::cout<<n<<": ["<<_CacheAll[0][n]<<","<<_CacheAll[1][n]<<"]= \t"<<_CacheAll[nDim][n]<<std::endl; } } }

		// Calculate sub-volume results
		loc->_SubVol=m_Volume/2.;
		if(bResOK && (!m_Gen->m_bUsePreviousGrid)) {
			//Update 'loc->_ISubVal', 'loc->_SDSub', ...
			for(i=0; i<nDim; i++) {
				if((loc->_nEv[i][0]==0) || (loc->_nEv[i][1]==0)) { bResOK=false; break; } //This may NOT occur here!
				for(j=0; j<2; j++){ tReal subI=loc->_FSum[i][j]*loc->_SubVol/loc->_nEv[i][j]; loc->_ISubVal[i][j]=subI; loc->_SDSub[i][j]=loc->_FSqr[i][j]*loc->_SubVol*loc->_SubVol/loc->_nEv[i][j]-subI*subI; }
				if(bLogCell){ for(int i2=0;i2<m_nSubOrder;i2++)std::cout<<".";std::cout<<"["<<i<<"] I="<<loc->_ISubVal[i][0]<<"|"<<loc->_ISubVal[i][1]<<", E="<<loc->_SDSub[i][0]<<"|"<<loc->_SDSub[i][1]<<std::endl; }
				//
				for(j=0; j<10; j++){
					if(loc->nAveBin[i][j]>0){ loc->fAveBin[i][j]/=loc->nAveBin[i][j]; }
					else if(j>0){ loc->fAveBin[i][j]=loc->fAveBin[i][j-1]; } //copy previous to avoid statistical 'jumps'
					//else [fAve==0, nBin==0, j==0]: bad case - what to do?
				}
			}
		}

		// Select next splitting axis
		m_iSplitAxis=-1;
		if(bResOK && (!m_Gen->m_bUsePreviousGrid)){
			m_iSplitAxis=SelectSplitAxis(loc);
		}

		// Fill selected-dimension variables
		if(m_iSplitAxis>=0){
			loc->selVal1=loc->_ISubVal[m_iSplitAxis][0]; loc->selVal2=loc->_ISubVal[m_iSplitAxis][1];
			//selSDSqr1=loc->_SDSub[m_iSplitAxis][0]; selSDSqr2=loc->_SDSub[m_iSplitAxis][1];
			loc->selVariance1=loc->_SDSub[m_iSplitAxis][0]/loc->_nEv[m_iSplitAxis][0];
			loc->selVariance2=loc->_SDSub[m_iSplitAxis][1]/loc->_nEv[m_iSplitAxis][1];
			loc->_FMax1=loc->_FMax[m_iSplitAxis][0]; loc->_FMax2=loc->_FMax[m_iSplitAxis][1];
			loc->varMidSel=loc->varMid[m_iSplitAxis];
		}
		if(bLogCell){ for(i=0;i<m_nSubOrder;i++)std::cout<<".";std::cout<<"Sel: "<<m_iSplitAxis<<std::endl; }

		//Fill variables used at/after sub-integration step
		loc->_FSumCell=loc->_FSum[0][0]+loc->_FSum[0][1]; loc->_FSqrCell=loc->_FSqr[0][0]+loc->_FSqr[0][1];
	}

	//************************************************ Sub-regions integrate
	if(!bResOK)m_iSplitAxis=-1;
	if (m_iSplitAxis>=0) {
		int iMaxOrderBeforeSplitting = m_Gen->m_nSOrderMaxActual;
		m_Sub1=new CVMGenLiteCell(m_Gen, m_nSubOrder+1);
		m_Sub2=new CVMGenLiteCell(m_Gen, m_nSubOrder+1);
		// Fill margins & weights
		for(i=0; i<nDim; i++) {
			m_Sub1->m_iSplitAxisPrev=m_iSplitAxis; m_Sub1->m_iSplitAxisPrevTimes=((m_iSplitAxis==m_iSplitAxisPrev)?(m_iSplitAxisPrevTimes+1):1);
			m_Sub2->m_iSplitAxisPrev=m_iSplitAxis; m_Sub2->m_iSplitAxisPrevTimes=m_Sub1->m_iSplitAxisPrevTimes;
			m_Sub1->m_VarMin[i]=m_VarMin[i]; m_Sub2->m_VarMax[i]=m_VarMax[i];
			m_Sub1->m_nDimOrder[i]=m_nDimOrder[i]; m_Sub2->m_nDimOrder[i]=m_nDimOrder[i];
			if(i==m_iSplitAxis) {
				m_Sub1->m_VarMax[i]=loc->varMidSel; m_Sub2->m_VarMin[i]=loc->varMidSel; m_Sub1->m_nDimOrder[i]++; m_Sub2->m_nDimOrder[i]++;
			} else {
				m_Sub1->m_VarMax[i]=m_VarMax[i]; m_Sub2->m_VarMin[i]=m_VarMin[i];
			}
			m_Sub1->m_Volume=loc->_SubVol; m_Sub2->m_Volume=loc->_SubVol;
		}
		if (m_fWghtExpected>=0.) {
			tReal tmp1=fabs(loc->selVal1+loc->selVal2);
			if (tmp1>0.) {
				m_Sub1->m_fWghtExpected=m_fWghtExpected*(fabs(loc->selVal1)/tmp1);
				m_Sub2->m_fWghtExpected=m_fWghtExpected*(fabs(loc->selVal2)/tmp1);
			} else {
				m_Sub1->m_fWghtExpected=m_fWghtExpected;
				m_Sub2->m_fWghtExpected=m_fWghtExpected;
			}
		}
		// Fill preliminary values
		m_Sub1->m_Integral=loc->selVal1; m_Sub1->m_Variance=loc->selVariance1; m_Sub1->m_FuncMax=loc->_FMax1;
		m_Sub2->m_Integral=loc->selVal2; m_Sub2->m_Variance=loc->selVariance2; m_Sub2->m_FuncMax=loc->_FMax2;

		// Call sub-integration
		{ //Fill '_CachePart' -- sub-volume part of cached events
			long nAct=0; for(n=0;n<m_nEvts;n++){ if(_CacheOrd[m_iSplitAxis][n]==0){ for(i=0;i<=nDim;i++)_CachePart[i][nAct]=_CacheAll[i][n]; nAct++; } }
			bool bIntOK=((m_Gen->m_bUseEventsSmart)?(m_Sub1->Integrate(nAct,_CachePart)):(m_Sub1->Integrate(0,NULL)));
			if(bIntOK){ if(m_FuncMax<m_Sub1->m_FuncMax)m_FuncMax=m_Sub1->m_FuncMax; }
			else{ m_Gen->m_nCellIntegrFault++; m_Sub1->m_Integral=loc->selVal1; m_Sub1->m_Variance=loc->selVariance1; }
		}
		{ //Fill '_CachePart' -- sub-volume part of cached events
			long nAct=0; for(n=0;n<m_nEvts;n++){ if(_CacheOrd[m_iSplitAxis][n]>0){ for(i=0;i<=nDim;i++)_CachePart[i][nAct]=_CacheAll[i][n]; nAct++; } }
			bool bIntOK=((m_Gen->m_bUseEventsSmart)?(m_Sub2->Integrate(nAct,_CachePart)):(m_Sub2->Integrate(0,NULL)));
			if(bIntOK){ if(m_FuncMax<m_Sub2->m_FuncMax)m_FuncMax=m_Sub2->m_FuncMax; }
			else{ m_Gen->m_nCellIntegrFault++; m_Sub2->m_Integral=loc->selVal2; m_Sub2->m_Variance=loc->selVariance2; }
		}

		if(bResOK){ //Update cell
			//Merge & delete sub-cells if no integration was performed inside
			if((m_Gen->m_bMergeSkippedCells) && (m_Sub1->m_bIntegrationSkipped) && (m_Sub2->m_bIntegrationSkipped)) {
				//Note: 'loc->selVal1+loc->selVal2' is different to 'loc->_FSumCell*m_Volume/m_nEvts'.
				//It's due to (Sum1/N1+Sum2/N2) != (Sum1+Sum2)/(N1+N2).
				m_Integral=loc->selVal1+loc->selVal2; m_Variance=loc->selVariance1+loc->selVariance2;
				m_iSplitAxis=-1; delete m_Sub1; m_Sub1=NULL; delete m_Sub2; m_Sub2=NULL;
				if(m_Gen->m_nSOrderMaxActual > iMaxOrderBeforeSplitting){ m_Gen->m_nSOrderMaxActual=iMaxOrderBeforeSplitting; }
			} else {
				m_Integral=m_Sub1->m_Integral+m_Sub2->m_Integral; m_Variance=m_Sub1->m_Variance+m_Sub2->m_Variance;
			}
		}
	} else {
		m_Integral=loc->_FSumCell*m_Volume/m_nEvts;
		m_Variance=(loc->_FSqrCell*m_Volume*m_Volume/m_nEvts-m_Integral*m_Integral)/m_nEvts; // =s^2/N
	}
	if(!bUseCache) {
		if(_CachePart!=NULL){ for(i=nDim; i>=0; i--)delete[] _CachePart[i]; delete[] _CachePart; _CachePart=NULL; }
		if(_CacheAll!=NULL){ for(i=nDim; i>=0; i--)delete[] _CacheAll[i]; delete[] _CacheAll; _CacheAll=NULL; }
		if(_CacheOrd!=NULL){ for(i=nDim-1; i>=0; i--)delete[] _CacheOrd[i]; delete[] _CacheOrd; _CacheOrd=NULL; }
	}
	if (!bResOK) return false;
	if(bLogCell){
		for(i=0;i<m_nSubOrder;i++)std::cout<<".";std::cout<<"Cell ("<<m_nSubOrder<<"): "<<m_Integral<<" +/-"<<m_Gen->m_fCLFactor*sqrt(m_Variance)<<std::endl;
	}
	//[debug]std::cout<<"cell #"<<m_Gen->m_nCellsCnt<<" value: "<<m_Integral<<std::endl; //debug
	if (m_iSplitAxis<0) {//TODO-NODEBUG //asdf
		if (m_Gen->m_MaxFinalVarianceCell==NULL) {
			m_Gen->m_MaxFinalVarianceCell=this; m_Gen->m_nMaxFinalVariance=dbgCellID;
		} else if (m_Variance>m_Gen->m_MaxFinalVarianceCell->m_Variance) {
			m_Gen->m_MaxFinalVarianceCell=this; m_Gen->m_nMaxFinalVariance=dbgCellID;
		}
	}
	return true;
}
/*
*	Select next splitting axis (different algorithms)
*/
int CVMGenLite::CVMGenLiteCell::SelectSplitAxis(CLocals* loc) {
	long nDim=m_Gen->m_nDim; long i;

	/*
	*	Algorithms:
	*	0:
	*	Select by max fMax1/fMax2 ratio, or by maxSD if fMax values are close.
	*	1:
	*	Select axis with max SD difference (if axis is allowed)
	*	If 'best' axis is forbidden -> Select by min SD difference
	*	(min-SD: max lower SD && min higher SD).
	*	Note: Min-SD critria may not be used as primary!
	*	2:
	*	Random selection.
	*	3:
	*	Select "less splitted" axis.
	*	4:
	*	Best test 1
	*	5:
	*	Best test 2
	*	6:
	*	Tests
	*/
	int	iCurrSplitMode=m_Gen->m_iAxisSelMode;
	if((iCurrSplitMode<0)||(iCurrSplitMode>10))iCurrSplitMode=0;
	//[alternate algorithms?] //if(m_nSubOrder%2==1)iCurrSplitMode=...;

	/*
	*	Parameter: iSplitAxisRepeatTimesMax <-- "Max-num-of-same-axis-selection"
	*/
	int iSplitAxisRepeatTimesMax=1; //number of same-axis-split allowed
	if(iCurrSplitMode>=4)iSplitAxisRepeatTimesMax=4;
	bool bAllowSameAxis=((m_iSplitAxisPrevTimes<iSplitAxisRepeatTimesMax) || (nDim<=1));

	//Old FAST check for emptyness - required?
	//[Old] for(i=0; i<nDim; i++){ if((loc->_SDSub[i][0]>0.)||(loc->_SDSub[i][1]>0.)){bAllEmpty=false; break;} }

	//TODO: all the stat info below MAY be calculated as 'double'

	//Collect Min/Max info
	//'Lower SD': min(sdSubLeft,sdSubRight)
	tReal minLowerSD2=-1.; int idMinLowerSD2=-1; // Minimal SD sub-volume
	tReal maxLowerSD2=-1.; int idMaxLowerSD2=-1; // Maximal lower SD sub-volume
	tReal minAlowedLoSD2=-1.; tReal maxAlowedHiSD2=0.; int idMinAlowedLoSD2=-1; // Minimal SD sub-volume for allowed axes
	tReal maxVMaxRatio=1.; int idMaxVMaxDiff=-1; // Maximal ratio of fMax values
	tReal minFAveRatio=1.; int idMinFAveRatio=-1; // Minimal (to keep 0/1) F-Ave (integral) ratio
	tReal minFMax2AveR=1.e10; int idMinFMax2Ave=-1;	// Minimal fMax/fAve at any sub-cell
	tReal maxFMax2AveR=0.; int idMaxFMax2Ave=-1;	// Maximal fMax/fAve at any sub-cell
	double minLinCorrSqr=1.; int idminLinCorr=-1;
	tReal maxTriangGrowth=-1.; int idmaxTriangGrowth=-1; //fDiff[x<xMid]+(-1)*fDiff[x>xMid]
	int idZero=-1;
	bool bAllEmpty=false;
	for(i=0; i<nDim; i++) {
		tReal fISub0=loc->_ISubVal[i][0]; tReal fISub1=loc->_ISubVal[i][1];
		tReal sdSub0=loc->_SDSub[i][0]; tReal sdSub1=loc->_SDSub[i][1];
		if(i==0){ // 'All empty' check: check sub[i==0] is enough!
			if((sdSub0<=0.)&&(sdSub1<=0.)&&(fISub0<=0.)&&(fISub1<=0.)){ bAllEmpty=true; break; }
		}
		if((fISub0==0.)||(fISub1==0.))idZero=i; // we may break here!
		bool bAllowAxis=((m_iSplitAxisPrev!=i) || (bAllowSameAxis));
		tReal lowSD2,hiSD2; if(sdSub1>=sdSub0){ lowSD2=sdSub0; hiSD2=sdSub1; }else{ lowSD2=sdSub1; hiSD2=sdSub0; };
		if ((idMinLowerSD2<0)||(lowSD2<minLowerSD2)) { minLowerSD2=lowSD2; idMinLowerSD2=i; }
		if (lowSD2>maxLowerSD2) { maxLowerSD2=lowSD2; idMaxLowerSD2=i; }
		tReal fMax0=loc->_FMax[i][0]; tReal fMax1=loc->_FMax[i][1];
		if ((fMax0>0.)&&(fMax1>0.)) { tReal rMax=fMax0/fMax1; if(rMax<1.)rMax=1./rMax; if(maxVMaxRatio<rMax){ maxVMaxRatio=rMax; idMaxVMaxDiff=i; } }
		if(bAllowAxis) {
			if ((idMinAlowedLoSD2<0)||(lowSD2<minAlowedLoSD2)) { minAlowedLoSD2=lowSD2; maxAlowedHiSD2=hiSD2; idMinAlowedLoSD2=i; }
			if ((fISub0>0.)||(fISub1>0.)){ tReal rMin=((fISub1>0.)?(fISub0/fISub1):0.); if(rMin>1.)rMin=1./rMin; if(minFAveRatio>rMin){ minFAveRatio=rMin; idMinFAveRatio=i; } }
			//
			int j;
			double yAve=4.5; double yS2=8.25; //Nt. y[0..9]: yAve: 4.5, yAveSqr: 28.5, yS^2: 8.25
			double xAve=0.; double x2Ave=0.; double xyAve=0.;
			for(j=0;j<10;j++){ double x=DBLtR(loc->fAveBin[i][j]); xAve+=x; x2Ave+=x*x; xyAve+=x*j; }
			xAve/=10; x2Ave/=10; xyAve/=10; double xS2=x2Ave-xAve*xAve; double xyCov=(xyAve-xAve*yAve);
			double xyCorr2=((xS2>0.)?(xyCov*xyCov/(xS2*yS2)):1.);
			if(minLinCorrSqr>xyCorr2){ minLinCorrSqr=xyCorr2; idminLinCorr=i; }
			//
			tReal fGrowthL=0.; tReal fGrowthR=0.; tReal fTriangGrowth=0.;
			for(j=1;j<5;j++){ tReal fDiff=loc->fAveBin[i][j]-loc->fAveBin[i][j-1]; fGrowthL+=fDiff; }
			for(j=6;j<10;j++){ tReal fDiff=loc->fAveBin[i][j]-loc->fAveBin[i][j-1]; fGrowthR+=fDiff; }
			//if(fGrowthL*fGrowthR<0.){
			if(((fGrowthL>0.)&&(fGrowthR<0.)) || ((fGrowthL<0.)&&(fGrowthR>0.))){
				fGrowthL=fabs(fGrowthL); fGrowthR=fabs(fGrowthR);
				fTriangGrowth=((fGrowthL>fGrowthR)?fGrowthR:fGrowthL); if(xAve>0.)fTriangGrowth/=xAve;
			}
			//tReal fPlusGrowth=0.;
			//for(j=1;j<10;j++){
			//	tReal fDiff=loc->fAveBin[i][j]-loc->fAveBin[i][j-1];
			//	if(j<5){ fTriangGrowth+=fDiff; fPlusGrowth+=fDiff; }else if(j>5){ fTriangGrowth-=fDiff; fPlusGrowth+=fDiff; }
			//}
			//fTriangGrowth=fabs(fTriangGrowth); fPlusGrowth=fabs(fPlusGrowth);
			//if(fPlusGrowth>0.){ fTriangGrowth/=fPlusGrowth; }else{ if(fTriangGrowth>0.)fTriangGrowth=1.e10; }
			if(maxTriangGrowth<fTriangGrowth){ maxTriangGrowth=fTriangGrowth; idmaxTriangGrowth=i; }
		}
		if(fISub0>0.){
			tReal r= fMax0*m_Volume/fISub0;
			if(minFMax2AveR>r){ minFMax2AveR=r; idMinFMax2Ave=i; }
			if(maxFMax2AveR<r){ maxFMax2AveR=r; idMaxFMax2Ave=i; }
		}
		if(fISub1>0.){
			tReal r= fMax1*m_Volume/fISub1;
			if(minFMax2AveR>r){ minFMax2AveR=r; idMinFMax2Ave=i; }
			if(maxFMax2AveR<r){ maxFMax2AveR=r; idMaxFMax2Ave=i; }
		}
	}
	if(bAllEmpty)iCurrSplitMode=3; //(or 2) select axis as 'less-splitted' or randomly

	int iSplitAxis=-1;
	switch(iCurrSplitMode) {
	case 0: { //fMax1/fMax2 ratio
		double _ParSelByMaxFRatio=1.2; //<--Paramater
		if(maxVMaxRatio>_ParSelByMaxFRatio){ iSplitAxis=idMaxVMaxDiff; }else{ iSplitAxis=idMinAlowedLoSD2; }
			} break;
	case 1: { //max SD difference
		if(idMinLowerSD2==idMinAlowedLoSD2) { iSplitAxis=idMinAlowedLoSD2; } else { iSplitAxis=idMaxLowerSD2; }
			} break;
	case 2: { //Ramdom axis
		iSplitAxis=(int)(DBLtR(m_Gen->GetRndRef()->Rnd()*nDim)); if(iSplitAxis>=nDim)iSplitAxis=nDim-1;
			} break;
	case 3: { //Less-splitted axis
		int nLess=m_nSubOrder; for(i=0;i<nDim;i++){ if((m_nDimOrder[i]<nLess)||(i==0)){ nLess=m_nDimOrder[i]; iSplitAxis=i; } }
			} break;
	case 4: { // Best test 1
		for(;;){
			//'minFAveRatio' (includes 'Zero' sub-cell case)
			double _ParSelByMinAveRatio=0.1; //<--Paramater
			if((idMinFAveRatio>=0)&&(minFAveRatio<_ParSelByMinAveRatio)){ iSplitAxis=idMinFAveRatio; break; }

			// Max fMax1/fMax2 ratio
			double _ParSelByMaxFRatio=3; //<--Paramater
			if(maxVMaxRatio>_ParSelByMaxFRatio){ iSplitAxis=idMaxVMaxDiff; break; }

			iSplitAxis=idMinAlowedLoSD2; //Default splitting
			break;
		}
			} break;
	case 5: { // Best test 2
		for(;;){
			//
			if((idMinFAveRatio>=0)&&(minFAveRatio<0.1)){ iSplitAxis=idMinFAveRatio; break; }
			//
			//if(minLinCorrSqr<0.001){ iSplitAxis=idminLinCorr; break; }
			//
			if(maxVMaxRatio>10.){ iSplitAxis=idMaxVMaxDiff; break; }
			//
			if(maxTriangGrowth>1.){ iSplitAxis=idmaxTriangGrowth; break; }
			//
			//if((idMinFAveRatio>=0)&&(minFAveRatio<0.2)){ iSplitAxis=idMinFAveRatio; break; }
			//
			if(maxVMaxRatio>3.){ iSplitAxis=idMaxVMaxDiff; break; }
			//
			//if(maxVMaxRatio>2.){ iSplitAxis=idMaxVMaxDiff; break; }
			//
			if(minAlowedLoSD2 < 0.8*maxAlowedHiSD2) { iSplitAxis=idMinAlowedLoSD2; break; }
			//iSplitAxis=idMinAlowedLoSD2; //Default splitting

			//asdfg
			break;
		}
			} break;
	case 6: { // Tests
		for(;;){
			if((idMinFAveRatio>=0)&&(minFAveRatio<0.1)){ iSplitAxis=idMinFAveRatio; break; }
			//if(minLinCorrSqr<0.001){ iSplitAxis=idminLinCorr; break; }
			if(maxVMaxRatio>10.){ iSplitAxis=idMaxVMaxDiff; break; }
			if(maxTriangGrowth>1.){ iSplitAxis=idmaxTriangGrowth; break; }
			//if((idMinFAveRatio>=0)&&(minFAveRatio<0.2)){ iSplitAxis=idMinFAveRatio; break; }
			if(maxVMaxRatio>3.){ iSplitAxis=idMaxVMaxDiff; break; }
			//if(minAlowedLoSD2 < 0.8*maxAlowedHiSD2) { iSplitAxis=idMinAlowedLoSD2; break; }
			iSplitAxis=idMinAlowedLoSD2; //Default splitting
		}
			} break;
	default: { //Full set of checks
		for(;;){
			//'Zero' sub-sell
			//if(idZero>=0){ iSplitAxis=idZero; break; }

			//'minFAveRatio' (includes 'Zero' sub-cell case)
			double _ParSelByMinAveRatio=0.1; //<--Paramater
			if((idMinFAveRatio>=0)&&(minFAveRatio<_ParSelByMinAveRatio)){ iSplitAxis=idMinFAveRatio; break; }

			// Min 'fMax/fAve' ratio separation
			//if(maxFMax2AveR>100.){ iSplitAxis=idMinFMax2Ave; break; }
			//if(true){ iSplitAxis=idMinFMax2Ave; break; } //if(minFMax2AveR)...

			// Max fMax1/fMax2 ratio
			double _ParSelByMaxFRatio=3; //<--Paramater
			if(maxVMaxRatio>_ParSelByMaxFRatio){ iSplitAxis=idMaxVMaxDiff; break; }

			//if(minAlowedLoSD2 < 0.8*maxAlowedHiSD2) {
				iSplitAxis=idMinAlowedLoSD2; //Default splitting
			//}
			break;
		}
			 } break;
	}

	//if(iSplitAxis<0) { iSplitAxis=m_iSplitAxisPrev+1; if(iSplitAxis>=nDim)iSplitAxis=0; } // Default: 'Split-Next-Axis'

	///*
	//Debug
	//if((iCurrSplitMode==4)&&(m_nSubOrder>=148)&&(m_nSubOrder<=151)){
	//if((m_nSubOrder>=148)&&(m_nSubOrder<=151)){
	if((m_nSubOrder>=(m_Gen->m_nSubOrderMax-2)) && (!m_Gen->m_bPreIntegrationMode)) {
		std::cout<<" --- Warning: the maximum splitting order is reached --- "<<std::endl;
		std::cout<<" --- Log --- "<<std::endl;
		std::cout<<"m_nSubOrder: "<<m_nSubOrder<<std::endl;
		std::cout<<"idMinFAveRatio: "<<idMinFAveRatio<<" ("<<minFAveRatio<<")"<<std::endl;
		std::cout<<"idMaxVMaxDiff:  "<<idMaxVMaxDiff<<" ("<<maxVMaxRatio<<")"<<std::endl;
		std::cout<<"idMinAlowedLoSD2:  "<<idMinAlowedLoSD2<<" ("<<minAlowedLoSD2<<" - "<<maxAlowedHiSD2<<")"<<std::endl;
		std::cout<<"idMinLowerSD2:  "<<idMinLowerSD2<<" ("<<minLowerSD2<<")"<<std::endl;
		std::cout<<"idMaxLowerSD2:  "<<idMaxLowerSD2<<" ("<<maxLowerSD2<<")"<<std::endl;
		std::cout<<"idMinFMax2Ave:  "<<idMinFMax2Ave<<" ("<<minFMax2AveR<<")"<<std::endl;
		std::cout<<"idMaxFMax2Ave:  "<<idMaxFMax2Ave<<" ("<<maxFMax2AveR<<")"<<std::endl;
		std::cout<<"idminLinCorr:   "<<idminLinCorr<<" ("<<minLinCorrSqr<<")"<<std::endl;
		std::cout<<"idmaxTriangGrowth:   "<<idmaxTriangGrowth<<" ("<<maxTriangGrowth<<")"<<std::endl;		
		//
		for(i=0; i<nDim; i++) {
			std::cout<<"["<<i<<"] I:    "<<loc->_ISubVal[i][0]<<" --- "<<loc->_ISubVal[i][1]<<std::endl;
			std::cout<<"["<<i<<"] SD:   "<<loc->_SDSub[i][0]<<" --- "<<loc->_SDSub[i][1]<<std::endl;
			std::cout<<"["<<i<<"] Fmax: "<<loc->_FMax[i][0]<<" --- "<<loc->_FMax[i][1]<<std::endl;
			std::cout<<"["<<i<<"] Fave: "<<loc->_ISubVal[i][0]/m_Volume<<" --- "<<loc->_ISubVal[i][1]/m_Volume<<std::endl;
			std::cout<<"["<<i<<"]"; for(int j=0;j<10;j++){ std::cout<<((j>0)?" ":"")<<DBLtR(loc->fAveBin[i][j]); } std::cout<<std::endl;
		}
		//
		std::cout<<"Cell I:     "<<m_Integral<<std::endl;
		std::cout<<"Cell SD:    "<<m_Variance*m_nEvts<<std::endl;
		std::cout<<"Cell Fmax:  "<<m_FuncMax<<std::endl;
		//
		m_iSplitAxis=iSplitAxis; ViewCellPicture(m_nSubOrder);
		//if(m_nSubOrder==151)exit(0);
		exit(0);
	}
	//*/

	//Nt 'm_bUsePreviousGrid' mode is processed in Cell.Integrate()
	return iSplitAxis;
}
void CVMGenLite::CVMGenLiteCell::UpdateResults() { //Updates integral/error values. To be called if child cells are changed.
	//Note: the three checks MUST always be equal: (m_iSplitAxis>=0)==(m_Sub1!=NULL)==(m_Sub2!=NULL)
	if((m_Sub1!=NULL) && (m_Sub2!=NULL)){
		m_Sub1->UpdateResults(); m_Sub2->UpdateResults();
		m_Integral=m_Sub1->m_Integral+m_Sub2->m_Integral;
		m_Variance=m_Sub1->m_Variance+m_Sub2->m_Variance;
		m_nEvts=m_Sub1->m_nEvts+m_Sub2->m_nEvts;				// useless
		//'m_FuncMax' is required for final cells only
		if(m_FuncMax<m_Sub1->m_FuncMax)m_FuncMax=m_Sub1->m_FuncMax; if(m_FuncMax<m_Sub2->m_FuncMax)m_FuncMax=m_Sub2->m_FuncMax;
	}
	//Not required: update 'm_fWghtExpected' for cells
}
void CVMGenLite::CVMGenLiteCell::SaveCell(std::ostream& ofs) {
	ofs<<'{';
	ofs.write((char*)&m_Integral,sizeof(m_Integral));
	ofs.write((char*)&m_Variance,sizeof(m_Variance));
	ofs.write((char*)&m_FuncMax,sizeof(m_FuncMax));
	ofs.write((char*)&m_nEvts,sizeof(m_nEvts));
	///*
	ofs.write((char*)&m_nSubOrder,sizeof(m_nSubOrder)); //For checks only!
	int nDim=m_Gen->m_nDim; int i;
	// Save 'nDim' part of arrays
	for(i=0; i<nDim; i++){ ofs.write((char*)&(m_VarMin[i]),sizeof(m_VarMin[i])); ofs.write((char*)&(m_VarMax[i]),sizeof(m_VarMax[i])); }
	ofs.write((char*)&m_Volume,sizeof(m_Volume)); // Ambigous info (restorable)
	ofs.write((char*)&m_iSplitAxis,sizeof(m_iSplitAxis));
	ofs.write((char*)&m_iSplitAxisPrev,sizeof(m_iSplitAxisPrev));
	ofs.write((char*)&m_iSplitAxisPrevTimes,sizeof(m_iSplitAxisPrevTimes));
	ofs.write((char*)&m_fWghtExpected,sizeof(m_fWghtExpected));
	ofs<<'['; for(i=0; i<nDim; i++){ ofs.write((char*)&(m_nDimOrder[i]),sizeof(m_nDimOrder[i])); } ofs<<']';
	//
	if(m_Sub1!=NULL) { ofs<<'l'; m_Sub1->SaveCell(ofs); }
	if(m_Sub2!=NULL) { ofs<<'r'; m_Sub2->SaveCell(ofs); }
	ofs<<'}';
}
/*
*	nDig==0 means ==sizeof(tReal)
*	iTypeID==0 means unknown
*/
bool CVMGenLite::CVMGenLiteCell::LoadCell(std::istream& ifs, int iVer, int nDig, int iTypeID) {
	char c=0; ifs>>c; for(int iSp=0;((iSp<10)&&(c==' '));iSp++){ifs>>c;} if(c!='{')return false;

	if(!CVMGenLite::LoadBinaryVal(ifs,m_Integral,nDig,iTypeID))return false;	//ifs.read((char*)&m_Integral, sizeof(m_Integral));
	if(!CVMGenLite::LoadBinaryVal(ifs,m_Variance,nDig,iTypeID))return false;	//ifs.read((char*)&m_Variance,sizeof(m_Variance));
	if(iVer<3){ //Note: in Ver<3 'SD' was saved instead of 'Variance' --> fix it!
		if(m_Gen->m_SubEvts>0){
			//TODO: correct it with m_SubEvts/m_FirstSubEvts switch
			m_Variance/=m_Gen->m_SubEvts;
		}else{ std::cout<<"Error loading old-version grid: m_SubEvts is incorrect"<<std::endl; return false; }
	}
	if(!CVMGenLite::LoadBinaryVal(ifs,m_FuncMax,nDig,iTypeID))return false;		//ifs.read((char*)&m_FuncMax,sizeof(m_FuncMax));
	if(iVer>=3){ ifs.read((char*)&m_nEvts,sizeof(m_nEvts)); }else{ m_nEvts=m_Gen->m_SubEvts; }

	int iTmp=m_nSubOrder; ifs.read((char*)&m_nSubOrder,sizeof(m_nSubOrder)); if(iTmp!=m_nSubOrder){ std::cout<<"CVMGenLiteCell::LoadCell FAILED: unexpected cell order (exp: "<<iTmp<<", read: "<<m_nSubOrder<<")"<<std::endl; m_nSubOrder=iTmp; return false; }
	int nDim=m_Gen->m_nDim; int i;
	for(i=0; i<nDim; i++){
		if(!CVMGenLite::LoadBinaryVal(ifs,m_VarMin[i],nDig,iTypeID))return false;
		if(!CVMGenLite::LoadBinaryVal(ifs,m_VarMax[i],nDig,iTypeID))return false;
	}
	if(!CVMGenLite::LoadBinaryVal(ifs,m_Volume,nDig,iTypeID))return false;
	ifs.read((char*)&m_iSplitAxis, sizeof(m_iSplitAxis));
	ifs.read((char*)&m_iSplitAxisPrev, sizeof(m_iSplitAxisPrev));
	ifs.read((char*)&m_iSplitAxisPrevTimes, sizeof(m_iSplitAxisPrevTimes));
	if(!CVMGenLite::LoadBinaryVal(ifs,m_fWghtExpected,nDig,iTypeID))return false;

	ifs>>c;
	if(c=='[') { for(i=0; i<nDim; i++){ ifs.read((char*)&m_nDimOrder[i], sizeof(m_nDimOrder[i])); } ifs>>c; if(c!=']'){ std::cout<<"CVMGenLiteCell::LoadCell FAILED: invalid 'm_nDimOrder' data"<<std::endl; return false; } ifs>>c; }
	else {
		//calculate 'm_nDimOrder[i]' from Split-Axis or 'm_VarMin/Max[i]' values?
		//Note: 'm_nDimOrder[i]' values are used for integration only, and not required for generation.
	}
	if(c=='l') { if(m_Sub1!=NULL){delete m_Sub1; m_Sub1=NULL;} m_Sub1=new CVMGenLiteCell(m_Gen, m_nSubOrder+1); if(!m_Sub1->LoadCell(ifs,iVer,nDig,iTypeID))return false; ifs>>c; }
	if(c=='r') { if(m_Sub2!=NULL){delete m_Sub2; m_Sub2=NULL;} m_Sub2=new CVMGenLiteCell(m_Gen, m_nSubOrder+1); if(!m_Sub2->LoadCell(ifs,iVer,nDig,iTypeID))return false; ifs>>c; }
	if(c!='}')return false;
	return true;
}
void CVMGenLite::CVMGenLiteCell::ViewCellPicture(int id){ //id: to create unique file "cell_%ld.eps"
#ifdef Decl_CVMOutput
	char buf[100]; int nDim=m_Gen->m_nDim; tReal var[CVMGen_MAXDIM]; int n;
	int nAxisX=m_iSplitAxis; if(m_iSplitAxisPrev<0)return;
	std::cout<<"<Cell> m_iSplitAxisPrev: "<<m_iSplitAxisPrev<<", m_iSplitAxis: "<<m_iSplitAxis<<", Order: "<<m_nSubOrder<<std::endl;
	std::cout<<"<Split history> "; for(int n=0;n<nDim;n++){ std::cout<<n<<": "<<m_nDimOrder[n]<<" "; } std::cout<<std::endl;
#ifdef VMLIB_WIN
	unsigned int uiFormatOld=_set_output_format(_TWO_DIGIT_EXPONENT);
#endif //VMLIB_WIN
	bool bAxis0OnlySplit=(nDim>4);
	for(int nAxis0=0; nAxis0<nDim; nAxis0++){
		if(bAxis0OnlySplit && (nAxis0!=m_iSplitAxisPrev))continue;
		for(int nAxis1=0; nAxis1<nDim; nAxis1++){
			if((!bAxis0OnlySplit) && (nAxis1<nAxis0))continue;
			if(nAxis1==nAxis0)continue;
			if(id>=0){ sprintf(buf,"Cell_%ld_%ldx%ld.eps",id,nAxis0,nAxis1); }else{ sprintf(buf,"Cell_X_%ldx%ld.eps",nAxis0,nAxis1); }
			CVMOutputEPS out(buf); out.Start();
			if(!out.IsOpen()) { std::cout<<"[WARNING] Cell::ViewCellPicture(): can't save EPS object (file: "<<buf<<")"<<std::endl; continue; }
			double x0,x1,y0,y1; out.GetMargins(&x0,&x1,&y0,&y1); out.SetTextAlignX(CVMOutput::TA_LEFT); out.SetTextAlignY(CVMOutput::TA_BOTTOM); out.SetFontSize(out.GetFontSize()/2.f);
			int i,j; int iNPts=9;
			for(i=0;i<iNPts;i++){
				for(j=0;j<iNPts;j++){
					tReal fI=0.; int NfAve=11; tReal fISqr=0.;
					var[nAxis0]=m_VarMin[nAxis0]+(m_VarMax[nAxis0]-m_VarMin[nAxis0])*i/(iNPts-1);
					var[nAxis1]=m_VarMin[nAxis1]+(m_VarMax[nAxis1]-m_VarMin[nAxis1])*j/(iNPts-1);
					for(int iDif=0; iDif<NfAve; iDif++){
						//for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*(iDif+1)/(NfAve+1); } }
						for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*(iDif)/(NfAve-1); } }
						tReal f=m_Gen->CallFunction(var); //Nt: RND is used inside for azimutal angle
						fI+=f; fISqr+=f*f;
					}
					fI/=NfAve;
					for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*0.5; } }
					//tReal fIMid=m_Gen->CallFunction(var); //Nt: RND is used inside for azimutal angle
					//tReal fDiff=fabs(fI-fIMid)/((fI>0.)?fI:1.);
					tReal fDiff=sqrt(fabs(fISqr/NfAve-fI*fI))/((fI>0.)?fI:1.);
					sprintf(buf,"%.2g %ld%%",float(DBLtR(fI)),(int)(100.*DBLtR(fDiff)));
					//std::cout<<"i="<<i<<", j="<<j<<", f="<<fI<<", buf="<<buf<<std::endl;
					double yAdd=((i%2==1)?((y1-y0)/25):0.);
					double xAdd=-(x1-x0)/25;
					out.TextOut(x0+(x1-x0)*i/iNPts+xAdd, y0+(y1-y0)*j/iNPts+yAdd, buf);
				}
			}
			out.Finalize();
			//
			iNPts=51;
			if(id>=0){ sprintf(buf,"Cell_%ld_%ldx%ld.Scat.eps",id,nAxis0,nAxis1); }else{ sprintf(buf,"Cell_X_%ldx%ld.Scat.eps",nAxis0,nAxis1); }
			CVMScatLite scat("",DBLtR(m_VarMin[nAxis0]),DBLtR(m_VarMax[nAxis0]),DBLtR(m_VarMin[nAxis1]),DBLtR(m_VarMax[nAxis1]));
			//CVMScatLite scat("",0,1,0,1);
			//CVMScatLite scatD("D",m_VarMin[nAxis0],m_VarMax[nAxis0],m_VarMin[nAxis1],m_VarMax[nAxis1]);
			scat.SetAutoAverage(true,iNPts,iNPts);
			for(i=0;i<iNPts;i++){
				for(j=0;j<iNPts;j++){
					var[nAxis0]=m_VarMin[nAxis0]+(m_VarMax[nAxis0]-m_VarMin[nAxis0])*i/(iNPts-1);
					var[nAxis1]=m_VarMin[nAxis1]+(m_VarMax[nAxis1]-m_VarMin[nAxis1])*j/(iNPts-1);
					int NfAve=iNPts; 
					for(int iDif=0; iDif<NfAve; iDif++){
						//for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*(iDif+1)/(NfAve+1); } }
						for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*(iDif)/(NfAve-1); } }
						tReal f=m_Gen->CallFunction(var); //Nt: RND is used inside for azimutal angle
						scat.Fill(DBLtR(var[nAxis0]),DBLtR(var[nAxis1]),DBLtR(f));
						//scat.Fill((double)i/(iNPts-1),(double)j/(iNPts-1),DBLtR(f));
					}
					//for(n=0; n<nDim; n++){ if((n!=nAxis0)&&(n!=nAxis1)){ var[n]=m_VarMin[n]+(m_VarMax[n]-m_VarMin[n])*0.5; } }
					//scatD.Fill(var[nAxis0],var[nAxis1],DBLtR(m_Gen->CallFunction(var)));
				}
			}
			{ CVMHistLiteView vi(&scat); vi.SaveAsEPS(buf); }
		}
	}
#ifdef VMLIB_WIN
	_set_output_format(uiFormatOld);
#endif //VMLIB_WIN
#else
	std::cout<<"[WARNING] Cell::ViewCellPicture() skipped: no Output objects is linked"<<std::endl;
#endif //Decl_CVMOutput
}

//
CVMGenLite::CVMGenLite() : CVMGen() {
	m_fPrecisionReq=0.01;
	m_fCLFactor=MC_CL_fact_used; //(def: ~3) Integral[sqrt(1/2pi)exp(-x2/2),x,-CLFactor,+CLFactor]=C.L.
	m_fIntegralExpected=0.;
	//Fails for 1-dim integration: m_nSubOrderMin=15; // min 16 cells per every axis
	m_nSubOrderMin=5;	// actually 6 (the first cell is of order 0)
	m_nSubOrderMax=200; // 20 --> max 2k cells per axis
	m_SubEvts=1000;
	m_nFirstOrders=-1;				//number of 'First' orders (with separate 'm_SubEvts')
	m_SubEvtsFirstOrders=8000;	//num of sub-events at 'First' orders (order <= m_nFirstOrders)
	m_nCellsCnt=0; m_nSOrderMaxActual=0; m_nCellGenRepeated=0; m_nMachinePrecFault=0; m_nCellIntegrFault=0;
	m_bLogCellsCnt=true;
	m_fMaxWgtFactor=-1.;
	m_fCellErrMaxSqr=-1.;
	m_Cells=NULL;
	m_bUseEventsSmart=true;
	m_bGenWeightErrors=false;		// Faster by default
	m_bUpdateMaxValue=false;		//Faster by default
	m_bGenFixThreshold=false;		//fix 'rnd'-threshold for all iterations in GenEvent()
	m_fMaxSafetyFactor=-1.;			// (default: -1) generate with Max=Fmax*SafetyFactor (<0 if not applied)
	m_bUpdateIntegrationNext=false;
	m_bUsePreviousGrid=false;
	//
	// 'm_bMergeSkippedCells': saves memory, but slows down unweighed generation.
	// Test[pe->peA,1%]. generation: ~20% slower, integration: <3% slower, memory saved: ~36% (16489 vs. 25803 cells)
	m_bMergeSkippedCells=true; //save memory, but slow generation
	//
	m_iAxisSelMode=0; //0:(not next), 1:(lowest SD<->mid SD), //2: random
	m_nGenEvtFailed=0;
	//Prev: m_iAxisSelMode=1;
	_Cache1=NULL; _Cache2=NULL; _iCache1=NULL; m_LocalsCache=NULL;
	//
	m_iGenEvtCalls=0; m_iGenEvtLoops=0; // used to calculate efficiency of 'GenEvent' calls
}
CVMGenLite::~CVMGenLite() {
	if(m_Cells!=NULL) { delete m_Cells; m_Cells=NULL; }
	if(m_nCellsCnt!=0) { std::cout<<"CVMGenLite memory leak ERROR: Not all the cells are deleted. Cells left: "<<m_nCellsCnt<<std::endl; }
	if((_Cache1!=NULL) || (_Cache2!=NULL) || (_iCache1!=NULL) || (m_LocalsCache!=NULL)) { std::cout<<"CVMGenLite memory leak ERROR: cache buffers not released."<<std::endl; }
}
bool CVMGenLite::InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy) {
	bool ret=CVMGen::InitImpl(nDim, _VarMin, _VarMax, func, bFuncOwner, rndCpy); if(!ret)return false;
	//Clear values changed with function change
	m_fIntegralExpected=0.; //Affects rate of convergence only, don't set if unknown (default: 0)
	m_nGenEvtFailed=0;
	for(int i=0; i<100; i++){ m_nGenEvtIterations[i]=0; }
	return true;
}
void CVMGenLite::SetExpectedIntegral(tReal valExpected) {
	m_fIntegralExpected=valExpected;
}
tReal CVMGenLite::Integrate(double fPrecision) {
#ifdef VM_DEBUG_FULL
	CAlHepLib::ViewStackUsed("CVMGenLite::Integrate");
#endif //VM_DEBUG_FULL
	bool bUpdateInt=m_bUpdateIntegrationNext; m_bUpdateIntegrationNext=false;
	if(bUpdateInt) {
		bool bPrevIntOK=(m_Cells!=NULL);
		if(bPrevIntOK && (m_Cells->m_Integral<=0.))bPrevIntOK=false;
		if(!bPrevIntOK){ std::cout<<"[GenLite]WARNING: Can't update integration. No previous valid integration is loaded."<<std::endl; bUpdateInt=false; }
	}
	if(m_bUsePreviousGrid){ ApplyGridFactor(0.,false); } //Clear previous grid

	if((!bUpdateInt) && (!m_bUsePreviousGrid)) {
		//Clear previous integration values
		m_MaxFinalVarianceCell=NULL; m_nMaxFinalVariance=-1; // Debug info
		m_nCellsCnt=0; m_nSOrderMaxActual=0; m_nCellGenRepeated=0; m_iGenEvtCalls=0; m_iGenEvtLoops=0; // Reset statistics
		if(m_Cells!=NULL){ delete m_Cells; m_Cells=NULL; }
	}

	//Check out parameters compliance
	//Note: these parameters MAY be changed after Init() call
	//Note: 'm_fPrecisionReq' is previous 'requested' (but not 'achieved') precision => ignore it here!
	m_fPrecisionReq=fPrecision; if(m_fPrecisionReq<=0.){ std::cout<<"[GenLite]ERROR: ZERO error requested. Set to 1%."<<std::endl; m_fPrecisionReq=0.01; }
	if(m_fCLFactor<=0.){ std::cout<<"[GenLite]ERROR: ZERO 'm_fCLFactor' defined. Set to "<<MC_CL_fact_used<<std::endl; m_fCLFactor=MC_CL_fact_used; }
	if(m_nSubOrderMin>20){ std::cout<<"[GenLite]ERROR: 'm_nSubOrderMin' parameter is too high. Set to 20."<<std::endl; m_nSubOrderMin=20; }
	if((m_nSubOrderMax<m_nSubOrderMin)||(m_nSubOrderMax>200)){ std::cout<<"[GenLite]ERROR: 'm_nSubOrderMax' parameter is invalid. Set to 200."<<std::endl; m_nSubOrderMax=200; }
	if(m_SubEvts>CVMGenLite_MAXSUBEVTS){ std::cout<<"[GenLite]ERROR: 'm_SubEvts' parameter is too high. Set to "<<CVMGenLite_MAXSUBEVTS<<std::endl; m_SubEvts=CVMGenLite_MAXSUBEVTS; }
	//if(m_nFirstOrders>40){ std::cout<<"[GenLite]ERROR: 'm_nFirstOrders' parameter is too high ("<<m_nFirstOrders<<"). Set to 40"<<std::endl; m_nFirstOrders=40; }
	if(m_nFirstOrders>=m_nSubOrderMax){ std::cout<<"[GenLite]ERROR: 'm_nFirstOrders' parameter is too high ("<<m_nFirstOrders<<"). Set to "<<m_nSubOrderMax-1<<std::endl; m_nFirstOrders=m_nSubOrderMax-1; }
	if(m_SubEvtsFirstOrders>CVMGenLite_MAXSUBEVTS){ if(m_nFirstOrders>=0){std::cout<<"[GenLite]ERROR: 'm_SubEvtsFirstOrders' parameter is too high ("<<m_SubEvtsFirstOrders<<"). Set to "<<CVMGenLite_MAXSUBEVTS<<std::endl;} m_SubEvtsFirstOrders=CVMGenLite_MAXSUBEVTS; }
	if(m_SubEvtsFirstOrders<m_SubEvts){ if(m_nFirstOrders>=0){std::cout<<"[GenLite]ERROR: 'm_SubEvtsFirstOrders' parameter is too small ("<<m_SubEvtsFirstOrders<<"). Set to "<<m_SubEvts<<std::endl;} m_SubEvtsFirstOrders=m_SubEvts; }

	//Print out parameters
	if(bUpdateInt)std::cout<<"[GenLite] Parameter 'Update Previous': "<<bUpdateInt<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_nDim': "<<m_nDim<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_fPrecisionReq': "<<m_fPrecisionReq<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_fCLFactor': "<<m_fCLFactor<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_fIntegralExpected': "<<m_fIntegralExpected<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_nSubOrderMin': "<<m_nSubOrderMin<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_nSubOrderMax': "<<m_nSubOrderMax<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_SubEvts': "<<m_SubEvts<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_nFirstOrders': "<<m_nFirstOrders<<std::endl;
	if(m_nFirstOrders>=0){ std::cout<<"[GenLite] Parameter 'm_SubEvtsFirstOrders': "<<m_SubEvtsFirstOrders<<std::endl; }
	std::cout<<"[GenLite] Parameter 'm_iAxisSelMode': "<<m_iAxisSelMode<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_bMergeSkippedCells': "<<m_bMergeSkippedCells<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_fMaxWgtFactor': "<<m_fMaxWgtFactor<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_fCellErrMax': "<<((m_fCellErrMaxSqr>0.)?DBLtR(sqrt(m_fCellErrMaxSqr)):-1.)<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_bUseEventsSmart': "<<m_bUseEventsSmart<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_bGenWeightErrors': "<<m_bGenWeightErrors<<std::endl;
	std::cout<<"[GenLite] Parameter 'm_bUpdateMaxValue': "<<m_bUpdateMaxValue<<std::endl;

	//Locals
	int iTmp=0; int i;

	//Allocate buffers. TODO: allocate if (m_bUseEventsSmart) only
	//Size: _Cache1,_Cache2: m_nSubOrderMax*(m_nDim+1)*m_SubEvts*8 --> 100*21*10000*8 ~ 17[x2]Mb [x2 <== caches]
	//Size: [Omax=200, nDim=4, nSubEvt=10000, sizeof(tReal)=8] --> 200*5*10000*8 ~ 80[x2]Mb [x2 <== caches]
	//Size: [Omax=200, nDim=10, nSubEvt=1000, sizeof(tReal)=8] --> 200*11*1000*8 ~ 17.2[x2]Mb [x2 <== caches]
	//Size: [Omax=200, nDim=10, nSubEvt=10000, sizeof(tReal)=32] --> 200*11*10000*32 ~ 688[x2]Mb [x2 <== caches]
	long dbgNValsPerDim=((m_nFirstOrders>=0)?(m_SubEvtsFirstOrders*(m_nFirstOrders+1)+m_SubEvts*(m_nSubOrderMax-(m_nFirstOrders+1))):(m_SubEvts*m_nSubOrderMax));
	std::cout<<"[GenLite] Function values cache allocating: "<<((float)(m_nSubOrderMax*(2*sizeof(tReal*)*(m_nDim+2)+sizeof(int*)*(m_nDim+1))+dbgNValsPerDim*(2*sizeof(tReal)*(m_nDim+1)+sizeof(int)*(m_nDim))))/1024/1024<<" Mb"<<std::endl;
	_Cache1=new tReal**[m_nSubOrderMax]; _Cache2=new tReal**[m_nSubOrderMax]; _iCache1=new int**[m_nSubOrderMax]; bool bCacheAllocOK=((_Cache1!=NULL)&&(_Cache2!=NULL)&&(_iCache1!=NULL));
	for(iTmp=0; iTmp<m_nSubOrderMax; iTmp++){
		if(!bCacheAllocOK){ if(_Cache1!=NULL)_Cache1[iTmp]=NULL; if(_Cache2!=NULL)_Cache2[iTmp]=NULL; if(_iCache1!=NULL)_iCache1[iTmp]=NULL; continue; }
		_Cache1[iTmp]=new tReal*[m_nDim+1]; _Cache2[iTmp]=new tReal*[m_nDim+1]; _iCache1[iTmp]=new int*[m_nDim];
		if((_Cache1[iTmp]==NULL) || (_Cache2[iTmp]==NULL) || (_iCache1[iTmp]==NULL))bCacheAllocOK=false;
		long nSubEvtAtOrder=m_SubEvts; if((m_nFirstOrders>=0)&&(iTmp<=m_nFirstOrders))nSubEvtAtOrder=m_SubEvtsFirstOrders;
		for(register int i1=0; i1<=m_nDim; i1++){
			if(!bCacheAllocOK){ if(_Cache1[iTmp]!=NULL)_Cache1[iTmp][i1]=NULL; if(_Cache2[iTmp]!=NULL)_Cache2[iTmp][i1]=NULL; continue; }
			_Cache1[iTmp][i1]=new tReal[nSubEvtAtOrder]; _Cache2[iTmp][i1]=new tReal[nSubEvtAtOrder]; if((_Cache1[iTmp][i1]==NULL)||(_Cache2[iTmp][i1]==NULL))bCacheAllocOK=false;
		}
		for(register int i2=0; i2<m_nDim; i2++){
			if(!bCacheAllocOK){ if(_iCache1[iTmp]!=NULL)_iCache1[iTmp][i2]=NULL; continue; }
			_iCache1[iTmp][i2]=new int[nSubEvtAtOrder]; if(_iCache1[iTmp][i2]==NULL)bCacheAllocOK=false;
		}
	}
	if(!bCacheAllocOK){ //Clear parts of '_CacheX' if allocation failed
		std::cout<<"[GenLite] ERROR! Function values cache allocating FAILED"<<std::endl;
		if(_Cache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_Cache1[i]!=NULL){ for(register int j=m_nDim;j>=0;j--){ if(_Cache1[i][j]!=NULL){delete[] _Cache1[i][j];} } delete[] _Cache1[i]; } } delete[] _Cache1; _Cache1=NULL; }
		if(_Cache2!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_Cache2[i]!=NULL){ for(register int j=m_nDim;j>=0;j--){ if(_Cache2[i][j]!=NULL){delete[] _Cache2[i][j];} } delete[] _Cache2[i]; } } delete[] _Cache2; _Cache2=NULL; }
		if(_iCache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_iCache1[i]!=NULL){ for(register int j=m_nDim-1;j>=0;j--){ if(_iCache1[i][j]!=NULL){delete[] _iCache1[i][j];} } delete[] _iCache1[i]; } } delete[] _iCache1; _iCache1=NULL; }
	}
	std::cout<<"[GenLite] Local variables cache allocating: "<<((float)(sizeof(CVMGenLiteCell::CLocals)*m_nSubOrderMax))/1024/1024<<" Mb"<<std::endl;
	m_LocalsCache=new CVMGenLiteCell::CLocals[m_nSubOrderMax];
	if(m_LocalsCache==NULL){
		std::cout<<"[GenLite] ERROR! Local variables cache allocating FAILED!"<<std::endl;
		exit(0); //No chance to continue
	}

	//First cell
	if((!bUpdateInt) && (!m_bUsePreviousGrid)) {
		m_Cells=new CVMGenLiteCell(this,0);
		for(i=0;i<m_nDim;i++){m_Cells->m_VarMin[i]=m_VarMin[i];m_Cells->m_VarMax[i]=m_VarMax[i];} m_Cells->m_Volume=m_Volume;
		m_Cells->m_fWghtExpected=1.;
	}

	//Run
	//Step #1: estimate cross section with small precision
	bool intOK=true; tReal fRelErrorAchievedSqr=1.;
	if(m_fIntegralExpected>0.){ std::cout<<"[GenLite] 'Expected' integral value: "<<m_fIntegralExpected<<std::endl; }
	if(bUpdateInt){
		m_fIntegralExpected=m_Cells->m_Integral; //Use loaded value as 'expected', 'm_Cells->m_Integral' was checked at Integrate's startup.
		if(m_fIntegralExpected>0.)fRelErrorAchievedSqr=fabs(m_Cells->m_Variance*m_fCLFactor*m_fCLFactor/(m_fIntegralExpected*m_fIntegralExpected));
		std::cout<<"[GenLite] Pre-loaded integral value: "<<m_fIntegralExpected<<std::endl;
	}
	bool bNeedIntegralEstimate = ((m_fIntegralExpected<=0.) && (!m_bUsePreviousGrid));
	if(bNeedIntegralEstimate) {
		int nSubOrderMin=m_nSubOrderMin; int nSubOrderMax=m_nSubOrderMax; tReal fPrecisionReq=m_fPrecisionReq; //save parameters
		m_bPreIntegrationMode=true;
		int iIt=0;

		//[TODO] //parameter: allow use previous grid in next iterations (for re-integration with higher precision, or other 'expected value')
		//[TODO] bool bAllowGridUpdate = true;	//parameter

		//Step #1.1: try to integrate with flat algorithm (increasing splitting order in each iteration)
		//
		m_nSubOrderMax=m_nSubOrderMin;	//don't split volume deeper at this step
		for(;((m_fIntegralExpected<=0.) && intOK);){
			std::cout<<"[GenLite] Pre-integration step. Iteration no. "<<iIt<<", order: "<<m_nSubOrderMin<<".."<<m_nSubOrderMax<<", precision: "<<m_fPrecisionReq<<", integral: "<<m_fIntegralExpected<<std::endl; iIt++;
			intOK=m_Cells->Integrate(0,NULL);
			if(!intOK){ std::cout<<"[GenLite] Pre-integration step failed. Integration error spotted."<<std::endl; continue; }
			if(m_Cells->m_Integral>0.) { m_fIntegralExpected=m_Cells->m_Integral; }
			else{
				//increase splitting order
				if(m_nSubOrderMin >= (nSubOrderMin+8)) {
					std::cout<<"[GenLite] Pre-integration step failed. Can't estimate integral value (final splitting order: "<<m_nSubOrderMin<<")"<<std::endl;
					intOK=false;
				}else{
					m_nSubOrderMin+=2; m_nSubOrderMax+=2;
				}
			}
		}

		//Step #1.2: The expected value found may be much less than actual ==> try to raise it
		//
		if ((m_fIntegralExpected>0.) && intOK) {
			m_nSubOrderMin=nSubOrderMin; m_nSubOrderMax=nSubOrderMax; m_fPrecisionReq=0.1;
			bool bLoop=true; tReal fPrev=m_fIntegralExpected;
			for(;(bLoop && intOK);) {
				std::cout<<"[GenLite] Pre-integration step. Iteration no. "<<iIt<<", order: "<<m_nSubOrderMin<<".."<<m_nSubOrderMax<<", precision: "<<m_fPrecisionReq<<", integral: "<<m_fIntegralExpected<<std::endl; iIt++;
				m_fIntegralExpected*=4; //to be restored later
				intOK=m_Cells->Integrate(0,NULL);
				if(!intOK){ std::cout<<"[GenLite] Pre-integration step failed."<<std::endl; continue; }
				if(m_Cells->m_Integral<=0.) { std::cout<<"[GenLite] Pre-integration step failed."<<std::endl; intOK=false; continue; }
				m_fIntegralExpected=m_Cells->m_Integral;
				if(m_fIntegralExpected < fPrev*1.5) { bLoop=false; }
				fPrev=m_fIntegralExpected;
			}
		}

		//Step #1.3: update integral value with higher precision
		//
		bool bUseStep13=false;	//TODO: test & debug this!
		if ((m_fIntegralExpected>0.) && intOK && bUseStep13) {
			m_nSubOrderMin=nSubOrderMin; m_nSubOrderMax=nSubOrderMax; m_fPrecisionReq=0.01;
			for(;((m_fPrecisionReq>=(fPrecisionReq*10)) && intOK);) {
				std::cout<<"[GenLite] Pre-integration step. Iteration no. "<<iIt<<", order: "<<m_nSubOrderMin<<".."<<m_nSubOrderMax<<", precision: "<<m_fPrecisionReq<<", integral: "<<m_fIntegralExpected<<std::endl; iIt++;
				intOK=m_Cells->Integrate(0,NULL);
				if(!intOK){ std::cout<<"[GenLite] Pre-integration step failed. Integration error spotted."<<std::endl; continue; }
				if(m_Cells->m_Integral>0.) { m_fIntegralExpected=m_Cells->m_Integral; m_fPrecisionReq*=0.1; }
				else { std::cout<<"[GenLite] Pre-integration step failed (Internal error)."<<std::endl; intOK=false; }
				
			}
		}

		m_bPreIntegrationMode=false;
		m_nSubOrderMin=nSubOrderMin; m_nSubOrderMax=nSubOrderMax; m_fPrecisionReq=fPrecisionReq; //restore parameters
		if (intOK) {
			if(m_fIntegralExpected>0.) {
				fRelErrorAchievedSqr=fabs(m_Cells->m_Variance*m_fCLFactor*m_fCLFactor/(m_fIntegralExpected*m_fIntegralExpected));
				std::cout<<"[GenLite] Pre-integration step finished. Estimated integral value: "<<m_fIntegralExpected<<std::endl;
			}
		}
	}

	//Step #2: integrate with required precision
	//
	//	Note:
	//		- more smart is to achieve 'm_fPrecisionReq' in several iterations (say, *0.1 error each),
	//		  this allows to use 'updated' integral value in every iteration
	//		- TODO: implement this!  -- or move 'Step #1.3' here?
	//
	bool bNeedIntegrate= (intOK && ((fRelErrorAchievedSqr > (m_fPrecisionReq*m_fPrecisionReq)) || m_bUsePreviousGrid));
	if(bNeedIntegrate) {
		//TODO: clear cells & repeat, or skip cells already splitted
		//if(fRelErrorAchievedSqr<1.) { std::cout<<"Not implemented"<<std::endl; exit(0); }
		intOK=m_Cells->Integrate(0,NULL);
	}
	if(m_bUsePreviousGrid)m_bUsePreviousGrid=false;

	//Get results
	if(intOK){
		m_Integral=m_Cells->m_Integral;
		m_Error=m_fCLFactor*sqrt(fabs(m_Cells->m_Variance)); if(m_Cells->m_Variance<0.)m_Error=-m_Error;//-1 to mark error
	}

	//Show results banner
	if ((intOK) && (fabs(m_Error)>fabs(m_Integral))) {//TODO: allow negative values (and result=0) integration here!
		intOK=false;
	}
	int iCoutPrec=std::cout.precision();
	std::cout<<"+------------------------------------------------------------------------"<<std::endl;
	std::cout<<"| CVMGenLite integration "<<(intOK?" DONE ":"FAILED")<<std::endl;
	if(intOK && (m_nCellIntegrFault>0)){ std::cout<<"| ... with "<<m_nCellIntegrFault<<" ERRORS !"<<std::endl; std::cout<<"| The result is suspicious"<<std::endl; }
	std::cout.precision((int)DBLtR(2.-log(m_fPrecisionReq)/2.3));
	std::cout<<"| Integration result: "; CVMUtil::Cout(&std::cout,m_Integral); std::cout<<" +/- "; CVMUtil::Cout(&std::cout,m_Error); std::cout<<" (estimated: "; CVMUtil::Cout(&std::cout,m_fIntegralExpected); std::cout<<")"<<std::endl;
	std::cout.precision(iCoutPrec);
	std::cout<<"| Precision requested: "<<m_fPrecisionReq<<", achieved: "<<((m_Integral>0.)?(m_Error/m_Integral):1.)<<std::endl;
	std::cout<<"+------------------------------------------------------------------------"<<std::endl;
	std::cout<<"| Number of Cells Integration Failed: "<<m_nCellIntegrFault<<std::endl;
	std::cout<<"| Number of Cells Failed due to Machine Precision: "<<m_nMachinePrecFault<<std::endl;
	std::cout<<"| Maximal Cell Order: "<<m_nSOrderMaxActual<<std::endl;
	std::cout<<"| Total Number of Cells: "<<m_nCellsCnt<<std::endl;
	std::cout<<"| Total Function Calls: "<<m_nFuncCalls<<std::endl;
	std::cout<<"| Function Calls per Cell: "<<m_SubEvts<<std::endl;
	if(m_nFirstOrders>=0){ std::cout<<"| Function Calls per first "<<m_nFirstOrders<<" cell-orders: "<<m_SubEvtsFirstOrders<<std::endl; }
	std::cout<<"| Number of sub-generations repeated: "<<m_nCellGenRepeated<<std::endl;
	long iMaxOrders[CVMGen_MAXDIM]; GetMaxSplittingOrders(iMaxOrders);
	std::cout<<"| Split order per dim: ";for(i=0;i<m_nDim;i++){ std::cout<<i<<">"<<iMaxOrders[i]<<" "; }std::cout<<std::endl;
	std::cout<<"+------------------------------------------------------------------------"<<std::endl;
	std::cout.precision(iCoutPrec);
	bool bShowMaxVarianceCell=false;
	if(bShowMaxVarianceCell && (m_MaxFinalVarianceCell!=NULL)) {
		std::cout<<"maxFinalVariance="<<m_MaxFinalVarianceCell->m_Variance<<" ["<<m_MaxFinalVarianceCell->m_VarMin[0]<<":"<<m_MaxFinalVarianceCell->m_VarMin[1]<<"]"<<m_nMaxFinalVariance<<std::endl;
		std::cout<<"w="<<m_MaxFinalVarianceCell->m_fWghtExpected<<std::endl;
		std::cout<<"o="<<m_MaxFinalVarianceCell->m_nSubOrder<<std::endl;
		std::cout<<"I="<<m_MaxFinalVarianceCell->m_Integral<<", I/I0="<<m_MaxFinalVarianceCell->m_Integral/m_Integral<<std::endl;
		std::cout<<"vol="<<m_MaxFinalVarianceCell->m_Volume<<", rel. vol="<<m_MaxFinalVarianceCell->m_Volume/((m_Volume>=0.)?m_Volume:1)<<std::endl;
		std::cout<<"m_Volume="<<m_Volume<<std::endl;
	}
	//release memory
	if(_Cache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_Cache1[i]!=NULL){ for(register int j=m_nDim;j>=0;j--){ if(_Cache1[i][j]!=NULL){delete[] _Cache1[i][j];} } delete[] _Cache1[i]; } } delete[] _Cache1; _Cache1=NULL; }
	if(_Cache2!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_Cache2[i]!=NULL){ for(register int j=m_nDim;j>=0;j--){ if(_Cache2[i][j]!=NULL){delete[] _Cache2[i][j];} } delete[] _Cache2[i]; } } delete[] _Cache2; _Cache2=NULL; }
	if(_iCache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ if(_iCache1[i]!=NULL){ for(register int j=m_nDim-1;j>=0;j--){ if(_iCache1[i][j]!=NULL){delete[] _iCache1[i][j];} } delete[] _iCache1[i]; } } delete[] _iCache1; _iCache1=NULL; }
	//[To delete] if(_Cache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ for(register int j=m_nDim;j>=0;j--)delete[] _Cache1[i][j]; delete[] _Cache1[i];}delete[] _Cache1; _Cache1=NULL; }
	//[To delete] if(_Cache2!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ for(register int j=m_nDim;j>=0;j--)delete[] _Cache2[i][j]; delete[] _Cache2[i];}delete[] _Cache2; _Cache2=NULL; }
	//[To delete] if(_iCache1!=NULL){ for(int i=m_nSubOrderMax-1;i>=0;i--){ for(register int j=m_nDim-1;j>=0;j--)delete[] _iCache1[i][j]; delete[] _iCache1[i];}delete[] _iCache1; _iCache1=NULL; }
	if(m_LocalsCache!=NULL){ delete[] m_LocalsCache; m_LocalsCache=NULL; }
	return m_Integral;
}
bool CVMGenLite::GenEvent() {
	m_Weight=0.; m_WeightDebug=0.;
	if((m_Integral<=0.) || (m_Cells==NULL)){ m_nGenEvtFailed++; return false; }
	CVMGenLiteCell* _Cell=m_Cells; tReal _Ileft=0.; tReal _Iright=_Cell->m_Integral; tReal _Ix=_Ileft+(_Iright-_Ileft)*m_rnd->Rnd();
	for(;;) {
		if ((_Cell->m_Sub1==NULL) || (_Cell->m_Sub2==NULL)) { break; }
		tReal _Imid=_Ileft+_Cell->m_Sub1->m_Integral;
		//Note: is there precision lost in I=Sum{left#i->Integral}? //asdf
		//
		//[Sub-cell==0 check]:
		//	(m_Sub1->m_Integral==0.) ==> (_Ix>=_Imid) always
		//	(m_Sub2->m_Integral==0.) ==> (_Ix>=_Imid) is possible
		if((_Ix<_Imid) || (_Cell->m_Sub2->m_Integral==0.)){
			_Iright=_Imid; _Cell=_Cell->m_Sub1;
		}else{
			_Ileft=_Imid; _Cell=_Cell->m_Sub2;
		}
	}
	if(_Cell->m_Integral<=0.){
		if(m_nGenEvtFailed<100)std::cout<<"[GenLite] Event failed. Invalid cell."<<std::endl;
		m_nGenEvtFailed++; return false; //TODO: remove this line after 1/I will be avoided (weight normalization changed)
	}

	tReal fMax=_Cell->m_FuncMax;
	tReal fMaxOrig=fMax; //to debug the 'iterations-limit-achieved' case
	if(fMax<=0.){
		if(m_nGenEvtFailed<100){
			std::cout<<"[GenLite] Event failed. Invalid cell max. value: "<<fMax<<std::endl;
			std::cout<<"[GenLite] Invalid cell integral: "<<_Cell->m_Integral<<std::endl;
			std::cout<<"[GenLite] Invalid cell error: "<<sqrt(fabs(_Cell->m_Variance))*m_fCLFactor<<std::endl;
		}
		m_nGenEvtFailed++; return false; //Internal error
	}
	if(m_fMaxSafetyFactor>0.){ fMax*=m_fMaxSafetyFactor; } //Increase 'fMax' to avoid overlaps
	int nIt=0; bool gGenOk=false;
	//long nItMax=1000000;
	long nItMax=100000;
	tReal rThreshold; if(m_bGenFixThreshold)rThreshold=m_rnd->Rnd();
	tReal fGenSum=0.; tReal val=0.; //to debug the 'iterations-limit-achieved' case
	for(;nIt<nItMax;nIt++) {
		for(int i=0;i<m_nDim;i++) { tReal r=m_rnd->Rnd(); m_GenValues[i]=_Cell->m_VarMin[i]*(1.-r)+_Cell->m_VarMax[i]*r; }
		val=CallFunction(m_GenValues); // TODO: check what's happened if val<0.

		// update generation statistics
		if(m_iGenEvtLoops>=ULONG_MAX) { m_iGenEvtLoops=0; m_iGenEvtCalls=0; /*truncate stats*/ }
		m_iGenEvtLoops++;

		fGenSum+=val;
		if(m_bUpdateMaxValue && (fMax<val)){ //update maximum
			_Cell->m_FuncMax=val; fMax=val; if(m_fMaxSafetyFactor>0.){ fMax*=m_fMaxSafetyFactor; }
		}
		if(!m_bGenFixThreshold)rThreshold=m_rnd->Rnd();
		if(val>=rThreshold*fMax){ gGenOk=true; break; }
	}
	m_nGenEvtIterations[(nIt<100)?nIt:99]++; //debug
	if(!gGenOk){
		if(m_nGenEvtFailed<100){
			std::cout<<"[GenLite] Event failed. Iterations limit achieved."<<std::endl;
			std::cout<<"[GenLite] Average achieved:  "<<fGenSum/nItMax<<std::endl;
			std::cout<<"[GenLite] Average estimated: "<<_Cell->m_Integral/_Cell->m_Volume<<std::endl;
			std::cout<<"[GenLite] Maximum achieved:  "<<fMax<<std::endl;
			std::cout<<"[GenLite] Maximum original:  "<<fMaxOrig<<std::endl;
#ifdef VM_DEBUG_FULL
			/*_Cell->ViewCellPicture();*/ exit(0);
#endif //VM_DEBUG_FULL
		}
		m_nGenEvtFailed++; return false;
	}

	m_Weight=m_Integral;
	m_WeightDebug=m_Weight; if(val>fMax){ m_WeightDebug*=(val/fMax); } //overweight if f>fMax (factor: f/fMax)
	if(m_bGenWeightErrors){
		m_WgtError=m_Weight*m_fCLFactor*(sqrt(fabs(
			_Cell->m_Variance/(_Cell->m_Integral*_Cell->m_Integral)	// Cell error
// Note: No total CS error here, see Notebook (22.2)
//			+ m_Cells->m_Variance/(m_Integral*m_Integral)				// Total cross section error
			)));
	}
// Note: m_WgtError is about 100% here, see Notebook (22.1)
	m_iGenEvtCalls++; // update generation statistics
	return true;
}
bool CVMGenLite::GenEventWgt() {
	m_Weight=0.; m_WeightDebug=0.;
	if((m_Integral<=0.) || (m_Cells==NULL))return false;

	CVMGenLiteCell* _Cell=m_Cells;
	tReal _Ileft=0.; tReal _Iright=_Cell->m_Integral; tReal _Ix=_Ileft+(_Iright-_Ileft)*m_rnd->Rnd();
	for(;;) {
		if ((_Cell->m_Sub1==NULL) || (_Cell->m_Sub2==NULL)) { break; }
		tReal _Imid=_Ileft+_Cell->m_Sub1->m_Integral;
		if(_Ix < _Imid) { _Iright=_Imid; _Cell=_Cell->m_Sub1; } else { _Ileft=_Imid; _Cell=_Cell->m_Sub2; }
	}
	if(_Cell->m_Integral<=0.) return false; //TODO: remove this line after 1/I will be avoided (weight normalization changed)

	//TODO: make more correct generation here ==> update Weight (it's not uniform)!
	int i;
	// Absolutelly random point in cell
	for(i=0;i<m_nDim;i++) { tReal r=m_rnd->Rnd(); m_GenValues[i]=_Cell->m_VarMin[i]*(1.-r)+_Cell->m_VarMax[i]*r; }
	tReal val=CallFunction(m_GenValues); // TODO: check what's happened if var<0.
	m_Weight=m_Integral*val*_Cell->m_Volume/_Cell->m_Integral; m_WeightDebug=m_Weight;
	if(m_bGenWeightErrors) {
		m_WgtError=m_Weight*m_fCLFactor*(sqrt(fabs(
			_Cell->m_Variance/(_Cell->m_Integral*_Cell->m_Integral)	// Cell error
// Note: No total CS error here, see Notebook (22.2)
//			+ m_Cells->m_Variance/(m_Integral*m_Integral)				// Total cross section error
			)));
	}
// Note: m_WgtError is about 100% here, see Notebook (22.1)

	/*
	//Old: Unweighted-like generation (n-1 centers + 1(any random) according to _Ix) -- no function calls here
	int iDim=(int)(m_rnd->Rnd()*m_nDim); if(iDim>=m_nDim)iDim=m_nDim-1;
	for(i=0;i<m_nDim;i++) {
		if (i!=iDim) { m_GenValues[i]=(_Cell->m_VarMin[i]+_Cell->m_VarMax[i])/2.; }
		else {
			if (_Iright>_Ileft) { m_GenValues[i]=(_Cell->m_VarMin[i]*(_Iright-_Ix)+_Cell->m_VarMax[i]*(_Ix-_Ileft))/(_Iright-_Ileft); }
			else { m_GenValues[i]=(_Cell->m_VarMin[i]+_Cell->m_VarMax[i])/2.; }
		}
	}
	m_Weight=1.;
	*/

	//std::cout<<"Cell Integral= "<<_Cell->m_Integral<<std::endl;
	//for(i=0;i<m_nDim;i++)std::cout<<((i>0)?"_":"")<<m_GenValues[i]; std::cout<<std::endl;
	return true;
}
bool CVMGenLite::GenEvtDebug() { //Generate event according to GRID only. No function calls, fast but incorrect.
	m_Weight=0.; m_WeightDebug=0.; int i;
	if((m_Integral<=0.) || (m_Cells==NULL)){ m_nGenEvtFailed++; return false; }

	CVMGenLiteCell* _Cell=m_Cells; tReal _Ileft=0.; tReal _Iright=_Cell->m_Integral; tReal _Ix=_Ileft+(_Iright-_Ileft)*m_rnd->Rnd();
	for(;;) {
		if ((_Cell->m_Sub1==NULL) || (_Cell->m_Sub2==NULL)) { break; }
		tReal _Imid=_Ileft+_Cell->m_Sub1->m_Integral;
		if((_Ix<_Imid)||(_Cell->m_Sub2->m_Integral==0.)){ _Iright=_Imid;_Cell=_Cell->m_Sub1; }else{ _Ileft=_Imid;_Cell=_Cell->m_Sub2; }
	}
	if(_Cell->m_Integral<=0.){
		if(m_nGenEvtFailed<100)std::cout<<"[GenLite] Event failed. Invalid cell."<<std::endl;
		m_nGenEvtFailed++; return false; //TODO: remove this line after 1/I will be avoided (weight normalization changed)
	}

	//Uniform distribution in cell is presumed:
	for(i=0;i<m_nDim;i++) { tReal r=m_rnd->Rnd(); m_GenValues[i]=_Cell->m_VarMin[i]*(1.-r)+_Cell->m_VarMax[i]*r; }
	m_Weight=m_Integral; m_WeightDebug=m_Weight;
	if(m_bGenWeightErrors){ m_WgtError=m_Weight*m_fCLFactor*(sqrt(fabs(_Cell->m_Variance/(_Cell->m_Integral*_Cell->m_Integral)))); }
	return true;
}
double CVMGenLite::GetGenEventEfficiency() { //Efficiency of previous 'GenEvent' calls
	double res=0.;
	if(m_iGenEvtLoops>0){ res=((double)m_iGenEvtCalls)/m_iGenEvtLoops; }
	return res;
}
void CVMGenLite::SaveGrid(const char* fn) {
	if(m_Cells==NULL)return;
	std::ofstream ofs(fn,std::ios::out|std::ios::trunc|std::ios::binary);
	if(ofs) {
		int iVer=3; int i;
#ifdef VM_TREAL_ID
		ofs<<"<"<<iVer<<">["<<sizeof(tReal)<<","<<(int)VM_TREAL_ID<<"]"; //Format version + digits info + type ID
#else
		std::cout<<"Internal WARNING: no 'VM_TREAL_ID' macros defined. No type info available"<<std::endl;
		ofs<<"<"<<iVer<<">["<<sizeof(tReal)<<"]"; //Format version + digits info
#endif
		// Integration-only used parameters
		//m_fMaxWgtFactor
		//tReal tmp=0.;//old ver: m_fStopSafetyFactor
		//Not saved: tReal m_fIntegralExpected;
		//Not saved: (Gen-only) m_bGenWeightErrors, m_bUpdateMaxValue, m_bMergeSkippedCells;
		//Not saved: (Integration-only) m_nFirstOrders, m_SubEvtsFirstOrders

		ofs<<m_fPrecisionReq<<" "<<m_iAxisSelMode<<" "<<m_nSubOrderMin<<" "<<m_nSubOrderMax<<" "<<m_SubEvts<<" "<<(m_bUseEventsSmart?1:0)<<" "<<((m_fMaxWgtFactor>0.)?1:0)<<" "<<m_fMaxWgtFactor<<" "; //<<tmp<<" ";
		ofs<<m_fCLFactor<<" ";
		// Integration statistics
		ofs<<m_nCellGenRepeated<<" "<<m_nMachinePrecFault<<" "<<m_nCellIntegrFault<<" ";
		// Restorable: m_Integral, m_Error, m_nCellsCnt, m_nSOrderMaxActual;
		// Not saved (volatile): m_MaxFinalVarianceCell, m_nMaxFinalVariance;
		// Not saved (set by Init()): m_Volume
		ofs<<m_nDim<<" "; // For checks only
		m_Cells->SaveCell(ofs);

		//Save additional info
		const char* info=GetMetadata();
		ofs<<"{";
		if(info!=NULL){ for(long i=0;i<GEN_META_MAX_SIZE;i++){ char c=info[i]; if(c=='}')c='?'; if(c=='\0')break; ofs.write(&c,sizeof(c)); } }
		ofs<<"}"; //Empty '{}' are requires for correct further reading

		//Save user variables
		int nUserVars=0;
		for(i=0; i<CVMGen_MAXUSERVARS; i++) {
			if(m_UserVarsNames[i]==NULL)break;
			if(nUserVars==0){ofs<<"{";}else{ofs<<";";} nUserVars++;
			ofs<<m_UserVarsNames[i]<<"="<<DBLtR(m_UserVarsVals[i])<<"["; ofs.write((char*)&m_UserVarsVals[i],sizeof(m_UserVarsVals[i])); ofs<<"]";
		}
		if(nUserVars>0){ofs<<"}";}

		//End mark
		ofs<<"E";

		ofs.close();
	}
}
bool CVMGenLite::LoadGrid(const char* fn) {
	std::ifstream ifs(fn,std::ios::in|std::ios::binary); bool bReadOK=true;
	if(ifs) {
		//Clear Generator
		m_MaxFinalVarianceCell=NULL; m_nMaxFinalVariance=-1; // Debug info
		m_nCellsCnt=0; m_nSOrderMaxActual=0; m_nCellGenRepeated=0; m_iGenEvtCalls=0; m_iGenEvtLoops=0; // Reset statistics
		SetMetadata(NULL); // Clear metadata

		if(m_Cells!=NULL){ delete m_Cells; m_Cells=NULL; }
		int iVer=0;
		if(bReadOK) { //Load: header & version
			bool bVerOK=true; char c=0; ifs>>c; if(c!='<')bVerOK=false; if(bVerOK){ ifs>>iVer; if((iVer<1)||(iVer>3))bVerOK=false; }
			if(bVerOK){ ifs>>c; if(c!='>'){ bVerOK=false; } } if(!bVerOK){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected grid format"<<std::endl; bReadOK=false; }
		}
		int nDig=0;
		int iTypeID=0; //compare to 'VM_TREAL_ID'
		if(bReadOK){ //Check digits number
			char c=0; ifs>>c;
			if(c=='[') {
				ifs>>nDig; ifs>>c;
				if(c==','){ ifs>>iTypeID; ifs>>c; }
				if(c!=']'){ std::cout<<"CVMGenLite::LoadGrid() ERROR: invalid grid structure. ']' expected."<<std::endl; bReadOK=false; }
				else {
					if(nDig!=sizeof(tReal)){
						//std::cout<<"LoadGrid ERROR: Can't read grid with "<<nDig<<"-digit numbers. Current precision = "<<sizeof(tReal)<<"-digits."<<std::endl; bReadOK=false;
						std::cout<<"LoadGrid WARNING: loading grid with different precision ("<<nDig<<" bytes numbers. Currently: "<<sizeof(tReal)<<" bytes)."<<std::endl;
					}
				}
			} else {
				std::cout<<"CVMGenLite::LoadGrid() ERROR: Type info expected. Grid file is corrupted."<<std::endl; bReadOK=false;
			}
		}
		//Note: take care of binary load if (nDig!=sizeof(tReal)).
		if(bReadOK) { //Load: Settings & statistics
			ifs>>m_fPrecisionReq>>m_iAxisSelMode>>m_nSubOrderMin>>m_nSubOrderMax>>m_SubEvts;
			int iTmp=0; ifs>>iTmp; m_bUseEventsSmart=(iTmp>0); ifs>>iTmp; bool bWgtRestrictMode=(iTmp>0); ifs>>m_fMaxWgtFactor;
			if(iVer>=3){
				ifs>>m_fCLFactor;
				if(m_fCLFactor<=0.){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected CL-factor loaded ("<<m_fCLFactor<<"). Set to "<<MC_CL_fact_used<<std::endl; m_fCLFactor=MC_CL_fact_used; }
			}else{
				tReal tmp; ifs>>tmp; //old vers: m_fStopSafetyFactor
			}
			ifs>>m_nCellGenRepeated>>m_nMachinePrecFault>>m_nCellIntegrFault;
			ifs>>iTmp; if(iTmp!=m_nDim){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected dimension number or corrupted file (nDim: "<<m_nDim<<", read: "<<iTmp<<")"<<std::endl; bReadOK=false; }
		}
		//Note: statistics must be inited before cells loading -> some values are used by LoadCell (e.g. 'm_SubEvts')
		if(bReadOK) { //Load: Cells
			m_Cells=new CVMGenLiteCell(this,0); bReadOK=m_Cells->LoadCell(ifs,iVer,nDig,iTypeID); if(!bReadOK)std::cout<<"CVMGenLite::LoadGrid(): failed to load cells tree."<<std::endl;
		}
		if((bReadOK) && (iVer>=2)) {
			char c=0; ifs.read(&c,sizeof(c));

			//Load additional info
			if(c=='{'){
				char* buf=new char[GEN_META_MAX_SIZE+2];
				long l=0;
				for(;;l++){
					if(l>=GEN_META_MAX_SIZE){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected size of metadata"<<std::endl; bReadOK=false; break; }
					if((!ifs)||(ifs.eof())){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected end of file while metadata loading"<<std::endl; bReadOK=false; break; }
					ifs.read(&c,sizeof(c)); if(c=='}')break; *(buf+l)=c;
				}
				if(bReadOK && (l>0)){ *(buf+l)='\0'; SetMetadata(buf); c=0; } //Nt: "c=0;" to skip '}' symbol
				delete[] buf;
			}

			//Load user variables
			if(bReadOK){
				if(c==0){ ifs.read(&c,sizeof(c)); }
				if(c=='{'){
					char* buf=new char[100]; int nVarsRead=0; //100: max variable name size
					for(;;){
						long l=0;
						for(;;l++){
							if(l>=99){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected size of user variable"<<std::endl; bReadOK=false; break; }
							if((!ifs)||(ifs.eof())){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected end of file while user variables loading"<<std::endl; bReadOK=false; break; }
							ifs.read(&c,sizeof(c)); if((c=='=')||(c==';')||(c=='}'))break; *(buf+l)=c;
						}
						if((c=='}')&&(nVarsRead==0)&&(l==0))break; //empty {}
						if((c==';')||(c=='}')){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected '"<<c<<"' symbol while user variables loading"<<std::endl; bReadOK=false; }
						if(!bReadOK)break;
						// (c=='=') here
						*(buf+l)='\0'; if(l<=0){ std::cout<<"CVMGenLite::LoadGrid() ERROR: empty user variable name red"<<std::endl; bReadOK=false; break; }
						double dVal; ifs>>dVal;
						ifs.read(&c,sizeof(c)); if(c!='['){ std::cout<<"CVMGenLite::LoadGrid() ERROR reading user variables list"<<std::endl; bReadOK=false; break; }
						tReal val; if(!CVMGenLite::LoadBinaryVal(ifs,val,nDig,iTypeID)){ std::cout<<"CVMGenLite::LoadGrid() ERROR reading user variables list"<<std::endl; bReadOK=false; break; }
						ifs.read(&c,sizeof(c)); if(c!=']'){ std::cout<<"CVMGenLite::LoadGrid() ERROR reading user variables list"<<std::endl; bReadOK=false; break; }
						AddUserVariable(buf,val); nVarsRead++;
						std::cout<<"[restored] User variable #"<<nVarsRead<<" loaded: "<<buf<<" = "<<val<<std::endl;
						ifs.read(&c,sizeof(c)); if(c=='}')break; if(c!=';'){ std::cout<<"CVMGenLite::LoadGrid() ERROR: unexpected '"<<c<<"' symbol while user variables loading"<<std::endl; bReadOK=false; break; }
					}
					delete[] buf;
					c=0; //Nt: "c=0;" to skip '}' symbol
				}
			}

			//Check 'end' symbol
			if(bReadOK){ if(c==0){ ifs.read(&c,sizeof(c)); } if(c!='E'){ std::cout<<"CVMGenLite::LoadGrid() ERROR: can't find 'END' symbol. Grid file is corrupt."<<std::endl; bReadOK=false; } }
		}
		ifs.close();
		if(bReadOK) { // Init additional fields
			m_Integral=m_Cells->m_Integral;
			m_Error=m_fCLFactor*sqrt(fabs(m_Cells->m_Variance)); if(m_Cells->m_Variance<0.)m_Error=-m_Error;//-1 to mark error
			//View restored (not loaded) parameters
			std::cout<<"--- Integration grid loaded ---"<<std::endl;
			std::cout<<"[restored] m_Integral= "<<m_Integral<<std::endl;
			std::cout<<"[restored] m_Error= "<<m_Error<<std::endl;
			std::cout<<"[restored] m_nCellsCnt= "<<m_nCellsCnt<<std::endl;
			std::cout<<"[restored] m_nSOrderMaxActual= "<<m_nSOrderMaxActual<<std::endl;
		}
	} else { std::cout<<"CVMGenLite::LoadGrid() FAILED to open grid file."<<std::endl; bReadOK=false; }
	if(!bReadOK)std::cout<<"CVMGenLite::LoadGrid() FAILED."<<std::endl;
	return bReadOK;
}
/*
*	MergeWithGrid(): combine current grid with other one.
*	The other grid MUST have similar cell-by-cell structure.
*	The usual way to obtain similar grids:
*		- Integrate in 'm_bUsePreviousGrid && !m_bMergeSkippedCells' mode with different RND seeds.
*
*	The values are mixed in proportion to 'm_SubEvts', m_SubEvts is updated with other->m_SubEvts.
*
*	Warning: current GRID may be destroyed if operation failed.
*/
bool CVMGenLite::MergeWithGrid(const char* fn) {
	//
	bool bMergeOK=true;
	CVMGenLite* oGen = new CVMGenLite();
	if(!oGen->Init(m_nDim,m_VarMin,m_VarMax,m_pFunc)){ std::cout<<"CVMGenLite::MergeWithGrid() ERROR creating other grid object"<<std::endl; bMergeOK=false; }
	if(bMergeOK){ if(!oGen->LoadGrid(fn)){ std::cout<<"CVMGenLite::MergeWithGrid() ERROR loading other grid object"<<std::endl; bMergeOK=false; } }
	if(!bMergeOK) { if(oGen!=NULL){delete oGen; oGen=NULL;} return false; }

	//
	// Compartibility check
	if((m_nSOrderMaxActual != oGen->m_nSOrderMaxActual) || (m_nCellsCnt != oGen->m_nCellsCnt)) {
		std::cout<<"CVMGenLite::MergeWithGrid() ERROR: loaded grid is not compartible"<<std::endl; bMergeOK=false;
	}
	if(!bMergeOK) { if(oGen!=NULL){delete oGen; oGen=NULL;} return false; }
	if(oGen->m_SubEvts<=0){ if(oGen!=NULL){delete oGen; oGen=NULL;} return true; } //Nothing to merge with

	//
	//[Delete:] bool bLog=true;
	// Merge
	CVMGenLiteCell** arrCells1=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells1[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	CVMGenLiteCell** arrCells2=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells2[0]=oGen->m_Cells;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell1=arrCells1[i]; CVMGenLiteCell* _Cell2=arrCells2[i];  if((_Cell1==NULL)||(_Cell2==NULL)){ bMergeOK=false; /*error*/ break; }
		if((arrCellN[i]==0) && (_Cell1->m_iSplitAxis<0)) { //first access to 'final' cell
			if(_Cell2->m_iSplitAxis>=0){ bMergeOK=false; break; }

			//Old: EvtsSum=o_SubEvts+m_SubEvts; tReal f1=((tReal)m_SubEvts)/nEvtsSum; tReal f2=((tReal)o_SubEvts)/nEvtsSum;
			long n1=_Cell1->m_nEvts; long n2=_Cell2->m_nEvts; long nS=n1+n2; if(nS<=0){ bMergeOK=false; /*Error in grid*/ break; }
			tReal f1=((double)n1)/nS; tReal f2=((double)n2)/nS;
			tReal fI1=_Cell1->m_Integral; tReal fI2=_Cell2->m_Integral; tReal fIAve=f1*fI1+f2*fI2; _Cell1->m_Integral=fIAve;
			//[old] sd^2 = Sum[F^2]*V^2/N-subI*subI; ==> Sum[F^2]*V^2=(sd^2+subI^2)*N
			//[old] sd^2 = (Sum[F1^2]+Sum[F2^2])/(N1+N2)*V^2-(aveI*aveI) = ((sd1^2+I1^2)*N1+(sd2^2+I2^2)*N2)/(N1+N2)-(aveI*aveI)
			//[old]  = |ki=Ni/(N1+N2)| = sd1^2*k1 + sd2^2*k2 + I1^2*k1+I2^2*k2-(aveI*aveI)
			_Cell1->m_Variance =
				_Cell1->m_Variance*f1*f1 + _Cell2->m_Variance*f2*f2 +
				(fI1*fI1*f1 + fI2*fI2*f2 - fIAve*fIAve)/nS;
			if(_Cell2->m_FuncMax>_Cell1->m_FuncMax)_Cell1->m_FuncMax=_Cell2->m_FuncMax;
			_Cell1->m_nEvts+=n2;
			//[Delete:] if(bLog)std::cout<<"[Log-Merge] n1="<<n1<<", n2="<<n2<<", nS="<<nS<<std::endl;
			//[Delete:] if(bLog)std::cout<<"[Log-Merge] f1="<<f1<<", f2="<<f2<<std::endl;
			//[Delete:] if(bLog)bLog=false;
		}
		if(arrCellN[i]==0){ //sub-1
			if((_Cell1->m_Sub1!=NULL)&&(_Cell2->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells1[i]=_Cell1->m_Sub1; arrCells2[i]=_Cell2->m_Sub1; arrCellN[i]=0; continue; }
			else if((_Cell1->m_Sub1!=NULL)||(_Cell2->m_Sub1!=NULL)){ bMergeOK=false; /*error*/ break; }
		}
		if(arrCellN[i]<2){ //sub-2
			if((_Cell1->m_Sub2!=NULL)&&(_Cell2->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells1[i]=_Cell1->m_Sub2; arrCells2[i]=_Cell2->m_Sub2; arrCellN[i]=0; continue; }
			else if((_Cell1->m_Sub2!=NULL)||(_Cell2->m_Sub2!=NULL)){ bMergeOK=false; /*error*/ break; }
		}
		i--; if(i<0)break;
	}
	delete[] arrCells1; delete[] arrCells2; delete[] arrCellN;
	if(!bMergeOK) {
		std::cout<<"CVMGenLite::MergeWithGrid() ERROR: loaded grid is not compartible. Current grid is DESTROYED!!!"<<std::endl;
	}

	// Update intermediate cells & global variables
	if(bMergeOK) {
		m_Cells->UpdateResults();
		m_Integral = m_Cells->m_Integral;
		m_Error = m_fCLFactor*sqrt(fabs(m_Cells->m_Variance));
		m_SubEvts += oGen->m_SubEvts;
	}

	if(oGen!=NULL){delete oGen; oGen=NULL;}
	return bMergeOK;
}
/*
*	ApplyGridFactor() is used to:
*	1. Co-scale grid if function is scaled.
*	2. Start integration from pre-created but empty grid (f==0.).
*/
void CVMGenLite::ApplyGridFactor(double f, bool bAffectIExpected) {
	m_Integral*=f; CVMGenLite::m_Error*=f; if(bAffectIExpected){ CVMGenLite::m_fIntegralExpected*=f; }
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break;//Error
		if(arrCellN[i]==0) { //first access to cell
			_Cell->m_Integral*=f; _Cell->m_Variance*=(f*f); _Cell->m_FuncMax*=f;
		}
		if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
		if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;
}
void CVMGenLite::ViewGrid(int dim1, int dim2) {
#ifdef CVMHistLite_H_INCLUDED_
	const char* title=((m_Title!=NULL)?m_Title:"");
	if((dim1>=m_nDim) || (dim2>=m_nDim) || (dim1==dim2) || (m_nDim< 2))return;
	CVMFreeDrawing fdraw("Grid"); CVMScatLite plotI("Int"); plotI.SetAutoAverage(true,128,128); CVMScatLite plotVariance("Error^2"); plotVariance.SetAutoAverage(true,128,128);
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL) break;//Error
		if(arrCellN[i]==0) { //first access to cell
			tReal xC=(_Cell->m_VarMin[dim1]+_Cell->m_VarMax[dim1])/2.; tReal yC=(_Cell->m_VarMin[dim2]+_Cell->m_VarMax[dim2])/2.;
			if ((_Cell->m_iSplitAxis == dim1) || (_Cell->m_iSplitAxis == dim2)) {
				tReal x1,x2,y1,y2;
				if (_Cell->m_iSplitAxis==dim1) { x1=xC; x2=xC; y1=_Cell->m_VarMin[dim2]; y2=_Cell->m_VarMax[dim2]; }
				else if (_Cell->m_iSplitAxis==dim2) { x1=_Cell->m_VarMin[dim1]; x2=_Cell->m_VarMax[dim1]; y1=yC; y2=yC; }
				fdraw.MoveTo(DBLtR(x1),DBLtR(y1)); fdraw.LineTo(DBLtR(x2),DBLtR(y2));
			} else {
				plotI.Fill(DBLtR(xC),DBLtR(yC),DBLtR(_Cell->m_Integral/_Cell->m_Volume));
				plotVariance.Fill(DBLtR(xC),DBLtR(yC),DBLtR(_Cell->m_Variance));
			}
		}

		if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
		if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;
	plotI.Average(128,128);
	plotVariance.Average(128,128); char buf[100];
	CVMHistLiteView view(&fdraw); sprintf(buf,"%s_Grid.eps",title); view.SaveAsEPS(buf);
	view.AddPlotToView(&plotVariance); view.AddPlotToView(&plotI); sprintf(buf,"%s_Grid.Mix.eps",title); view.m_bShiftMergedPoints=false; view.SaveAsEPS(buf);
	CVMHistLiteView viewI(&plotI); sprintf(buf,"%s_Grid.I.eps",title); viewI.SaveAsEPS(buf);
	CVMHistLiteView viewVariance(&plotVariance); sprintf(buf,"%s_Grid.Variance.eps",title); viewVariance.SaveAsEPS(buf);
#endif //CVMHistLite_H_INCLUDED_
}
void CVMGenLite::ViewCellsLog(int iOrderMin, int iOrderMax) {
#ifdef CVMHistLite_H_INCLUDED_
	if(iOrderMin<0)iOrderMin=0; if(iOrderMax<0)iOrderMax=m_nSOrderMaxActual;
	std::cout<<"=== Cells log for "<<iOrderMin<<" to "<<iOrderMax<<" orders ==="<<std::endl;
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL) break;//Error
		if(arrCellN[i]==0) { //first access to cell
			if(i>=iOrderMin){
				for(int i2=0;i2<(i-iOrderMin);i2++)std::cout<<".";
				std::cout<<"I= "<<_Cell->m_Integral<<" +/- "<<m_fCLFactor*sqrt(fabs(_Cell->m_Variance))<<std::endl;
			}
		}
		if(i<iOrderMax) {
			if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
			if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		}
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;
	std::cout<<"=== Cells log end ==="<<std::endl;
#endif //CVMHistLite_H_INCLUDED_
}
void CVMGenLite::ViewCellsPlot(int id) {
#ifdef CVMHistLite_H_INCLUDED_
	const char* title=((m_Title!=NULL)?m_Title:""); char buf[100];
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	//
	switch(id){
	case 0: //id: 0(cells order)
		{
			CVMHistLite histOrd("Cell Order",m_nSOrderMaxActual,0.5,m_nSOrderMaxActual+0.5);
			CVMHistLite histOrdW("Weight/Order",m_nSOrderMaxActual,0.5,m_nSOrderMaxActual+0.5);
			CVMHistLite histOrdA("fAve/Order",m_nSOrderMaxActual,0.5,m_nSOrderMaxActual+0.5);
			CVMHistLite histOrdM("fMax2fAve/Order",m_nSOrderMaxActual,0.5,m_nSOrderMaxActual+0.5);
			for(long i=0;;) { // Iterate cells
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0) && (_Cell->m_iSplitAxis<0)) { //first access to 'final' cell
					histOrd.Fill(_Cell->m_nSubOrder);
					if((_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)){
						double fWgt=DBLtR(_Cell->m_Integral/((m_Integral>0)?m_Integral:1.)); histOrdW.Fill(_Cell->m_nSubOrder,fWgt);
						double fAve=DBLtR(_Cell->m_Integral/_Cell->m_Volume); histOrdA.Fill(_Cell->m_nSubOrder,fAve); histOrdM.Fill(_Cell->m_nSubOrder,DBLtR(_Cell->m_FuncMax)/fAve);
					}
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			histOrdW.Normalize(histOrd.GetIntegral());
			CVMHistLiteView view(&histOrd); view.AddPlotToView(&histOrdW); view.m_bLogAxisY=true; view.SetPlotBins(true); view.m_bPrintNEvts=true; view.m_iLegendPos=4; view.m_bLegendOverlapped=true; sprintf(buf,"%s_CellOrder.eps",title); view.SaveAsEPS(buf);
			histOrdM.AverageBinValues();
			CVMHistLiteView viewM(&histOrdM); viewM.m_bLogAxisY=true; viewM.SetPlotBins(true); viewM.m_bLegendOverlapped=true; sprintf(buf,"%s_CellFMax2FAvePerOrder.eps",title); viewM.SaveAsEPS(buf);
			//histOrdA.Normalize(histOrd.GetIntegral());
			CVMHistLiteView viewA(&histOrdA); viewA.m_bLogAxisY=true; viewA.SetPlotBins(true); viewA.m_bPrintNEvts=true; viewA.m_iLegendPos=3; viewA.m_bLegendOverlapped=true; sprintf(buf,"%s_CellFAvePerOrder.eps",title); viewA.SaveAsEPS(buf);
		} break;
	case 1: //id: 1(fMax/fAve)
	case 2: //id: 2(fMax/fAve*cellWeight)
		{
			bool bToCellWeight=(id==2);
			int iOrdThr=((m_nFirstOrders>0)?m_nFirstOrders:40);
			sprintf(buf,"Cell fMax/fAve%s",(bToCellWeight?"*cWgt":"")); CVMHistLite hist(buf,100,1,10000,true);
			sprintf(buf,"Order < %ld",iOrdThr); CVMHistLite histL40(buf,100,1,10000,true);
			sprintf(buf,"Order >=%ld",iOrdThr); CVMHistLite histG40(buf,100,1,10000,true);
			for(long i=0;;) { // Iterate cells
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0) && (_Cell->m_iSplitAxis<0)){ //first access to 'final' cell
					if((_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)){
						double fMax2fAve=DBLtR(_Cell->m_FuncMax/(_Cell->m_Integral/_Cell->m_Volume));
						double wgt=1.; if(bToCellWeight)wgt*=DBLtR(_Cell->m_Integral/((m_Integral>0)?m_Integral:1.));
						hist.Fill(fMax2fAve,wgt); if(i<iOrdThr){ histL40.Fill(fMax2fAve,wgt); }else{ histG40.Fill(fMax2fAve,wgt); }
					}
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			CVMHistLiteView view(&hist); view.AddPlotToView(&histL40); view.AddPlotToView(&histG40); view.m_bLogAxisY=true; view.SetPlotBins(true); view.m_bPrintNEvts=true; view.m_bLegendOverlapped=true; sprintf(buf,"%s_fMax2Ave%s.eps",title,(bToCellWeight?"2cWgt":"")); view.SaveAsEPS(buf);
		} break;
	case 3: //id==3: ave,max(fMax/fAve)[cellWghtS]+Ncells[cellWghtS]
		{
			double wgtMax=-1.; double wgtMin=-1.; long i;
			arrCells[0]=m_Cells; arrCellN[0]=0;
			for(i=0;;) { // Calc. 'wgtMax'
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0)&&(_Cell->m_iSplitAxis<0)&&(_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)) {
					double wgt=DBLtR(_Cell->m_Integral/((m_Integral>0)?m_Integral:1.)); if(wgt>wgtMax)wgtMax=wgt; if((wgt>0.)&&((wgt<wgtMin)||(wgtMin<0.)))wgtMin=wgt;
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			if(wgtMax<=0.){ std::cout<<"Warning: can't create fMax2Ave-vs-Wgt plot. WgtMax=0"<<std::endl; break; }
			//int iOrdThr=((m_nFirstOrders>0)?m_nFirstOrders:40);
			CVMHistLite histAW("Ave[fMax/fAve]/CellWgt",100,wgtMin,wgtMax*1.1,true);
			CVMHistLite histMW("Max[fMax/fAve]/CellWgt",100,wgtMin,wgtMax*1.1,true);
			CVMHistLite histNW("NCells/Wgt (scaled)",100,wgtMin,wgtMax*1.1,true);
			arrCells[0]=m_Cells; arrCellN[0]=0;
			for(i=0;;) { // Iterate cells
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0) && (_Cell->m_iSplitAxis<0)){ //first access to 'final' cell
					if((_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)){
						double wgt=DBLtR(_Cell->m_Integral/((m_Integral>0)?m_Integral:1.));
						double fMax2fAve=DBLtR(_Cell->m_FuncMax/(_Cell->m_Integral/_Cell->m_Volume));
						histAW.Fill(wgt,fMax2fAve); histMW.FillMax(wgt,fMax2fAve); histNW.Fill(wgt,1.);
						//if(i<iOrdThr){ histL40.Fill(fMax2fAve,wgt); }else{ histG40.Fill(fMax2fAve,wgt); }
					}
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			histAW.AverageBinValues();
			histNW.Normalize(histMW.GetIntegral());
			CVMHistLiteView view(&histAW); view.AddPlotToView(&histMW); view.AddPlotToView(&histNW);
			view.SetPlotBins(true); view.m_bLogAxisY=true; view.m_bLegendOverlapped=true; //view.m_bPrintNEvts=true;
			sprintf(buf,"%s_fMax2Ave-vs-CellWgt.eps",title); view.SaveAsEPS(buf);
		} break;
	case 4: //id==4: NCells/CellRelativeError + NCells/CellAbsError(scaled)
		{
			double errRelMax=-1.; double errRelMin=-1.; double errAbsMax=-1.; double errAbsMin=-1.; long i;
			arrCells[0]=m_Cells; arrCellN[0]=0;
			for(i=0;;) { // Calc. 'wgtMax'
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0)&&(_Cell->m_iSplitAxis<0)&&(_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)) {
					long N=((_Cell->m_nEvts>0)?_Cell->m_nEvts:m_SubEvts); //TODO: insert 'first-N' here!
					if(N>0){
						double errA=DBLtR(m_fCLFactor*sqrt(fabs(_Cell->m_Variance))); if(errA>errAbsMax)errAbsMax=errA; if((errA>0.)&&((errA<errAbsMin)||(errAbsMin<0.)))errAbsMin=errA;
						double errR=errA/DBLtR(_Cell->m_Integral); if(errR>errRelMax)errRelMax=errR; if((errR>0.)&&((errR<errRelMin)||(errRelMin<0.)))errRelMin=errR;
					}
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			//int iOrdThr=((m_nFirstOrders>0)?m_nFirstOrders:40);
			CVMHistLite histErrR("NCells/error Rel",100,errRelMin*0.99,errRelMax*1.01,true);
			CVMHistLite histErrRWgt("Wgt/error Rel (scaled)",100,errRelMin*0.99,errRelMax*1.01,true);
			CVMHistLite histErrA("NCells/error Abs",100,errAbsMin*0.99,errAbsMax*1.01,true);
			CVMHistLite histErrAWgt("Wgt/error Abs (scaled)",100,errAbsMin*0.99,errAbsMax*1.01,true);
			arrCells[0]=m_Cells; arrCellN[0]=0;
			for(i=0;;) { // Iterate cells
				CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break; //Error
				if((arrCellN[i]==0) && (_Cell->m_iSplitAxis<0)){ //first access to 'final' cell
					if((_Cell->m_Integral>0.)&&(_Cell->m_Volume>0.)){
						long N=((_Cell->m_nEvts>0)?_Cell->m_nEvts:m_SubEvts); //TODO: insert 'first-N' here!
						if(N>0){
							double wgt=DBLtR(_Cell->m_Integral/((m_Integral>0)?m_Integral:1.));
							double errA=DBLtR(m_fCLFactor*sqrt(fabs(_Cell->m_Variance))); double errR=errA/DBLtR(_Cell->m_Integral);
							histErrA.Fill(errA,1.); histErrAWgt.Fill(errA,wgt); histErrR.Fill(errR,1.); histErrRWgt.Fill(errR,wgt);
						}
					}
				}
				if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
				if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
				i--; if(i<0)break;
			}
			histErrRWgt.Normalize(histErrR.GetIntegral());
			{ CVMHistLiteView view(&histErrR); view.AddPlotToView(&histErrRWgt); view.SetPlotBins(true); view.m_bLegendOverlapped=true; view.m_bPrintNEvts=true; sprintf(buf,"%s_ErrorsRelPerCell.eps",title); view.SaveAsEPS(buf); }
			histErrAWgt.Normalize(histErrA.GetIntegral());
			{ CVMHistLiteView view(&histErrA); view.AddPlotToView(&histErrAWgt); view.SetPlotBins(true); view.m_bLegendOverlapped=true; view.m_bPrintNEvts=true; view.m_iLegendPos=3; sprintf(buf,"%s_ErrorsAbsPerCell.eps",title); view.SaveAsEPS(buf); }
			//view.m_bLogAxisY=true;
		} break;
	}
	delete[] arrCells; delete[] arrCellN;
#endif //CVMHistLite_H_INCLUDED_
}
void CVMGenLite::DropOutCells(int iOrderMin, int iOrderMax) { //for debug only
	if(iOrderMax<0)iOrderMax=m_nSOrderMaxActual+1;
	int nSOrderMaxNew=0;
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break;//Error
		// Modify sub-cells first
		if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
		if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		// Modify cell
		bool bEmptyCell=false;
		if (_Cell->m_iSplitAxis<0) {
			if(_Cell->m_nSubOrder<iOrderMin){ bEmptyCell=true; }
			if(_Cell->m_nSubOrder>iOrderMax){ bEmptyCell=true; }
		} else {
			if ((_Cell->m_Sub1 != NULL) && (_Cell->m_Sub2 != NULL)) {
				// Update integral
				if ((_Cell->m_Sub1->m_Integral==0.) && (_Cell->m_Sub2->m_Integral==0.) &&
					(_Cell->m_Sub1->m_Variance==0.) && (_Cell->m_Sub2->m_Variance==0.)) {
					_Cell->m_iSplitAxis=-1; delete _Cell->m_Sub1; _Cell->m_Sub1=NULL; delete _Cell->m_Sub2; _Cell->m_Sub2=NULL; bEmptyCell=true;
				} else {
					//Note: this can be done later using 'm_Cells->UpdateResults()'
					_Cell->m_Integral=_Cell->m_Sub1->m_Integral+_Cell->m_Sub2->m_Integral;
					_Cell->m_Variance=_Cell->m_Sub1->m_Variance+_Cell->m_Sub2->m_Variance;
				}
			} else { std::cout<<"[err]CVMGenLite::DropOutCells: unexpected case."<<std::endl; }
		}
		if(bEmptyCell) { _Cell->m_Integral=0.; _Cell->m_Variance=0.; }
		if ((!bEmptyCell) && (nSOrderMaxNew<_Cell->m_nSubOrder))nSOrderMaxNew=_Cell->m_nSubOrder;
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;

	// Update cells (if not updated earlier)
	m_Cells->UpdateResults(); //The 'F-max' values are only updated here

	// Update generator
	m_Integral=m_Cells->m_Integral;
	m_Error=m_fCLFactor*sqrt(fabs(m_Cells->m_Variance)); if(m_Cells->m_Variance<0.)m_Error=-m_Error;//-1 to mark error
	m_nSOrderMaxActual=nSOrderMaxNew;
}
void CVMGenLite::DropOutCellsPrec(tReal fPrecision, bool bDropGoodCells) { //for debug only
	if(fPrecision<0.){ std::cout<<"[ERROR] CVMGenLite::DropOutCellsPrec(): invalid parameter."<<std::endl; return; }
	tReal fPrecCache=m_fPrecisionReq; m_fPrecisionReq=fPrecision;
	if(m_fIntegralExpected<=0.){ m_fIntegralExpected=m_Integral; } //Used by 'IsPrecisionOK()'

	int nSOrderMaxNew=0;
	long nCellsDropped=0;
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break;//Error
		// Modify sub-cells first
		if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
		if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		// Modify cell
		bool bDropCell=false;
		if (_Cell->m_iSplitAxis<0) {
			if(_Cell->IsPrecisionOK()){ if(bDropGoodCells)bDropCell=true; }else{ if(!bDropGoodCells)bDropCell=true; }
			if(bDropCell)nCellsDropped++; 
		} else {
			if ((_Cell->m_Sub1 != NULL) && (_Cell->m_Sub2 != NULL)) {
				// Update integral (drop out zero cells)
				if ((_Cell->m_Sub1->m_Integral==0.) && (_Cell->m_Sub2->m_Integral==0.) && (_Cell->m_Sub1->m_Variance==0.) && (_Cell->m_Sub2->m_Variance==0.)) {
					_Cell->m_iSplitAxis=-1; delete _Cell->m_Sub1; _Cell->m_Sub1=NULL; delete _Cell->m_Sub2; _Cell->m_Sub2=NULL; bDropCell=true;
				} else {
					//Note: this can be done later using 'm_Cells->UpdateResults()'
					_Cell->m_Integral=_Cell->m_Sub1->m_Integral+_Cell->m_Sub2->m_Integral; _Cell->m_Variance=_Cell->m_Sub1->m_Variance+_Cell->m_Sub2->m_Variance;
				}
			} else { std::cout<<"[err]CVMGenLite::DropOutCellsPrec(): unexpected case."<<std::endl; }
		}
		if(bDropCell) { _Cell->m_Integral=0.; _Cell->m_Variance=0.; }
		if ((!bDropCell) && (nSOrderMaxNew<_Cell->m_nSubOrder))nSOrderMaxNew=_Cell->m_nSubOrder;
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;

	// Update cells (if not updated earlier)
	m_Cells->UpdateResults(); //The 'F-max' values are only updated here

	// Update generator
	m_Integral=m_Cells->m_Integral;
	m_Error=m_fCLFactor*sqrt(fabs(m_Cells->m_Variance)); if(m_Cells->m_Variance<0.)m_Error=-m_Error;//-1 to mark error
	m_nSOrderMaxActual=nSOrderMaxNew;

	// Restore parameter
	m_fPrecisionReq=fPrecCache;

	std::cout<<"CVMGenLite::DropOutCellsPrec() finished. "<<nCellsDropped<<" cells dropped out."<<std::endl;
}
void CVMGenLite::GetMaxSplittingOrders(long* iMaxOrders) { //for debug only
	if(iMaxOrders==NULL)return;
	long n;
	for(n=0;n<m_nDim;n++){ iMaxOrders[n]=0; }
	CVMGenLiteCell** arrCells=new CVMGenLiteCell*[m_nSOrderMaxActual+1]; arrCells[0]=m_Cells; int* arrCellN=new int[m_nSOrderMaxActual+1]; arrCellN[0]=0;
	for(long i=0;;) { // Iterate cells
		CVMGenLiteCell* _Cell=arrCells[i]; if(_Cell==NULL)break;//Error
		if(arrCellN[i]==0) { //first access to cell
			for(n=0;n<m_nDim;n++){ if(iMaxOrders[n]<_Cell->m_nDimOrder[n])iMaxOrders[n]=_Cell->m_nDimOrder[n]; }
		}
		if((arrCellN[i]==0)&&(_Cell->m_Sub1!=NULL)){ arrCellN[i]=1; i++; arrCells[i]=_Cell->m_Sub1; arrCellN[i]=0; continue; }
		if((arrCellN[i]<2)&&(_Cell->m_Sub2!=NULL)){ arrCellN[i]=2; i++; arrCells[i]=_Cell->m_Sub2; arrCellN[i]=0; continue; }
		i--; if(i<0)break;
	}
	delete[] arrCells; delete[] arrCellN;
}
const char* CVMGenLite::CLASS_NAME = "CVMGenLite";
const char* CVMGenLite::ClassName() { return CLASS_NAME; }

/*
*	--- Library function ---
*	nDig==0 means nDim==sizeof(tReal)
*	iTypeID==0 means unknown
*	Note: iTypeID is used to separate 16-bytes 'long double' and 'DD' types.
*	Note: 'long double' of size>8 may only be red by the same type.
*/
bool CVMGenLite::LoadBinaryVal(std::istream& ifs, tReal& res, int nDig, int iTypeID) {
	if(iTypeID==0){ //Adjust 'iTypeID' to simplify further checks
		if(nDig==8)iTypeID=VM_TREAL_DBL;
		else if(nDig==16)iTypeID=VM_TREAL_DD;
		else if(nDig==32)iTypeID=VM_TREAL_QD;
		else if((nDig>=8)&&(nDig<=16))iTypeID=VM_TREAL_LONGDBL;
	}

	bool bLoadOk=true;
	if( (nDig==0) || ((nDig==sizeof(res))&&(iTypeID==VM_TREAL_ID)) ){
		ifs.read((char*)&res, sizeof(res));
	}else{
		if( (nDig==8) && (iTypeID==VM_TREAL_DBL) ){
			double tmp; ifs.read((char*)&tmp, sizeof(tmp)); res=tmp;
		}else if( (nDig==sizeof(long double)) && (iTypeID==VM_TREAL_LONGDBL)) {
			long double tmp; ifs.read((char*)&tmp, sizeof(tmp)); res=tmp;
		}else if( (nDig==16) && (iTypeID==VM_TREAL_DD) ){
#ifdef dd_real_DEFINED
			dd_real tmp; ifs.read((char*)&tmp, sizeof(tmp));
			if((sizeof(tReal)==8) || (VM_TREAL_ID==VM_TREAL_LONGDBL)){ res=to_double(tmp); } //to 'double' or 'long double'
			else if(sizeof(tReal)==32){ res=tmp; } //to 'qd'
			else{bLoadOk=false;}
#else
			bLoadOk=false;
#endif //dd_real_DEFINED
		}else if( (nDig==32) && (iTypeID==VM_TREAL_QD) ){
#ifdef qd_real_DEFINED
			qd_real tmp; ifs.read((char*)&tmp, sizeof(tmp));
			if((sizeof(tReal)==8) || (VM_TREAL_ID==VM_TREAL_LONGDBL)){ res=to_double(tmp); } //to 'double' or 'long double'
			//TODO: add conversion to 'dd_real'
			else{bLoadOk=false;}
#else
			bLoadOk=false;
#endif //qd_real_DEFINED
		}
		if(!bLoadOk){
			std::cout<<"Error loading number. Unable to read the real*"<<nDig<<" type (type id: "<<iTypeID<<")"<<std::endl;
			//exit(0); //halt program (or skip nDim bytes in file?)
		}
	}
	return bLoadOk;
}

/////////////////////////////////////////////////////////////////////// CVMGenVegas
#ifndef CVMGen_NO_VEGAS
CVMGenVegas::CVMGenVegas() : CVMGen() {
	m_Chi2It=0.; m_StdDevIt=0.;
	m_SI=0.; m_SI2=0.; m_SWGT=0.; m_SCHI=0.;
	for(int i=0;i<CVMVegas_AXBINMAX;i++){ for(int j=0;j<CVMGen_MAXDIM;j++){m_spGrid[i][j]=0.;} } m_nAxisParts=0; m_nIt=0;
}
/*
*	BY G.P.  LEPAGE  SEPT 1976/ (REV) APR 1978
*	-FTN5 VERSION 21-8-1984
*	-LAST MODIFICATIONS HJM 7/12/89
*	AUTHOR : G. P. LEPAGE
*/
tReal CVMGenVegas::Integrate(double fPrecision) { return Integrate(fPrecision,false,false,100000,1); }
/*
*	Note: one should not use (bUseOldGrid,bUseOldRes) options until MDS behaviour will be clarified.
*/
tReal CVMGenVegas::Integrate(double fPrecision, bool bUseOldGrid, bool bUseOldRes, long nCalls, int nItMax) {
	if ((m_nDim<=0) || (m_pFunc==NULL)) return m_Integral; // not inited
	if (nItMax<=0) { std::cout<<"VEGAS called with 0 iterations. Stopped."<<std::endl; return m_Integral; }
	if ((fPrecision<=0) || (fPrecision>1.)) fPrecision=0.01; //default
	if ((bUseOldRes)&&(!bUseOldGrid)) bUseOldGrid=true; //bUseOldRes=false;
	int i,j; long k;

	// Parameters
	//fPrecision; //			-->ACC
	//CVMGen_MAXDIM;//			-->NDIMX
	//CVMVegas_AXBINMAX//		-->NDMX
	//nItMax;//max. iterations	-->ITMX ('INT-OPT-NC/CC' in Djangoh)
	//nCalls;//number of function evaluations (actual calls number will be 'nCallsAct') -->NCALL ('INT-POINTS' in Djangoh)
	int nLogMode=3;//			-->NPRN
	//IT //TODO: why param?
	//m_SI; m_SI2; m_SWGT; m_SCHI;//Sub-results to allow 'bUseOldRes' option -->SI,SI2,SWGT,SCHI;
	//m_nIt;//iteration number	-->IT

	// Output:
	//m_Integral,m_Error,m_Chi2It,m_StdDevIt;											-->COMMON/VGRES/S1,S2,S3,S4
	//m_spGrid[][]; //Actual subdivision of N-dimensional coordinate space				-->XI
	//m_nAxisParts; //Number of intervals per coordinate axis from VEGAS sub-division	-->NDO

	// Locals & others
	//m_GenValues[];//			-->X[CVMGen_MAXDIM]
	//m_VarMax[],m_VarMin[];//	-->XL[CVMGen_MAXDIM],XU[CVMGen_MAXDIM]
	tReal _VarDel[CVMGen_MAXDIM];// to keep (m_VarMax[]-m_VarMin[])	-->DX[CVMGen_MAXDIM]
	tReal spGridI[CVMVegas_AXBINMAX]; //Sub-part of m_spGrid for local use (TODO: aviod it, see code) -->XIN
	tReal R[CVMVegas_AXBINMAX],DT[CVMGen_MAXDIM];
	long IA[CVMGen_MAXDIM],KG[CVMGen_MAXDIM];
	tReal D[CVMVegas_AXBINMAX][CVMGen_MAXDIM],DI[CVMVegas_AXBINMAX][CVMGen_MAXDIM];
	tReal fRan[CVMGen_MAXDIM]; //keeps random values (can be avoided, see code) -->QRAN

	// Constants
	tReal ALPH=1.5; int MDS=1;
	//TODO: check MDS behaviour in Djangoh (It is always re-set at evety entry start?). Need to be saved?
	//is DATA// called et every entry?

	//***************************** No initialization before this line
	if (!bUseOldGrid) {
		//Clear grid
		m_nAxisParts=1; for(j=0;j<m_nDim;j++){m_spGrid[0][j]=1.;}
		bUseOldRes=false;
	} // else VEGAS1+ call
	if(!bUseOldRes){
		//Clear cummulative variables
		m_nIt=0; m_SI=0.; m_SI2=m_SI; m_SWGT=m_SI; m_SCHI=m_SI;
	} // else VEGAS2+ call (no initialization)

	long nItMaxAct=nItMax+m_nIt;	//max. no of iteration (started from m_nIt)	-->ITMXX
	int nAxisPartsReq=CVMVegas_AXBINMAX; //	-->ND, nAxisPartsReq-1<-->NDM
	int NG=1;
	long nCallsPerG; //	-->NPG
	if (MDS!=0) {
		NG=(int)pow((tReal)(((tReal)nCalls)/2.),(tReal)(1./m_nDim));
		MDS=1;
		if ((2*NG-CVMVegas_AXBINMAX)>=0) { MDS=-1; nCallsPerG=NG/CVMVegas_AXBINMAX+1; nAxisPartsReq=NG/nCallsPerG; NG=nCallsPerG*nAxisPartsReq; }
	}
	k=(long)pow((tReal)NG,m_nDim);
	nCallsPerG=nCalls/k; if(nCallsPerG<2)nCallsPerG=2;
	long nCallsAct=nCallsPerG*k; // Actual number of function calls -->CALLS
	tReal DXG=1./NG;
	tReal DV2G=pow((tReal)DXG,(tReal)(((tReal)2.*((long)m_nDim))/nCallsPerG/nCallsPerG/(nCallsPerG-1.)));
	DXG*=nAxisPartsReq;
	tReal varVolume=1.;//	-->XJAC
	for(j=0;j<m_nDim;j++){ _VarDel[j]=m_VarMax[j]-m_VarMin[j]; varVolume*=_VarDel[j]; }

	//******************************* Re-bin preserving bin density
	if(m_nAxisParts != nAxisPartsReq){
		tReal tmpRC=((tReal)m_nAxisParts)/((long)nAxisPartsReq);
		for(j=0;j<m_nDim;j++){
			tReal tmpXN=0.; tReal tmpDR=tmpXN; k=0; i=k;
			for(;;){
				tmpDR+=1.; tReal tmpXO=tmpXN; tmpXN=m_spGrid[k][j]; k++; //TODO: check for possible k-overflow here!
				if(tmpRC>tmpDR)continue;
				bool bTmpBreak=false;
				for(;;){
					tmpDR-=tmpRC; spGridI[i]=tmpXN-(tmpXN-tmpXO)*tmpDR; i++;
					if(i>=(nAxisPartsReq-1)){bTmpBreak=true;break;} if(tmpRC>tmpDR)break;
				}
				if(bTmpBreak)break;
			}
			for(i=0;i<(nAxisPartsReq-1);i++){m_spGrid[i][j]=spGridI[i];} m_spGrid[nAxisPartsReq-1][j]=1.;
		}
		m_nAxisParts=nAxisPartsReq;
	}
	if(nLogMode!=0){
		if (nLogMode==10) {
			std::cout<<"VEGAS NDIM="<<m_nDim<<", NCALL="<<nCallsAct<<", nItMax="<<nItMax<<", Precision="<<fPrecision<<", MDS="<<MDS<<", m_nAxisParts="<<m_nAxisParts<<std::endl;
		} else {
			std::cout<<"INPUT PARAMETERS FOR VEGAS   NDIM="<<m_nDim<<std::endl;
			std::cout<<"NCALL="<<nCallsAct<<", m_nIt="<<m_nIt<<", nItMax="<<nItMax<<", NG="<<NG<<", NPerG="<<nCallsPerG<<std::endl;
			std::cout<<"Precision="<<fPrecision<<", MDS="<<MDS<<", m_nAxisParts="<<m_nAxisParts<<std::endl;
		}
	}
	//
	//Note: VEGAS3 entry should be here
	//
	//No initialization are required for the followed vars
	tReal itRes=0;			//iteration result (integral)				-->TI
	tReal itResSDev2=0;	//standard deviation squared for itRes		-->TSI
	tReal res=0.;			//accumulated integral result				-->AVGI
	tReal resSDev=0.;		//standard deviation for accumulated result	-->SD
	tReal chi2It=0.;		//Chi^2 per iteration						-->CHI2A
	//******************************* Main integration loop
	for(;;) { // 9
		m_nIt++; itRes=0.; itResSDev2=0.;
		for(j=0;j<m_nDim;j++){
			KG[j]=1;
			for(i=0;i<m_nAxisParts;i++){ D[i][j]=itRes; DI[i][j]=itRes; }
		} // 10
		for(;;) { // 11
			tReal fSum=0.;//sum of function values			--> FB
			tReal f2Sum=0.;//sum of function values quared	--> F2B
			k=0;
			for(;;) { //12
				k++;
				for(j=0;j<m_nDim;j++){ fRan[j]=m_rnd->Rnd(); } //121
				tReal vWgt=varVolume; //volume weight		-->WGT
				for(j=0;j<m_nDim;j++){
					tReal XN=((tReal)KG[j]-fRan[j])*DXG+1.; IA[j]=(long)XN; long iaJ=IA[j];
					tReal RC,XO;
					if(iaJ<=1){
						XO=m_spGrid[iaJ-1][j]; RC=(XN-(double)iaJ)*XO; //TODO check iaJ index!!!!
					} else {
						XO=m_spGrid[iaJ-1][j]-m_spGrid[iaJ-2][j]; RC=m_spGrid[iaJ-2][j]+(XN-(double)iaJ)*XO;
					}
					m_GenValues[j]=m_VarMin[j]+RC*_VarDel[j]; vWgt*=XO*m_nAxisParts;
				} //15
				tReal F=m_pFunc->Call(m_GenValues)*vWgt; tReal W=vWgt/nCallsAct; //tReal F1=F/nCallsAct;
				tReal fSqr=F*F; fSum+=F; f2Sum+=fSqr; //fSqr -->F2
				for(j=0;j<m_nDim;j++){
					long iaJ=IA[j]; DI[iaJ-1][j]=DI[iaJ-1][j]+F/nCallsAct; if(MDS>=0)D[iaJ-1][j]=D[iaJ-1][j]+fSqr;
				} //16
				if(k>=nCallsPerG)break;
			}

			f2Sum*=nCallsPerG;
			f2Sum=sqrt(f2Sum); f2Sum=fabs((f2Sum-fSum)*(f2Sum+fSum)); // slower, but saving more digits then "f2Sum=fabs(f2Sum-fSum*fSum);"
			itRes+=fSum; itResSDev2+=f2Sum;
			if(MDS<0){
				for(j=0;j<m_nDim;j++){ long iaJ=IA[j]; D[iaJ-1][j]=D[iaJ-1][j]+f2Sum; } //17
			}
			bool bBreak11=true;
			for(k=m_nDim-1; k>=0; k--) { //19
				KG[k]=KG[k]%NG+1;
				if(KG[k]!=1){bBreak11=false;break;}
			}
			if(bBreak11)break;
		} // loop 11 end
		//******************************* Final results for this iteration
		itRes/=nCallsAct; itResSDev2*=DV2G; tReal itRes2=itRes*itRes;
		tReal itWgt=((itResSDev2==0.)?0.:itRes2/itResSDev2); //iteration weight	-->WGT
		m_SI+=itRes*itWgt; m_SI2+=itRes2; m_SWGT+=itWgt; m_SCHI+=itRes2*itWgt;
		res=((m_SWGT==0.)?itRes:m_SI/m_SWGT);
		/* //Orig:
		resSDev=((m_SI2==0.)?itResSDev2:(m_SWGT*m_nIt/m_SI2));
		chi2It=0.; if(m_nIt>1)chi2It=resSDev*(m_SCHI/m_SWGT-res*res)/(m_nIt-1);
		if(resSDev!=0.){ resSDev=1./resSDev; resSDev=sqrt(resSDev); } else { resSDev=itResSDev2; }
		tReal itResSDev=sqrt(itResSDev2); //added by VM
		*/
		///* //Modified
		tReal itResSDev=sqrt(itResSDev2);
		tReal _1dStdDev2=((m_SI2==0.)?((itResSDev2>0.)?(1./itResSDev2):0.):(m_SWGT*(double)m_nIt/m_SI2));
		chi2It=0.; if(m_nIt>1)chi2It=_1dStdDev2*(m_SCHI/m_SWGT-res*res)/(m_nIt-1);
		if (_1dStdDev2>0.) { resSDev=sqrt(1./_1dStdDev2); } else { resSDev=itResSDev; }
		//*/
		if(nLogMode!=0){
			if(nLogMode==10){
				std::cout<<m_nIt<<" "<<itRes<<" "<<itResSDev<<" "<<res<<" "<<resSDev<<" "<<chi2It<<std::endl;
			} else {
				std::cout<<"INTEGRATION BY VEGAS, ITERATION NO "<<m_nIt<<std::endl;
				std::cout<<"INTEGRAL="<<itRes<<",\t STD.DEV="<<itResSDev<<std::endl;
				std::cout<<"ACCU RES="<<res<<",\t STD.DEV="<<resSDev<<", CHI^2/it="<<chi2It<<std::endl;
			}
			if(nLogMode<0){
				for(j=0;j<m_nDim;j++){
					std::cout<<"DATA FOR AXIS "<<j<<"\tX \tDELT I \tCONVCE \tX \tDELT I \tCONVCE \tX \tDELT I \tCONVCE"<<std::endl;
					for(i=0;i<m_nAxisParts;i++){ std::cout<<m_spGrid[i][j]<<'\t'<<DI[i][j]<<'\t'<<D[i][j]<<'\t'; }
					std::cout<<std::endl;
				}//20
			}
		}
		//******************************* Refine grid
		tReal relErr=((resSDev!=0.)?fabs(resSDev/res):0.);
		m_Integral=res; m_Error=resSDev; m_Chi2It=itRes; m_StdDevIt=itResSDev; //TODO: need sqrt here?
		for(j=0;j<m_nDim;j++){
			tReal XO=D[0][j]; tReal XN=D[1][j]; D[0][j]=(XO+XN)/2.; DT[j]=D[0][j];
			for(i=1;i<(m_nAxisParts-1);i++){
				D[i][j]=XO+XN; XO=XN; XN=D[i+1][j]; D[i][j]=(D[i][j]+XN)/3.; DT[j]+=D[i][j];
			}//22
			D[m_nAxisParts-1][j]=(XN+XO)/2.;
			DT[j]+=D[m_nAxisParts-1][j];
		}//23
		for(j=0;j<m_nDim;j++){
			tReal RC=0.;
			for(i=0;i<m_nAxisParts;i++){
				R[i]=0.; if(D[i][j]>0.){ tReal XO=DT[j]/D[i][j]; R[i]=pow((tReal)((XO-1.)/XO/log(XO)),ALPH); } RC+=R[i];
			}//24
			RC/=m_nAxisParts; k=0; tReal XN=0.; tReal DR=XN; i=k;
			for(;;){//25
				DR+=R[k]; tReal XO=XN; XN=m_spGrid[k][j]; k++;
				if(RC>DR)continue;
				bool bTmpBreak=false;
				for(;;){
					DR-=RC; spGridI[i]=((DR==0.)?XN:(XN-(XN-XO)*DR/R[k-1])); i++;
					if(i>=(m_nAxisParts-1)){bTmpBreak=true;break;} if(RC>DR)break;
				}
				if(bTmpBreak)break;
			}
			for(i=0;i<(m_nAxisParts-1);i++){ m_spGrid[i][j]=spGridI[i]; }//27
			m_spGrid[m_nAxisParts-1][j]=1.;
		}//28
		if((m_nIt<nItMaxAct) && (fabs(fPrecision)<relErr)) continue; //goto 9
		if (m_nIt>=nItMaxAct) {
			printf("Exit on m_nIt: %ld >= %ld\n",m_nIt,nItMax);
		}
		if (fabs(fPrecision)>=relErr) {
			printf("Exit on precision: %g >= %g\n",(double)fPrecision,(double)relErr);
		}
		break;
	}
	m_Integral=res; m_Error=resSDev; m_Chi2It=chi2It;
	return m_Integral;
}
bool CVMGenVegas::GenEventWgt() {
	return false;
}
#endif //#ifndef CVMGen_NO_VEGAS

/////////////////////////////////////////////////////////////////////// CVMGenFoam
#if !defined(CVMGen_NO_FOAM)
#define CVMGenFoam_MIN -1.0e150
#define CVMGenFoam_MAX  1.0e150
#define CVMGenFoam_SW2 std::setprecision(7)<<std::setw(12)

//FFFFFF  BoX-FORMATs for nice and flexible outputs
#define CVMGenFoam_BXOPE	std::cout<<\
	"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"<<std::endl<<\
	"F                                                                              F"<<std::endl
#define CVMGenFoam_BXTXT(text)	std::cout<<"F                   "<<std::setw(40)<<text<<"                   F"<<std::endl
#define CVMGenFoam_BX1I(name,numb,text)	std::cout<<\
	"F "<<std::setw(10)<<name<<" = "<<std::setw(10)<<numb<<" = "          <<std::setw(50)<<text<<" F"<<std::endl
#define CVMGenFoam_BX1F(name,numb,text)	std::cout<<"F "<<std::setw(10)<<name<<\
	" = "<<std::setw(15)<<std::setprecision(8)<<numb<<"   =    "<<std::setw(40)<<text<<" F"<<std::endl
#define CVMGenFoam_BX2F(name,numb,err,text) std::cout<<"F "<<std::setw(10)<<name<<\
	" = "<<std::setw(15)<<std::setprecision(8)<<numb<<" +- "<<std::setw(15)<<std::setprecision(8)<<err<<\
	"  = "<<std::setw(25)<<text<<" F"<<std::endl
#define CVMGenFoam_BXCLO std::cout<<\
"F                                                                              F"<<std::endl<<\
"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"<<std::endl
//FFFFFF  BoX-FORMATs ends here
///////////////////////////////////////////////// CVMGenFoam::TFDISTR
CVMGenFoam::TFDISTR::TFDISTR(int typ) { m_Type=typ; m_Pi = 3.1415926535897932e0; }
CVMGenFoam::TFDISTR::~TFDISTR() {}
double CVMGenFoam::TFDISTR::Density(int nDim, double *Xarg) {
	double rho = 1e-100;
	if (m_Type == 1)  rho = Sphere( nDim,Xarg);
	else if ( m_Type == 2)  rho = Void(   nDim,Xarg);
	else if ( m_Type == 3)  rho = Diagon( nDim,Xarg);
	else if ( m_Type == 4)  rho = Camel( nDim,Xarg);
	else if ( m_Type == 5)  rho = Gauss( nDim,Xarg);
	//else if ( m_Type == 6)  rho = 0.; // Fortran link was here
	else if ( m_Type == 7) rho = Ridge( nDim,Xarg);
	else{ std::cout<<" TFDISTR::Density: !!!!!! WRONG m_Type= "<<m_Type<<std::endl; exit(0); }
	return rho;
}
double CVMGenFoam::TFDISTR::Camel(int nDim, double *Xarg) { // Testing function of Peter Lepage
	double Fun1,Fun2,R1,R2;
	double pos1=1e0/3e0; double pos2=2e0/3e0;
	double Gam1= 0.100e0; double Gam2= 0.100e0;  // as in JPC
	double sPi = sqrt(m_Pi); double xn1=1e0; double xn2=1e0;
	int i; R1=0; R2=0;
	for(i = 0 ; i<nDim ; i++){
		R1=R1+(Xarg[i] -pos1)*(Xarg[i] -pos1); R2=R2+(Xarg[i] -pos2)*(Xarg[i] -pos2);
		xn1=xn1*Gam1*sPi; xn2=xn2*Gam2*sPi;      
	}
	R1=sqrt(R1); R2=sqrt(R2);
	Fun1 = exp(-(R1*R1)/(Gam1*Gam1))/xn1; Fun2 = exp(-(R2*R2)/(Gam2*Gam2))/xn2;  // Gaussian delta-like profile
	return 0.5e0*(Fun1+ Fun2);
}
double CVMGenFoam::TFDISTR::Gauss(int nDim, double *Xarg) {
	double Fun1,R1;
	double pos1=1e0/3e0; double Gam1= 0.100e0;  // as in Camel
	double sPi = sqrt(m_Pi); double xn1=1e0; int i; R1=0;
	for(i = 0 ; i<nDim ; i++){ R1=R1+(Xarg[i] -pos1)*(Xarg[i] -pos1); xn1=xn1*Gam1*sPi; }
	R1   = sqrt(R1); Fun1 = exp(-(R1*R1)/(Gam1*Gam1))/xn1;  // Gaussian delta-like profile
	return Fun1;
}
double CVMGenFoam::TFDISTR::Sphere(int nDim, double *Xarg) { //n-dimensional thin sphere
	double f1,f2; double *Pos = new double[nDim]; double Radius=0.35, Gam=0.02; double R,r; int i;
	for(i = 0 ; i<nDim ; i++){
		if( (2*(i/2)-i) == 0 ) Pos[i]=0.25;   // even
		else Pos[i]=0.40;  // odd
	}
	R=0;
	for(i = 0 ; i<nDim ; i++){ r=Xarg[i]-Pos[i]; R= R+r*r; }
	R=sqrt(R);
	f1 = Gam/((R-Radius)*(R-Radius) + Gam*Gam)/sqrt(m_Pi); // B-W profile
	f2 = exp(-R*R/(Gam*Gam))/Gam/sqrt(m_Pi);               // Gaussian profile
	delete [] Pos;
	return f1;
}
double CVMGenFoam::TFDISTR::Diagon(int nDim, double *Xarg) { //diagonal in n-dimensions
	double f1, Gam = 0.05e0; double *vtr = new double[nDim]; double sum,R,r; int i; sum=0;
	for(i = 0 ; i<nDim ; i++) sum=sum+Xarg[i];
	R=0;
	for(i = 0 ; i<nDim ; i++){ r=Xarg[i]-sum/nDim; R= R+r*r; }
	R=sqrt(R);
	f1 = Gam/(R*R + Gam*Gam)/sqrt(m_Pi); // B-W profile
	delete [] vtr;
	return f1;
}
double CVMGenFoam::TFDISTR::Void(int nDim, double *Xarg){ //surface of n-dimensional hypercubic
	double f1; double  Gam = 0.05; int i; f1 = 0e0;
	for(i = 0 ; i<nDim ; i++){ if(      Xarg[i] <Gam )  f1 = 1/Gam; if( (1.0-Xarg[i])<Gam )  f1 = 1/Gam; }
	return f1;
}
double CVMGenFoam::TFDISTR::Ridge(int nDim, double *Xarg){ //2-dim sharp ridge of Perret-Gallix
	double f1; double  alf; alf = 0.01; alf = 0.0001; alf = 1e-6; double D= Xarg[0]+Xarg[1]-1;
	f1 = 2*alf*Xarg[1]/( D*D  +alf*alf);
	return f1;
}

///////////////////////////////////////////////// CVMGenFoam::TFMAXWT
CVMGenFoam::TFMAXWT::TFMAXWT(const double wmax, const int nBin) {
	m_Nent = 0; m_nBin = nBin; m_wmax = wmax;
	m_WtHst1 = new CVMHistLite("",m_nBin,0.,wmax);      // histogram of wt
	m_WtHst2 = new CVMHistLite("",m_nBin,0.,wmax);      // histogram of wt with wt
}
CVMGenFoam::TFMAXWT::~TFMAXWT(){
	delete m_WtHst1; delete m_WtHst2; m_WtHst1=NULL; m_WtHst2=NULL;
}
void CVMGenFoam::TFMAXWT::Reset(){ m_Nent=0; m_WtHst1->Reset(); m_WtHst2->Reset(); }
CVMGenFoam::TFMAXWT& CVMGenFoam::TFMAXWT::operator =(CVMGenFoam::TFMAXWT &From){
	if (&From == this) return *this;
	m_nBin=From.m_nBin; m_wmax=From.m_wmax; m_WtHst1=From.m_WtHst1; m_WtHst2=From.m_WtHst2; return *this;
}
void CVMGenFoam::TFMAXWT::Fill(double wt){ m_Nent =  m_Nent+1.0;  m_WtHst1->Fill(wt,1.0); m_WtHst2->Fill(wt,wt); }
void CVMGenFoam::TFMAXWT::Make(const double eps, double &MCeff){
	// Calculates Efficiency= AveWt/WtLim for a given tolerance level epsilon<<1
	double WtLim,AveWt; GetMCeff(eps, MCeff, WtLim); AveWt = MCeff*WtLim;
	std::cout<< "00000000000000000000000000000000000000000000000000000000000000000000000"<<std::endl;
	std::cout<< "00 -->WtLim: No_evt ="<<m_Nent<<"   <Wt> = "<<AveWt<<"  WtLim=  "<<WtLim<<std::endl;
	std::cout<< "00 -->WtLim: For eps = "<<eps  <<"    EFFICIENCY <Wt>/WtLim= "<<MCeff<<std::endl;
	std::cout<< "00000000000000000000000000000000000000000000000000000000000000000000000"<<std::endl;
}
void CVMGenFoam::TFMAXWT::GetMCeff(const double eps, double &MCeff, double &WtLim){
	// Calculates Efficiency= AveWt/WtLim for a given tolerance level epsilon<<1 using information stored in two histograms
	int ib,ibX; double LowEdge,Bin,Bin1; double AveWt, AveWt1; //m_WtHst1->Print(); m_WtHst2->Print();
	// Convention on bin-numbering: nb=1 for 1-st bin, undeflow nb=0, overflow nb=Nb+1
	double sum   = 0.0; double sumWt = 0.0;
	//TODO: add outmargins?
	for(ib=0;ib<=m_nBin+1;ib++){ sum   += m_WtHst1->GetValueOrOverflow(ib); sumWt += m_WtHst2->GetValueOrOverflow(ib); }
	if( (sum == 0.0) || (sumWt == 0.0) ){ std::cout<<"TFMAXWT::Make: zero content of histogram !!!,sum,sumWt ="<<sum<<sumWt<<std::endl; }
	AveWt = sumWt/sum;
	//--------------------------------------
	for( ibX=m_nBin+1; ibX>0; ibX--){
		LowEdge = (ibX-1.0)*m_wmax/m_nBin; sum   = 0.0; sumWt = 0.0;
		for( ib=0; ib<=m_nBin+1; ib++){
			Bin=m_WtHst1->GetValueOrOverflow(ib); Bin1=m_WtHst2->GetValueOrOverflow(ib);
			if(ib>=ibX)Bin1=LowEdge*Bin; sum+=Bin; sumWt+=Bin1;
		}
		AveWt1 = sumWt/sum;
		if( fabs(1.0-AveWt1/AveWt) > eps ) break;
	}
	//---------------------------
	if(ibX == (m_nBin+1) ){
		WtLim = 1.0e200; MCeff   = 0.0; std::cout<< "+++++ WtLim undefined. Higher uper limit in histogram"<<std::endl;
	}else if( ibX == 1){
		WtLim = 0.0;
		MCeff   =-1.0;
		std::cout<< "+++++ WtLim undefined. Lower uper limit or more bins "<<std::endl;
	}else{
		WtLim= (ibX)*m_wmax/m_nBin; // We over-estimate WtLim, under-estimate MCeff
		MCeff  = AveWt/WtLim;
	}
}

///////////////////////////////////////////////// CVMGenFoam::TFVECT
CVMGenFoam::TFVECT::TFVECT(){ m_Dim=0; m_Coords=NULL; m_Next=NULL; m_Prev=NULL; } // empty vector of zero dimension
CVMGenFoam::TFVECT::TFVECT(const int n) {
// USER constructor creates n-densional vector and allocated dynamicaly array of components
	int i;
	m_Next=NULL; m_Prev=NULL; m_Dim=n; m_Coords = NULL;
	if (n>0){
		m_Coords = new double[m_Dim];
		if(m_chat) { if(m_Coords == NULL) { std::cout<<"TFVECT constructor failed to allocate"<<std::endl; exit(0);} }
		for (i=0; i<n; i++) *(m_Coords+i)=0.0;
	}
	if(m_chat) std::cout<<" USER CONSTRUCTOR TFVECT(const int) "<<std::endl;
}
CVMGenFoam::TFVECT::TFVECT(const TFVECT &Vect){
	m_Next=NULL; m_Prev=NULL; m_Dim=Vect.m_Dim; m_Coords = NULL;
	if(m_Dim>0)  m_Coords = new double[m_Dim];
	if(m_chat) { if(m_Coords == NULL){ std::cout<<"TFVECT constructor failed to allocate m_Coords"<<std::endl; exit(0); } }
	for(int i=0; i<m_Dim; i++) m_Coords[i] = Vect.m_Coords[i];
	std::cout<<"+++++ NEVER USE Copy constructor !!!!! "<<std::endl;
}
CVMGenFoam::TFVECT::~TFVECT(){
	if(m_chat) std::cout<<" DESTRUCTOR TFVECT~ "<<std::endl;
	delete [] m_Coords; //  free(m_Coords)
	m_Coords=NULL;
}
// operators
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator =(const CVMGenFoam::TFVECT& Vect){
	int i; if (&Vect == this) return *this;
	if( m_Dim != Vect.m_Dim )std::cout<<"++++>> TFVECT::operator=    Dims. are different: "<<m_Dim<<" and "<<Vect.m_Dim<<std::endl;
	if( m_Dim != Vect.m_Dim ){ delete [] m_Coords; m_Coords = new double[m_Dim]; }
	m_Dim=Vect.m_Dim;
	for(i=0; i<m_Dim; i++)m_Coords[i] = Vect.m_Coords[i];
	m_Next=Vect.m_Next; m_Prev=Vect.m_Prev;
	if(m_chat)  std::cout<<" SUBSITUTE operator = "<<std::endl;
	return *this;
}
double &CVMGenFoam::TFVECT::operator[](int n) {
// [] is for acces to elements as in ordinary matrix like a[j]=b[j]
// (Perhaps against some strict rules but very practical.)
// Range protection is built in, consequently for substitution
// one should use rather use a=b than explicit loop!
	if ((n<0) || (n>=m_Dim)){ std::cout<<"++++>> TFVECT::operator[], out of range"<<std::endl; exit(1); }
	return m_Coords[n];
}
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator*=(const double &x){// operator *=; multiply by scalar; c*=x, tested
	for(int i=0;i<m_Dim;i++)m_Coords[i] = m_Coords[i]*x; return *this;
}
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator+=(const CVMGenFoam::TFVECT& Shift){// operator +=; add vector; c*=x, tested
	if( m_Dim != Shift.m_Dim){ std::cout<<"++++>> TFVECT::operator+, different dimensions= "<<m_Dim<<" "<<Shift.m_Dim<<std::endl; }
	for(int i=0;i<m_Dim;i++) m_Coords[i] = m_Coords[i]+Shift.m_Coords[i]; return *this;
}
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator-=(const CVMGenFoam::TFVECT& Shift){// operator -=; add vector; c*=x, tested
	if( m_Dim != Shift.m_Dim){ std::cout<<"++++>> TFVECT::operator-, different dimensions= "<<m_Dim<<" "<<Shift.m_Dim<<std::endl; }
	for(int i=0;i<m_Dim;i++) m_Coords[i] = m_Coords[i]-Shift.m_Coords[i]; return *this;
}
CVMGenFoam::TFVECT CVMGenFoam::TFVECT::operator+(const CVMGenFoam::TFVECT &p2){
	// operator + ; sum of 2 vectors; c=a+b, a=a+b,       NEVER USE IT, SLOW!!!
	TFVECT Temp(m_Dim); Temp =(*this); Temp+=p2; return Temp;
}
CVMGenFoam::TFVECT CVMGenFoam::TFVECT::operator-(const CVMGenFoam::TFVECT &p2){
	// operator - ; difference of 2 vectors; c=a-b, a=a-b, NEVER USE IT, SLOW!!!
	TFVECT Temp(m_Dim); Temp  = (*this); Temp -= p2; return Temp;
}
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator =(double Vect[]){ // Load in double vector, sometimes can be usefull
	int i; for(i=0; i<m_Dim; i++)m_Coords[i] = Vect[i]; return *this;
}
CVMGenFoam::TFVECT& CVMGenFoam::TFVECT::operator =(double x){ // Load in double number, sometimes can be usefull
	if(m_Coords != NULL){ for(int i=0; i<m_Dim; i++) m_Coords[i] = x; } return *this;
}
// Others
void CVMGenFoam::TFVECT::Print(void){ // Printout of all components on "cout"
	int i; std::cout << "("; for(i=0; i<m_Dim-1; i++) std::cout  << CVMGenFoam_SW2 << *(m_Coords+i) << ","; std::cout  << CVMGenFoam_SW2 << *(m_Coords+m_Dim-1); std::cout << ")";
}
void CVMGenFoam::TFVECT::PrintList(void){ // Printout of all member vectors in the list starting from "this"
  long i=0; if(this == NULL) return; TFVECT *current=this; while(current != NULL){ std::cout<<"vec["<<i<<"]="; current->Print(); std::cout<<std::endl; current = current->m_Next; i++; }
}
const int &CVMGenFoam::TFVECT::GetDim(void){ // Getter for vector dimension
	return m_Dim;
}
///////////////////////////////////////////////// CVMGenFoam::TFCELL
// Implements single cell of foam in k dimensions hyper-cubical subspace and n dimensions in simplical subspace.
CVMGenFoam::TFCELL::TFCELL(){ m_nVert=0; m_nPool=0; m_Pool=NULL; m_Parent=-1; m_Daught0=-1; m_Daught1=-1; m_Verts=NULL; m_Posi=NULL; m_Size=NULL; }
CVMGenFoam::TFCELL::TFCELL(int nDim, int kDim, int OptMCell, int OptCu1st) {
	// Constructor of a single Cell, verices assigned later on
	if (  nDim+kDim >0){
		m_nDim     = nDim; m_kDim     = kDim;
		m_OptMCell = OptMCell;   // MegaCell option, slim memory for h-cubes
		m_OptCu1st = OptCu1st;   // =1 Numbering of dimens starts with h-cubic, DIPSWITCH
		m_nPool    = 0; m_Pool     = NULL; m_Status   = 1; m_Parent   =-1; m_Daught0  =-1; m_Daught1  =-1;
		m_Xdiv     = 0.0; m_Best     = 0; m_Volume   = 0.0; m_Integral = 0.0; m_Drive    = 0.0; m_Primary  = 0.0;
		// m_Verts is allocated  and filled in latter on in Fill !!!!!
		m_nVert = 0; m_Verts = NULL;
		//--- hyp-cubical subspace ---
		m_Posi = NULL; m_Size = NULL;
		if( (m_kDim>0) && (m_OptMCell == 0)  ){ m_Posi = new TFVECT(m_kDim); m_Size = new TFVECT(m_kDim); }
	} else { std::cout<<"TFCELL::TFCELL: total dimesion has to be >0 "<<std::endl; exit(0); }
}
CVMGenFoam::TFCELL::TFCELL(TFCELL &From){ //Copy constructor (not tested!)
	int i; std::cout<<"+++++ NEVER USE Copy constructor for TFCELL"<<std::endl;
	m_nPool       = From.m_nPool; for(i=0; i<m_nPool+1; i++) m_Pool[i]=From.m_Pool[i];
	m_Status=From.m_Status; m_Parent=From.m_Parent; m_Daught0=From.m_Daught0; m_Daught1=From.m_Daught1;
	m_Xdiv=From.m_Xdiv; m_Best=From.m_Best; m_Volume=From.m_Volume; m_Integral=From.m_Integral; m_Drive=From.m_Drive;
	m_Primary=From.m_Primary; m_nVert=From.m_nVert; m_Verts=NULL;
	if(m_nDim>0){ m_Verts = new int[m_nVert]; for(i=0; i<m_nVert; i++) m_Verts[i] = (From.m_Verts)[i]; }
	m_Posi        = From.m_Posi; // used copy constructor of FVec
	m_Size        = From.m_Size; // used copy constructor of FVec
}
CVMGenFoam::TFCELL::~TFCELL(){
	delete [] m_Verts;
	if(m_Posi != NULL ) delete m_Posi;
	if(m_Size != NULL ) delete m_Size;
}
CVMGenFoam::TFVECT*& CVMGenFoam::TFCELL::operator[](int i){ // Getter of vertex true pointer
	if ((i>=0) && (i<=m_nDim)) return m_Vert0[ m_Verts[i] ];
	else{
		std::cout<<"TFCELL::operator[] out of range, m_nDim= "<<m_nDim<<",  index= "<<i<<std::endl; exit(1);
		return m_Vert0[0]; // this is to make insure happy
	}
}
int CVMGenFoam::TFCELL::operator()(int i){ // Getter of vertex integer pointer
	if ((i>=0) && (i<=m_nDim)) return m_Verts[i];
	else{
		std::cout<<"TFCELL::operator() out of range, m_nDim= "<<m_nDim<<",  index= "<<i<<std::endl; exit(1);
		return -1; // this is to make insure happy
	}
}
CVMGenFoam::TFCELL& CVMGenFoam::TFCELL::operator=(CVMGenFoam::TFCELL &From){ //operator= (never used)
	int i; std::cout<<" TFCELL::operator= "<<std::endl;
	if (&From == this) return *this;
	m_nPool       = From.m_nPool; for(i=0; i<m_nPool+1; i++) m_Pool[i]=From.m_Pool[i];
	m_Status      = From.m_Status; m_Parent      = From.m_Parent; m_Daught0   = From.m_Daught0;  m_Daught1   = From.m_Daught1;
	m_Xdiv        = From.m_Xdiv; m_Best        = From.m_Best; m_Volume      = From.m_Volume; m_Integral    = From.m_Integral;
	m_Drive       = From.m_Drive; m_Primary     = From.m_Primary;
	for(i=0; i<m_nVert; i++) m_Verts[i] = From.m_Verts[i];
	m_Posi        = From.m_Posi; m_Size        = From.m_Size;
	return *this;
}
void CVMGenFoam::TFCELL::Fill(int Status, int Parent, int Daugh1, int Daugh2,
							  int *Vertices, TFVECT *Posi, TFVECT *Size) // Fills Cell which has to exist already
{
	int i; m_Status  = Status; m_Parent  = Parent; m_Daught0 = Daugh1; m_Daught1 = Daugh2;
	if(Vertices!=NULL){
		if( (m_Verts != NULL) || (m_nVert != 0) ){ std::cout<<"TFCELL::Fill ++++++  m_Verts already allocated m_nVert="<<m_nVert<<std::endl; exit(5); }
		m_nVert = m_nDim+1; m_Verts = new int[m_nVert]; for(i=0; i<m_nVert; i++) m_Verts[i] = Vertices[i];  // Pure Pointers
	}
	if(m_OptMCell == 0){ if(Posi!=NULL) (*m_Posi) = (*Posi); if(Size!=NULL) (*m_Size) = (*Size); }
}
// GETTERS/SETTERS
void CVMGenFoam::TFCELL::GetHcub(CVMGenFoam::TFVECT &Posi, CVMGenFoam::TFVECT &Size){ // Sophisticated Getter for Hyper-Cubic dimensions and size
	if(m_kDim<1) return;
	if(m_OptMCell==0){ Posi=*m_Posi; Size=*m_Size; }
	else{
		TFCELL *pCell,*dCell; Posi = 0.0; Size=1.0; // load all components
		dCell = this;
		while(dCell != NULL){
			pCell = dCell->GetPare(); if( pCell== NULL) break;
			if((pCell->GetDau0()==NULL) && (pCell->GetDau1()==NULL)) break; //case nDim>0, kDim>0
			int    kBest = pCell->m_Best;
			double xDivi = pCell->m_Xdiv;
			int kShift; if(m_OptCu1st) kShift = 0; else  kShift=m_nDim*(m_nDim+1)/2 ;
			if( ( kShift <= kBest) && (kBest < kShift+m_kDim) ){    // only if division was in h-cubic subspace
				int kDiv = kBest-kShift;
				if(         dCell == pCell->GetDau0()  ){
					Size[kDiv]=Size[kDiv]*xDivi; Posi[kDiv]=Posi[kDiv]*xDivi;
				}else if(   dCell == pCell->GetDau1()  ){
					Size[kDiv]=Size[kDiv]*(1.0-xDivi); Posi[kDiv]=Posi[kDiv]*(1.0-xDivi)+xDivi;
				}else{
					std::cout<<" +++++ STOP in TFCELL::GetHcub "<<std::endl; exit(2);
				}
			}
			dCell=pCell;
		}//while
	}//OptMCell
}//GetHcub
void CVMGenFoam::TFCELL::GetHSize(CVMGenFoam::TFVECT &Size){ // Sophisticated Getter for size vector of h-cubic cell
	if(m_kDim<1) return;
	if(m_OptMCell==0){ Size=*m_Size; }
	else{
		TFCELL *pCell,*dCell;
		Size=1.0; // load all components
		dCell = this;
		while(dCell != NULL){
			pCell = dCell->GetPare(); if( pCell== NULL) break;
			if((pCell->GetDau0()==NULL) && (pCell->GetDau1()==NULL)) break; //case nDim>0, kDim>0
			int    kBest = pCell->m_Best;
			double xDivi = pCell->m_Xdiv;
			int kShift; if(m_OptCu1st) kShift = 0; else  kShift=m_nDim*(m_nDim+1)/2 ;
			if( ( kShift <= kBest) && (kBest < kShift+m_kDim) ){    // only if division was in h-cubic subspace
				int kDiv = kBest-kShift;
				if(        dCell == pCell->GetDau0() ){ Size[kDiv]=Size[kDiv]*xDivi; }
				else if(  dCell == pCell->GetDau1()  ){ Size[kDiv]=Size[kDiv]*(1.0-xDivi); }
				else{ std::cout<<" +++++ STOP in TFCELL::GetHSize "<<std::endl; exit(2); }
			}
			dCell=pCell;
		}//while
	}//OptMCell
}//GetHSize
void CVMGenFoam::TFCELL::GetXSimp(CVMGenFoam::TFVECT & X, CVMGenFoam::TFVECT & lambda, int kVert){ // Translates internal simplicial coordinates into absolute
	TFCELL *pCell,*dCell; TFVECT *p; int i,j,k,iDiv,jDiv,kVold,kBest; long *PtsDiv; double sum,xDivi;
	if(m_nDim<2){  std::cout <<"No simplicial subspace"<<std::endl; exit(2); }
	PtsDiv = new long [(m_nDim+1)*m_nDim/2]; k=0; X=0.;
	for(i=0; i<m_nDim+1; i++) for(j=i+1; j<m_nDim+1; j++) PtsDiv[k++]=(m_nDim+1)*i+j;
	dCell = this; sum=0.; lambda[kVert]=0.;
	for(i=0; i< m_nDim+1; i++) sum+=lambda[i]; //lambda.Print();
	int kShift; if(m_OptCu1st) kShift = m_kDim; else  kShift=0 ;
	while(dCell != NULL){
		pCell = dCell->GetPare(); if( pCell== NULL) break;
		if((pCell->GetDau0()==NULL) && (pCell->GetDau1()==NULL)) break; //case of starting cubic divided into simplices 
		kBest = pCell->m_Best;
		if(kBest>=kShift ){  //If division was in simplicial subspace
			xDivi = pCell->m_Xdiv; iDiv=PtsDiv[kBest-kShift]/(m_nDim+1); jDiv=PtsDiv[kBest-kShift] % (m_nDim+1); kVold=kVert;
			if(  dCell == pCell->GetDau0()  ){
				kVert=jDiv;
				if(kVert!=kVold) { lambda[kVold]=1.-sum; sum=1.-lambda[kVert]; lambda[kVert]=0.; }
				sum+=(xDivi-1)*lambda[iDiv]; lambda[iDiv]*=xDivi;  
			}else if(   dCell == pCell->GetDau1()  ){
				kVert=iDiv;
				if(kVert!=kVold) { lambda[kVold]=1.-sum; sum=1.-lambda[kVert]; lambda[kVert]=0.; }
				sum+=-xDivi*lambda[jDiv]; lambda[jDiv]*=(1.-xDivi); 
			}
		}//End of condition that division was in simplicial subspace
		if(dCell != pCell->GetDau0() && dCell != pCell->GetDau1()) { std::cout<<" +++++ STOP in TFCELL::GetXSimp "<<std::endl; exit(2); }
		dCell=pCell;
	}// End while
	// std::cout <<std::endl << "Vertex = "<<kVert<<std::endl;
	//pCell->Print();
	for(i=0; i<m_nDim+1; i++){
		p=(*dCell)[i];
		if(i==kVert){ for(j=0; j<m_nDim; j++) X[j]+=(1.-sum+lambda[kVert])*p->GetCoord(j); }
		else{ for(j=0; j<m_nDim; j++) X[j]+=lambda[i]*p->GetCoord(j); }
	}
	delete PtsDiv;
} //End of GetXSimp
void CVMGenFoam::TFCELL::CalcVolume(void){ // Another way to calculate volume
	TFCELL *pCell,*dCell; int i,k,kBest; double xDivi,volu; int kShift; volu=1.0;
	if(m_nDim>0){ 
		if(m_OptCu1st) kShift = m_kDim; else  kShift=0 ; dCell = this;
		while(dCell != NULL){
			pCell = dCell->GetPare(); if( pCell== NULL) break;
			if((pCell->GetDau0()==NULL) && (pCell->GetDau1()==NULL)) break; //case of starting cubic divided into simplices
			kBest = pCell->m_Best;
			if(kBest>=kShift ){  //If division was in simplicial subspace
				xDivi = pCell->m_Xdiv;
				if(  dCell == pCell->GetDau0()  ){ volu*=xDivi; }else if(   dCell == pCell->GetDau1()  ){ volu*=(1.-xDivi); }
			}//End of condition that division was in simplicial subspace
			if(dCell != pCell->GetDau0() && dCell != pCell->GetDau1()) { std::cout<<" +++++ STOP in TFCELL::CalcVolume "<<std::endl; exit(2); }
			dCell=pCell;
		} //end while
		for(i=2; i<=m_nDim; i++) volu /= i;  // divide by factorial
	} //end of simplicial subspace
	if(m_kDim>0){         // h-cubical subspace
		if(m_OptMCell == 0){ for(k=0; k<m_kDim; k++) volu *= (*m_Size)[k]; }
		else{ TFVECT Size(m_kDim); (this)->GetHSize(Size); for(k=0; k<m_kDim; k++) volu *= Size[k]; }
	}
	m_Volume =volu;
}
void CVMGenFoam::TFCELL::Print(void){ // Printout for debug purpose
	int i;
	std::cout <<  " Status= "<<     m_Status   <<",";
	std::cout <<  " Volume= "<<     m_Volume   <<",";
	std::cout <<  " TrueInteg= " << m_Integral <<",";
	std::cout <<  " DriveInteg= "<< m_Drive    <<",";
	std::cout <<  " PrimInteg= " << m_Primary  <<",";
	std::cout<< std::endl;
	std::cout <<  " Xdiv= "<<m_Xdiv<<",";
	std::cout <<  " Best= "<<m_Best<<",";
	std::cout <<  " Parent=  {"<<m_Parent <<"} "; // extra DEBUG
	std::cout <<  " Daught0= {"<<m_Daught0<<"} "; // extra DEBUG
	std::cout <<  " Daught1= {"<<m_Daught1<<"} "; // extra DEBUG
	std::cout<< std::endl;
	if( m_Verts == NULL ){ std::cout<<"   Vertex list is NULL"<< std::endl; }
	else{
		for(i=0; i<m_nVert; i++){
			std::cout <<"   Vert["<<i<<"]= ";
			if( m_Verts==NULL ){std::cout<< " Vertices not assigned";}else{(*this)[i]->Print(); }
			std::cout<<" "<< std::endl;
		}
	}
	if(m_Posi!=NULL){ std::cout <<"   Posi= "; m_Posi->Print(); std::cout<<","<< std::endl;}
	if(m_Size!=NULL){ std::cout <<"   Size= "; m_Size->Print(); std::cout<<","<< std::endl;}
	// Alternatively, Posi and Size has to be calculated for m_OptMCell=1 
	if(m_kDim>0 && m_OptMCell != 0 ){
		TFVECT Posi(m_kDim); TFVECT Size(m_kDim); (this)->GetHcub(Posi,Size);
		std::cout <<"   Posi= "; Posi.Print(); std::cout<<","<< std::endl;
		std::cout <<"   Size= "; Size.Print(); std::cout<<","<< std::endl;
	}
	// Also print momenta from MC pool
	//if( m_Pool != NULL ) m_Pool->PrintList();
}
///////////////////////////////////////////////// CVMGenFoam::TFPARTITION
// Auxiliary class TFMATRIX of  square matrices
// Needed essentialy for calculating volume of the simplex
CVMGenFoam::TFMATRIX::TFMATRIX(){ Dim=0; Dim2=0; Matrix=NULL; std::cout<<"TFMATRIX Default constructor called"<<std::endl; }
CVMGenFoam::TFMATRIX::TFMATRIX(int dim ){ Dim = dim; Dim2 = dim*dim; Matrix = NULL; if (dim>0) Matrix = new double[dim*dim]; }
CVMGenFoam::TFMATRIX::TFMATRIX(TFMATRIX &From){
	int i, j; Dim= From.Dim; Matrix = NULL; if(Dim>0) Matrix = new double[Dim*Dim];
	for(i=0; i<Dim; i++) for(j=0; j<Dim; j++) *(Matrix + Dim*i + j) = *(From.Matrix + Dim*i + j);
}
CVMGenFoam::TFMATRIX::~TFMATRIX(){ if(Dim>0)delete [] Matrix; Matrix=NULL; }
CVMGenFoam::TFMATRIX& CVMGenFoam::TFMATRIX::operator =(CVMGenFoam::TFMATRIX &From){
	int i, j; if (&From == this) return *this;
	Dim=From.Dim; Matrix = new double[Dim*Dim];
	for(i=0; i<Dim; i++) for(j=0; j<Dim; j++) *(Matrix + Dim*i + j) = *(From.Matrix + Dim*i + j);
	return *this;
}
double& CVMGenFoam::TFMATRIX::operator()(int i, int j){ return *(Matrix + Dim*i + j); }
double CVMGenFoam::TFMATRIX::Determinant(){
	//  Determinan using moderately efficient method (Gauss)
	int i, j, k, l; double temp, det, elmax;
	TFMATRIX m = *this; //TFMATRIX m(Dim); m =*this;
	det=1;
	for(i=0; i<Dim; i++){
		for(j=i+1; j<Dim; j++){ // i-th raw will be subtracted from j-th such that M[j,i]=0
			l=i; elmax= 0.0;
			for(k=i; k<Dim;k++) // looking for a raw with maximum modulus of i-th element;
				if( fabs(m(k,i))>elmax){ elmax=fabs(m(k,i)); l=k; }
				if(elmax==0.0) return 0.0;
				if (l!=i){                      // swaping two raws
					for(k=0; k<Dim; k++){ temp = m(i,k); m(i,k)=m(l,k);  m(l,k)=temp; }
					det *= -1;
				}
				temp = m(j,i)/m(i,i); // subtract i-th from j-th such that M[j,i]=0
				for(k=0; k<Dim; k++) m(j,k) -= m(i,k) *temp;
		}
	} // matrix is now triangle type
	for(i=0; i<Dim; i++)det *= m(i,i);
	return det;
}
void CVMGenFoam::TFMATRIX::Print(){ // just printing
	int i, j; std::cout << "Matrix [" << std::endl;
	for(i=0; i<Dim; i++){
		std::cout << "(";
		for(j=0; j<Dim-1; j++) std::cout << CVMGenFoam_SW2 << (*this)(i,j) << ",";
		std::cout << CVMGenFoam_SW2 << (*this)(i,Dim-1) << ")" << std::endl;
	}
	std::cout << "] Matrix" << std::endl;
}
///////////////////////////////////////////////// CVMGenFoam::TFPARTITION
// Auxiliary class for organizing loop over partitions
CVMGenFoam::TFPARTITION::TFPARTITION(int len){
	int k; m_len = len;
	if(m_len>0){ m_digits = new int[m_len]; for(k=0;k<m_len;k++) m_digits[k]=0; }else{ m_digits = NULL; }
}
CVMGenFoam::TFPARTITION::~TFPARTITION(){ if(m_digits != NULL) delete [] m_digits; }
void CVMGenFoam::TFPARTITION::Reset(){  for(int k=0; k<m_len; k++){ m_digits[k]=0; } }
int CVMGenFoam::TFPARTITION::Next(){
	int j,status;
	if(m_len>0){
		status=0;
		m_digits[m_len-1]++;           // Basic increment
		for(j=m_len-1; j>=0; j--){
			if(m_digits[j]==2){          // Overflow goes to left-most digits
				m_digits[j]=0; if(j>0) m_digits[j-1]++;
			}
			status +=m_digits[j];
		}
	}else{ status=0; }
	return status;        // returns 0 AFTER last partition
}
const int& CVMGenFoam::TFPARTITION::Digit(int i){ if(m_len>0){ return m_digits[i]; }else{ return m_len; } }

///////////////////////////////////////////////// CVMGenFoam::TFOAM
CVMGenFoam::TFOAM::TFOAM(){  
  m_nDim     = 0; m_kDim     = 0; m_TotDim   = 0; m_nProj    = 0; m_NoAct    = 0; m_nCells   = 0; m_vMax     = 0; m_RNmax    = 0;
  m_MaskDiv  = NULL; m_InhiDiv  = NULL; m_XdivPRD  = NULL; m_VerX     = NULL; m_Cells    = NULL; m_Lambda   = NULL; m_Alpha    = NULL;
  m_CellsAct = NULL; m_PrimAcu  = NULL; m_HistEdg  = NULL; m_HistWt   = NULL;
  m_MCMonit    = NULL; m_MCvect     = NULL; m_Rvec       = NULL;
  m_PseRan     = NULL; m_Rho = NULL;
}
CVMGenFoam::TFOAM::TFOAM(const char* Name){
	if(strlen(Name)  >129) { std::cout<< " ++++STOP in TFOAM::TFOAM, Name too long "<<strlen(Name)<<std::endl; exit(2); }
	sprintf(m_Name,"%s",Name);         // Class name
	sprintf(m_Date,"%s","  Release date:  2003.10.07    "); // Release date
	sprintf(m_Version,"%s", "2.06"); // Release version
	m_OptDebug = 1;                // =1 additional histogram, not SetDirectory(0), DIPSWITCH
	m_OptCu1st = 1;                // =1 Numbering of dimens starts with h-cubic, DIPSWITCH
	m_MaskDiv  = NULL;             // Dynamic Mask for  cell division, h-cubic + simplical
	m_InhiDiv  = NULL;             // Flag alowing to inhibit cell division in certain projection/edge
	m_XdivPRD  = NULL;             // Lists of division values encoded in one verctor per direction
	m_VerX     = NULL;
	m_Cells    = NULL; m_Lambda   = NULL; m_Alpha    = NULL; m_CellsAct = NULL; m_PrimAcu  = NULL; m_HistEdg  = NULL; m_HistWt   = NULL;
	m_nDim     = 0;                // dimension of simplical sub-space
	m_kDim     = 0;                // dimension of hyp-cubical sub-space
	m_nCells   = 1000;             // Maximum number of Cells,    is usually re-set
	m_vMax     = 0;                // Maximum number of vertices, is re-calculated from m_nCells
	m_nSampl   = 200;              // No of sampling when dividing cell
	m_OptPRD   = 0;                // General Option switch for PRedefined Division, for quick check
	m_OptDrive = 2;                // type of Drive =1,2 for TrueVol,Sigma,WtMax
	m_OptEdge  = 0;                // decides whether vertices are included in the sampling
	m_OptPeek  = 0;                // type of Peek =0,1 for maximum, random
	m_OptOrd   = 0;                // root cell h-cub. or simplex, =0,1
	m_OptMCell = 1;                // MegaCell option, slim memory for h-cubes
	m_OptVert  = 1;                // Vertices of simplex are not stored
	m_Chat     = 1;                // Chat=0,1,2 chat level in output, Chat=1 normal level
	m_OptRej   = 0;                // OptRej=0, wted events; OptRej=0, wt-1 events
	//------------------------------------------------------
	m_nBin     = 8;                // binning of edge-histogram in cell exploration
	m_EvPerBin =25;                // maximum no. of EFFECTIVE event per bin, =0 option is inactive
	//------------------------------------------------------
	m_nCalls = 0;                  // No of function calls
	m_nEffev = 0;                  // Total no of eff. wt=1 events in build=up
	m_LastCe =-1;                  // Index of the last cell
	m_NoAct  = 0;                  // No of active cells (used in MC generation)
	m_LastVe =-1;                  // Index of the last vertex
	m_WtMin = CVMGenFoam_MAX;                 // Minimal weight
	m_WtMax = CVMGenFoam_MIN;                 // Maximal weight
	m_MaxWtRej =1.10;              // Maximum weight in rejection for getting wt=1 events
	m_PseRan   = NULL;             // Initialize private copy of random number generator
	m_MCMonit  = NULL;             // MC efficiency monitoring
}
CVMGenFoam::TFOAM::~TFOAM(){
	if(m_Cells!= NULL) {
		for(int i=0; i<m_nCells; i++) delete m_Cells[i]; // TFCELL*[]
		delete [] m_Cells;
	}
	delete [] m_Rvec;    //double[]
	delete [] m_Alpha;   //double[]
	delete [] m_Lambda;  //double[]
	delete [] m_MCvect;  //double[]
	delete [] m_PrimAcu; //double[]
	delete [] m_MaskDiv; //int[]
	delete [] m_InhiDiv; //int[]
	if( m_VerX!= NULL) {
		for(int i=0; i<m_vMax; i++) delete m_VerX[i]; // TFVECT*[]
		delete [] m_VerX;
	}
	if( m_XdivPRD!= NULL) {
		for(int i=0; i<m_kDim; i++) delete m_XdivPRD[i]; // TFVECT*[]
		delete [] m_XdivPRD;
	}
	for(int i=0;i<m_nProj;i++) delete m_HistEdg[i];
	delete [] m_HistEdg;
	delete m_MCMonit;
	delete m_HistWt;
}
CVMGenFoam::TFOAM::TFOAM(const CVMGenFoam::TFOAM &From){// Copy Constructor  NOT IMPLEMENTED (NEVER USED)
  std::cout<<"+++++ Stop in Copy Constructor  TFOAM::TFOAM  NOT IMPLEMENTED"<<std::endl; exit(1);
  m_nDim = From.m_nDim; // this is to apease insure
}
CVMGenFoam::TFOAM& CVMGenFoam::TFOAM::operator=(const CVMGenFoam::TFOAM &From){
	std::cout<<"+++++ Stop in  TFOAM::operator= NOT IMPLEMENTED"<<std::endl; exit(1);
	m_nDim = From.m_nDim; // this is to apease insure
	return *this;         // this is to apease insure
}
void CVMGenFoam::TFOAM::Initialize(CVMRnd *PseRanRef, CVMGenFoam::TFOAM_INTEGRAND *fun ){
	//  Basic initialization, create "FOAM of cells"
	if(m_Chat>0){
		CVMGenFoam_BXOPE;
		CVMGenFoam_BXTXT("****************************************");
		CVMGenFoam_BXTXT("******     TFOAM::Initialize      ******");
		CVMGenFoam_BXTXT("****************************************");
		CVMGenFoam_BXTXT(m_Name);
		CVMGenFoam_BX1F("  Version",m_Version,  m_Date);
		CVMGenFoam_BX1I("     nDim",m_nDim,     " Dimension of the simplical sub-space             ");
		CVMGenFoam_BX1I("     kDim",m_kDim,     " Dimension of the hyper-cubical sub-space         ");
		CVMGenFoam_BX1I("   nCells",m_nCells,   " Requested number of Cells (half of them active)  ");
		CVMGenFoam_BX1I("   nSampl",m_nSampl,   " No of MC events in exploration of a cell cell    ");
		CVMGenFoam_BX1I("     nBin",m_nBin,     " No of bins in histograms, MC exploration of cell ");
		CVMGenFoam_BX1I(" EvPerBin",m_EvPerBin, " Maximum No effective_events/bin, MC exploration  ");
		CVMGenFoam_BX1I(" OptDrive",m_OptDrive, " Type of Driver   =1,2 for Sigma,WtMax            ");
		CVMGenFoam_BX1I("  OptEdge",m_OptEdge,  " Decides whether vertices are included in the MC  ");
		CVMGenFoam_BX1I("  OptPeek",m_OptPeek,  " Type of the cell Peek =0,1 for maximum, random   ");
		CVMGenFoam_BX1I("   OptOrd",m_OptOrd,   " Root cell hyp-cub. or simplex, =0,1              ");
		CVMGenFoam_BX1I(" OptMCell",m_OptMCell, " MegaCell option, slim memory for hyp-cubes       ");
		CVMGenFoam_BX1I(" OptVert" ,m_OptVert, " Decides whether to store vertices of simplices   ");
		CVMGenFoam_BX1I(" OptDebug",m_OptDebug, " Additional debug histogram, SetDirectory(1)      ");
		CVMGenFoam_BX1I(" OptCu1st",m_OptCu1st, " Numbering of dimensions  starts with h-cubic     ");
		CVMGenFoam_BX1I("   OptRej",m_OptRej,   " MC rejection on/off for OptRej=0,1               ");
		CVMGenFoam_BX1F(" MaxWtRej",m_MaxWtRej, " Maximum wt in rejection for wt=1 evts");
		CVMGenFoam_BXCLO;
	}
	m_PseRan = PseRanRef; // use central random number generator
	//m_PseRan = new TPSEMAR(); // Initialize private copy of random number generator
	m_Rho = fun;  // Integrand function defined/stored

	m_TotDim=m_nDim+m_kDim;
	if(m_TotDim==0) StopM("TFOAM::Initialize: zero dimension not alowed");
	
	/////////////////////////////////////////////////////////////////////////
	//                   ALLOCATE SMALL LISTS                              //
	//   it is done globaly, not for each cell, to save on allocation time //
	/////////////////////////////////////////////////////////////////////////
	m_RNmax= m_TotDim+1;
	m_Rvec = new tReal[m_RNmax];   // Vector of random numbers
	if(m_Rvec==NULL)  StopM("TFOAM::Initialize: Cannot initialize buffer m_Rvec");
	
	if(m_nDim>0){
		m_Lambda = new double[m_nDim];   // sum<1 for internal parametrization of the simplex
		if(m_Lambda==NULL)  StopM("TFOAM::Initialize: Cannot initialize buffer m_Lambda");
	}
	if(m_kDim>0){
		m_Alpha = new double[m_kDim];    // sum<1 for internal parametrization of the simplex
		if(m_Alpha==NULL)  StopM("TFOAM::Initialize: Cannot initialize buffer m_Alpha");
	}
	m_MCvect = new double[m_TotDim]; // vector generated in the MC run
	if(m_MCvect==NULL)  StopM("TFOAM::Initialize: Cannot initialize buffer m_MCvect");
	
	//====== variables related to MC cell exploration (projections edges) =====
	if(m_OptCu1st){
		m_N0Cu=0;   m_N0Si=m_kDim;              // hcubic dimensions are first
		m_P0Cu=0;   m_P0Si=m_kDim;
	}else{
		m_N0Cu=m_nDim;                m_N0Si=0; // simplical dimensions are first
		m_P0Cu=m_nDim*(m_nDim+1)/2;   m_P0Si=0;
	}
	m_nProj = m_nDim*(m_nDim+1)/2 +m_kDim;        // number of the PROJECTION edges
	//====== List of directions inhibited for division
	if(m_InhiDiv == NULL){
		m_InhiDiv = new int[m_kDim];
		for(int i=0; i<m_kDim; i++) m_InhiDiv[i]=0;    // Hcubic space only!!!
	}
	//====== Dynamic mask used in Explore for edge determibation
	if(m_MaskDiv == NULL){
		m_MaskDiv = new int[m_nProj];
		for(int i=0; i<m_nProj; i++) m_MaskDiv[i]=1;    // Hcub+Simplical
	}
	//====== List of predefined division values in all directions (initialized as empty)
	if(m_XdivPRD == NULL){
		m_XdivPRD = new TFVECT*[m_kDim];
		for(int i=0; i<m_kDim; i++)  m_XdivPRD[i]=NULL; // Artificialy extended beyond m_kDim
	}
	//====== Initialize list of histograms
	m_HistWt  = new CVMHistLite("m_HistWt",100,0.0,1.5*m_MaxWtRej);
	m_HistEdg = new CVMHistLite*[m_nProj]; // Initialize list of histograms
	for(int i=0;i<m_nProj;i++) m_HistEdg[i]= new CVMHistLite("m_HistEdg",m_nBin,0.0,1.0); // Initialize histogram for each edge
	if(m_HistEdg==NULL) StopM("TFOAM::Initializes: Cannot allocate m_HistEdg");
	
	// ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||| //
	//                     BUILD-UP of the FOAM                            //
	// ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||| //
	//        Allocate and Initialize BIG list of vertices
	if(m_nDim>0) InitVertices();
	//        PrintVertices(); std::cout<<" ===== after InitVerices ====="<<std::endl;
	//
	//        Allocate BIG list of cells, within cMax limit
	//        Define and explore root cell(s)
	InitCells();
	//        PrintCells(); std::cout<<" ===== after InitCells ====="<<std::endl;
	Grow();
	//        PrintCells(); std::cout<<" ===== after Grow      ====="<<std::endl;
	MakeActiveList(); // Final Preperations for the M.C. generation
	
	// Preperations for the M.C. generation
	m_SumWt  = 0.0;               // M.C. generation sum of Wt
	m_SumWt2 = 0.0;               // M.C. generation sum of Wt**2
	m_SumOve = 0.0;               // M.C. generation sum of overweighed
	m_NevGen = 0.0;               // M.C. generation sum of 1d0
	m_WtMax  = CVMGenFoam_MIN;               // M.C. generation maximum wt
	m_WtMin  = CVMGenFoam_MAX;               // M.C. generation minimum wt
	m_MCresult=m_Cells[0]->GetIntg(); // M.C. Value of INTEGRAL,temporary asignment
	m_MCresult=m_Cells[0]->GetIntg(); // M.C. Value of INTEGRAL,temporary asignment
	m_MCerror =m_Cells[0]->GetIntg(); // M.C. Value of ERROR   ,temporary asignment
	m_MCMonit = new TFMAXWT(5.0,1000);  // monitoring M.C. efficiency

	if(m_Chat>0){
		double Driver = m_Cells[0]->GetDriv();
		CVMGenFoam_BXOPE;
		CVMGenFoam_BXTXT("***  TFOAM::Initialize FINISHED!!!   ***");
		CVMGenFoam_BX1I("    nCalls",m_nCalls,  "Total number of function calls         ");
		CVMGenFoam_BX1F("    XPrime",m_Prime,   "Primary total integral                 ");
		CVMGenFoam_BX1F("    XDiver",Driver,    "Driver  total integral                 ");
		CVMGenFoam_BX1F("  MCresult",m_MCresult,"Estimate of the true MC Integral       ");
		CVMGenFoam_BXCLO;
	}
	if(m_Chat==2){ PrintCells(); PrintVertices(); }
} // Initialize
void CVMGenFoam::TFOAM::InitVertices(void){
// Alocate and Define components of vertices in the simplical subspace
// OptOrd=0: 2^nDim  initial vertices of the root unit hypercube
//           like (0000),(0001),(0010),(0011),(0100)...(1111)
// OptOrd=1:  nDim+1 initial  vertices of the root unit simplex
//           like (0000),(1001),(1100),(1110),(1111)
//
  long i,j, NoVert, nDivi, iVe;
  int k;
  TFPARTITION partition(m_nDim);

  if( m_VerX != NULL ) delete [] m_VerX;

  if(m_nDim==0) StopM("TFOAM::InitVertices: m_nDim=0");
  switch(m_OptOrd){
  case 0:
    ///////////////////////////////////////////////////////////////////////
    // cMax = 1+ nDim! +2*nDivi, vMax = 2^nDim + nDivi, where  nDivi     //
    // is the no. of binary divisions after split of the unit h-cube     //
    ///////////////////////////////////////////////////////////////////////
    nDivi= (m_nCells -1 - silnia(m_nDim))/2;
    if(nDivi < 1 ) StopM("TFOAM::Initialize: too big m_nDim or too small m_nCells");
	m_vMax= (long)(pow((double)2, m_nDim))+nDivi;
    if(m_Chat>0)std::cout << "TFOAM::InitVertices: projected number of vertices = "<<m_vMax<<std::endl;
    if(m_OptVert==1)  m_vMax= (long)(pow((double)2, m_nDim));    
    /////////////////////////////////////////////////////////
    //          Allocate BIG list of vertices              //
    m_VerX = new TFVECT*[m_vMax];
    for(i=0;i<m_vMax;i++)
      m_VerX[i]= new  TFVECT(m_nDim);
    if(m_VerX==NULL) StopM("TFOAM::InitVertices: Cannot initialize buffer for vertices");
    /////////////////////////////////////////////////////////
    //   Define components of the first 2^nDim vertices    //
    NoVert = (long)pow((double)2,m_nDim);
    iVe=0;
    partition.Reset();
    do {
      for(k=0; k<m_nDim; k++) (*m_VerX[iVe])[k] = (partition.Digit(k));
      iVe++;
      //std::cout<<"****> "; for(k=0; k<m_nDim; k++) std::cout<< (partition.Digit(k)) <<" ";std::cout<<std::endl;
    } while ( partition.Next() != 0);
    if(iVe != NoVert ) StopM("TFOAM::InitVertices: Something wrong with sum ove partitions");
    break;
  case 1:
    ///////////////////////////////////////////////////////////////////////
    // cMax = 1+ 2*nDivi,  vMax = (nDim+1) + nDivi                       //
    ///////////////////////////////////////////////////////////////////////
    nDivi= (m_nCells -1)/2;
    m_vMax= (m_nDim+1)+nDivi;
    std::cout << "TFOAM::InitVertices: projected number of vertices = "<<m_vMax<<std::endl;
    if(m_OptVert==1)  m_vMax=  m_nDim+1;    
    /////////////////////////////////////////////////////////
    //          Allocate BIG list of vertices              //
    m_VerX = new TFVECT*[m_vMax];
    for(i=0;i<m_vMax;i++)
      m_VerX[i]= new  TFVECT(m_nDim);
    if(m_VerX==NULL) StopM("TFOAM::InitVertices: Cannot initialize buffer for vertices");
    /////////////////////////////////////////////////////////
    //   Define components of the first nDim+1 vertices    //
    NoVert = m_nDim+1;
    for(i=0; i<NoVert; i++){
      for(j=0; j<m_nDim; j++){
	if(j>=i)
	  (*m_VerX[i])[j] = 0;
	else
	  (*m_VerX[i])[j] = 1;
      }
      //for(j=0; j<m_nDim; j++) std::cout<< (*(m_VerX+i))[j]<<" ";std::cout<<std::endl;
    }
    break;
  default:
    StopM("STOP in TFOAM::InitVertices - wrong m_OptOrd = ");  
  }//switch
  m_LastVe = NoVert-1;
  //PrintVertices(); std::cout<<" ===== exiting InitVerices ====="<<std::endl;
}
void CVMGenFoam::TFOAM::InitCells(void){
// Initialize "root part" of the FOAM of cells
  int i,j,k,Mask;
  long iVe,jser,iter,npow;
  int  *Perm,*digit;
  int    *Vertices =NULL;
  TFVECT *Posi =NULL;
  TFVECT *Size =NULL;
  //
  m_LastCe =-1;                             // Index of the last cell
  if(m_Cells!= NULL) {
    for(i=0; i<m_nCells; i++) delete m_Cells[i];
    delete [] m_Cells;
  }
  //
  m_Cells = new TFCELL*[m_nCells];
  for(i=0;i<m_nCells;i++){
    m_Cells[i]= new TFCELL(m_nDim,m_kDim,m_OptMCell,m_OptCu1st); // Allocate BIG list of cells
    m_Cells[i]->SetCell0(m_Cells);
    m_Cells[i]->SetVert0(m_VerX);
    m_Cells[i]->SetSerial(i);
  }
  if(m_Cells==NULL) StopM("TFOAM::InitCells: Cannot initialize CELLS");

  // define initial values of Posi Size vectors, if h-cubic is present
  if(m_kDim>0){
     Posi = new TFVECT(m_kDim);
     Size = new TFVECT(m_kDim);
     if(Posi==NULL||Size==NULL) StopM("TFOAM::nitCells: Cannot allocate Posi or Size");
     (*Posi)=0.0;
     (*Size)=1.0;
  }

  if(m_nDim==0 && m_kDim>0){
    /////////////////////////////////////////////////////////////////////////////
    //              Single Root Hypercube                                      //
    /////////////////////////////////////////////////////////////////////////////
    // Status, Parent,     Verts,  Posi, Size
    CellFill(1,   NULL,  Vertices, Posi, Size);  //  0-th cell ACTIVE
  }else if(m_nDim>0){
    Vertices = new int[m_nDim+1];       // ALLOCATE array of pointers to vertices
    switch(m_OptOrd){
    case 0:
      // Status, Parent, Verts,  Posi, Size
      CellFill(0,   NULL,  NULL,  NULL, NULL); // 0-th cell INACTIVE
      /////////////////////////////////////////////////////////////////////////////
      //   Splits unit cube into nDim! SIMPLICES                                 //
      //   Method of constructing permutation is primitive/simple                //
      //   It is good enough for our n<10 problem                                //
      /////////////////////////////////////////////////////////////////////////////
      // LOOP OVER Permutations of (0,1,2,3,...,m_Ndim-1) starts here
      npow = (long)pow((double)m_nDim,m_nDim);
      Perm  = new int[m_nDim];
      digit = new int[m_nDim];
      for(i=0; i<m_nDim; i++) Perm[i]=0; // start
      for(iter=0; iter<npow; iter++){
	Mask=1;
	for(i=0; i<m_nDim; i++)
	  for(j=i+1; j<m_nDim; j++)
	    if(Perm[i]==Perm[j]) Mask=0;
	if(Mask==1){ //         NEW PERMUTATION is found, define NEW simplical CELL
	  for(iVe=0;iVe<m_nDim+1;iVe++){
	    // This digit represents single BASIC SIMPLEX, other obtained by Permuting dimensions
	    for(k=0;k<m_nDim;k++){
	      digit[k]=0;
	      if(k<iVe) digit[k]=1;
	    }
	    // Translation from "binary" digit to serial pointer of a given vertex
	    jser=0;
	    for( k=0;k<m_nDim;k++)
	      jser= 2*jser+  digit[Perm[k]];
	    Vertices[iVe] = jser;      // FILL in pointers to vertices
	  }//iVe
	  // Status,     Parent,     Verts,  Posi, Size
	  CellFill(1, m_Cells[0],  Vertices,  Posi, Size);
	} // end of permutation exploitation
	Perm[m_nDim-1]=Perm[m_nDim-1]+1; // increment
	for(k=m_nDim-1; k>0; k--)
	  if(Perm[k]==m_nDim){ Perm[k]=0; Perm[k-1]=Perm[k-1]+1;}
      } //END OF LOOP OVER Permutations
      delete [] Perm;  // insure delete_mismatch
      delete [] digit; // insure delete_mismatch
      break;
    case 1:
      /////////////////////////////////////////////////////////////////////////////
      //   ROOT 0-th Cell is just SINGLE SIMPLEX, it is ACTIVE here of course!   //
      /////////////////////////////////////////////////////////////////////////////
      for( k=0;k<m_nDim+1;k++)
	Vertices[k] = k;          // FILL in pointers to vertices
      // Status, Parent,     Verts, Posi, Size
      CellFill(1,   NULL,  Vertices, Posi, Size);  //  0-th cell ACTIVE
      break;
    default:
      StopM("STOP in TFOAM::InitCells - wrong m_OptOrd");
    }//switch(OptOrd)
  }//if(nDim>0)

  // Exploration of the root cell(s)
  int iStart=0;                           // normal case
  if( m_OptOrd==0 && m_nDim>0 ) iStart=1; // the case of nDim! root cells
  for(long iCell=iStart; iCell<=m_LastCe; iCell++){
    Explore( m_Cells[iCell] );               // Exploration of root cell(s)
  }
  //PrintCells(); std::cout<<" ===== After exploration of root cells  ====="<<std::endl;


  //========================= CLEANUP ==============================
  if(Vertices!=NULL)   delete [] Vertices;
  if(    Posi!=NULL)   delete Posi;
  if(    Size!=NULL)   delete Size;
  //PrintCells(); std::cout<<" ===== exiting InitCells ====="<<std::endl;
}//InitCells


///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::LinkCells(void){
  // lift up certain paremeters of cells after re-read from disk
  for(int i=0;i<m_nCells;i++){
    m_Cells[i]->SetCell0(m_Cells);
    m_Cells[i]->SetVert0(m_VerX);
  }
  MakeActiveList(); // Final Preperations for the M.C. generation
}


///////////////////////////////////////////////////////////////////////////////
int CVMGenFoam::TFOAM::CellFill(int Status, CVMGenFoam::TFCELL *Parent, int *Vertices, CVMGenFoam::TFVECT *Posi, TFVECT *Size){
// Fills in cell all content of the new active cell
  TFCELL *Cell;
  if (m_LastCe==m_nCells){
	  std::cout << "TFOAM::CellFill: Too many cells" << std::endl; exit(0);
  }
  m_LastCe++;   // 0-th cell is the first
  if (Status==1) m_NoAct++;

  Cell = m_Cells[m_LastCe];

  int kParent = -1;
  if(Parent!=NULL) kParent =Parent->GetSerial();

  Cell->Fill(Status, kParent, -1, -1, Vertices, Posi, Size);

  Cell->SetBest( -1);         // pointer for planning division of the cell
  Cell->SetXdiv(0.5);         // factor for division
  double xInt2,xDri2;
  if(Parent!=NULL){
    xInt2  = 0.5*Parent->GetIntg();
    xDri2  = 0.5*Parent->GetDriv();
    Cell->SetIntg(xInt2);
    Cell->SetDriv(xDri2);
  }else{
    Cell->SetIntg(0.0);
    Cell->SetDriv(0.0);
  }
  return m_LastCe;
}
void CVMGenFoam::TFOAM::Explore(CVMGenFoam::TFCELL *Cell){
//   Explore newly defined cell with help of special short MC sampling
//   As a result, estimetes of true and drive volume will be defined
//   Average and dispersion of the weight distribution will be found along
//   each edge and the best edge (minimum dispersion) is memorized
//   for future use.
//   Axerage x for eventual future cell division is also defined.
//   Recorded are aso minimum and maximu weight etc.
//   The volume estimate in all (inactive) parent cells is updated
//   Note that links to parents and initial volume = 1/2 parent has to be
//   already defined prior to calling this routine.
  double Factorial;
  double Wt, Dx, Dxx, Vsum, xBest, yBest;
  double IntOld, DriOld;

  long iev;
  double NevMC;
  int iv, jv, i, j, k;
  int nProj, kBest;
  double CeSum[5], Xproj;

  TFVECT  Size(m_kDim);
  TFVECT  Posi(m_kDim);
  TFVECT  VRand(m_nDim),Lambda(m_nDim+1), X(m_nDim);
  Cell->GetHcub(Posi,Size);

  TFCELL  *Parent;
  TFMATRIX Yrel(m_nDim), Xrel(m_nDim), XVert(m_nDim+1); // m_nDim=0 is handled internaly

  double *xRand = new double[m_TotDim];

  double *VolPart=NULL;
  if(m_nDim>0){
    VolPart= new double[m_nDim+1];   // partial volumes for handling simplex geometry
    if(VolPart==NULL)   StopM("TFOAM::Initialize: Cannot initialize buffer VolPart");
                         // Table of vertex position
    for(iv=0; iv<m_nDim+1; iv++)
      {
	if(m_OptVert==0){
            for(j=0; j< m_nDim; j++)
            XVert(iv,j)=(*(*Cell)[iv])[j];
	    }
          else
            {
             X=0.;
             Lambda=0.;
             Cell->GetXSimp(X,Lambda,iv);
             for(j=0; j< m_nDim; j++)
             XVert(iv,j)=X[j];
            }
      }
  }//endif m_nDim

  Cell->CalcVolume();
  Dx = Cell->GetVolume();   // Dx includes simplical and h-cubical parts

  Factorial=silnia(m_nDim);           // m_nDim=0 is handled internaly

  IntOld = Cell->GetIntg(); //memorize old values,
  DriOld = Cell->GetDriv(); //will be needed for correcting parent cells

  if(m_OptVert==0)
  {
  if(m_nDim>0){     // decode vertex vectors, relative last vertex
    for(iv=0; iv< m_nDim; iv++)
      for(j=0; j< m_nDim; j++)
	Xrel(iv,j) =  (*(*Cell)[iv])[j] - (*(*Cell)[m_nDim])[j];
  }
  }
  /////////////////////////////////////////////////////
  //    Special Short MC sampling to probe cell      //
  /////////////////////////////////////////////////////
  CeSum[0]=0;
  CeSum[1]=0;
  CeSum[2]=0;
  CeSum[3]=CVMGenFoam_MAX;  //wtmin
  CeSum[4]=CVMGenFoam_MIN;  //wtmax
  //
  if(m_nProj>0) for(i=0;i<m_nProj;i++) m_HistEdg[i]->Reset();              // Reset histograms
  m_HistWt->Reset();
  //
  ///////////////////////////////////////////////////////////////////////
  // additional scan over vertices in order to improve max/min weights //
  int icont =0;
  TFPARTITION BiPart(m_kDim);
  if(m_OptEdge==1){
    for(iv=0; iv<m_nDim+1; iv++){      // loop over vertices in the simplex
      BiPart.Reset();
      do {                             // loop over vertices in hyp-cube
	for(j=0; j<m_nDim; j++)
	  xRand[m_N0Si+j]  = XVert(iv,j);
	for(k=0; k<m_kDim; k++) 
	  xRand[m_N0Cu+k] =  Posi[k] +(BiPart.Digit(k))*(Size[k]);
	Wt = m_Rho->Density(m_TotDim, xRand )*Dx; // <wt> normalised to integral over the cell
	m_nCalls++;
	icont++; if(icont>100) break; // protection against excesive scann
	if (CeSum[3]>Wt) CeSum[3]=Wt;
	if (CeSum[4]<Wt) CeSum[4]=Wt;
	//std::cout<<"****> "; for(k=0; k<m_kDim; k++) std::cout<< (BiPart.Digit(k)) <<" ";std::cout<<std::endl;
      } while ( BiPart.Next() != 0);
    }
  }

  // ||||||||||||||||||||||||||BEGIN MC LOOP|||||||||||||||||||||||||||||
  double NevEff;
  for(iev=0;iev<m_nSampl;iev++){
    MakeLambda();              // generate uniformly vector inside simplex
    MakeAlpha();               // generate uniformly vector inside hypercube

    if( m_OptVert && m_nDim>0 ){
        for(j=0; j<m_nDim; j++) Lambda[j]=m_Lambda[j];
        Lambda[m_nDim]=0.;
        VRand=0.;
        Cell->GetXSimp(VRand,Lambda,m_nDim);
        for(j=0; j<m_nDim; j++) xRand[m_N0Si+j]=VRand[j];
    }else{
        for(j=0; j<m_nDim; j++){
        xRand[m_N0Si+j]=(*(*Cell)[m_nDim])[j];
        for(iv=0; iv<m_nDim; iv++)
	  xRand[m_N0Si+j] += m_Lambda[iv]*Xrel(iv,j);
       }
    }
    if(m_kDim>0){
      for(j=0; j<m_kDim; j++)
	xRand[m_N0Cu+j]= Posi[j] +m_Alpha[j]*(Size[j]);
    }
    
    //-------------------------------------------------------------------------
    Wt = m_Rho->Density(m_TotDim, xRand )*Dx;   // <wt> normalised to integral over the cell
    //-------------------------------------------------------------------------
    // calculate partial volumes, necessary for projecting in simplex edges
    if(m_nDim>0){
      Vsum = 0.0;
      for(jv=0; jv<m_nDim+1; jv++){ // all vertices relative to xRand, jv is omitted
	k=0;
	for(iv=0; iv<m_nDim+1; iv++){
	  if(iv!=jv){  // vertex jv will be replaced with the random vertex
	    for(j=0; j<m_nDim; j++)
	      Yrel(k,j) = XVert(iv,j) - xRand[m_N0Si+j];
	    k++;
	  }
	}
	Dxx = Yrel.Determinant();
	VolPart[jv] = fabs(Dxx/Factorial);
	Vsum += VolPart[jv];
      }
      // this x-check only makes sense for pure simplical case!
      if( (m_kDim*m_nDim ==0) && (fabs(Vsum-Dx) > 1.0e-7*(fabs(Vsum)+fabs(Dx))) ){
	std::cout<<"++++ WARNING: TFOAM::Explore: Vsum= "<< Vsum <<" Dx= "<< Dx <<std::endl;
	//StopM("TFOAM::Explore: something wrong with volume calculation ");
      }
    }
    //=============================================
    // $**$ IMPORTANT! Two Loops below determine indexing of edges (simplex and hypercube)
    nProj = m_P0Si;
    if(m_nDim>0){
      for(jv=0; jv<m_nDim+1; jv++){
	for(iv=jv+1; iv<m_nDim+1; iv++){
	  Xproj = VolPart[jv]/(VolPart[jv]+VolPart[iv]);
	  m_HistEdg[nProj]->Fill(Xproj,Wt); // fill all histograms, search the best edge-candidate
	  nProj++;
	}
      }
    }
    nProj = m_P0Cu;
    if(m_kDim>0){
      for(k=0; k<m_kDim; k++){
	Xproj =m_Alpha[k];
	m_HistEdg[nProj]->Fill(Xproj,Wt); // fill all histograms, search the best edge-candidate
	nProj++;
      }
    }
    //
    m_nCalls++;
    CeSum[0] += Wt;    // sum of weights
    CeSum[1] += Wt*Wt; // sum of weights squared
    CeSum[2]++;        // sum of 1
    if (CeSum[3]>Wt) CeSum[3]=Wt;  // minimum weight;
    if (CeSum[4]<Wt) CeSum[4]=Wt;  // maximum weight
    // test MC loop exit condition
    NevEff = CeSum[0]*CeSum[0]/CeSum[1];
    if( NevEff >= m_nBin*m_EvPerBin) break;
  }   // ||||||||||||||||||||||||||END MC LOOP|||||||||||||||||||||||||||||
  //------------------------------------------------------------------
  //---  predefine logics of searching for the best division edge ---
  for(k=0; k<m_kDim;k++){
    m_MaskDiv[m_P0Cu+k] =1;                       // default is all
    if( m_InhiDiv[k]==1) m_MaskDiv[m_P0Cu +k] =0; // inhibit some...
  }
  // Note that predefined division below overrule inhibition above
  kBest=-1;
  double rmin,rmax,rdiv;
  if(m_OptPRD){          // quick check
    for(k=0; k<m_kDim; k++){
      rmin= Posi[k];
      rmax= Posi[k] +Size[k];
      if( m_XdivPRD[k] != NULL){
	int N= (m_XdivPRD[k])->GetDim();
	for(j=0; j<N; j++){
	  rdiv=(*m_XdivPRD[k])[j];
	  // check predefined divisions is available in this cell
	  if( (rmin +1e-99 <rdiv) && (rdiv< rmax -1e-99)){
	    kBest=k;
	    xBest= (rdiv-Posi[k])/Size[k] ;
	    goto ee05;
	  }
	}
      }
    }//k
  }
  ee05:
  //------------------------------------------------------------------
  m_nEffev += (long)NevEff;
  NevMC          = CeSum[2];
  double IntTrue = CeSum[0]/(NevMC+0.000001);
  double IntDriv, IntPrim;
  switch(m_OptDrive){
  case 1:                       // VARIANCE REDUCTION
    if(kBest == -1) Varedu(CeSum,kBest,xBest,yBest); // determine the best edge,
    //IntDriv =sqrt( CeSum[1]/NevMC -IntTrue*IntTrue ); // Older ansatz, numericaly not bad
    IntDriv =sqrt(CeSum[1]/NevMC) -IntTrue; // Foam build-up, sqrt(<w**2>) -<w>
    IntPrim =sqrt(CeSum[1]/NevMC);          // MC gen. sqrt(<w**2>) =sqrt(<w>**2 +sigma**2)
    break;
  case 2:                       // WTMAX  REDUCTION
    if(kBest == -1) Carver(kBest,xBest,yBest);  // determine the best edge
    IntDriv =CeSum[4] -IntTrue; // Foam build-up, wtmax-<w>
    IntPrim =CeSum[4];          // MC generation, wtmax!
    break;
  default:
    StopM("STOP in TFOAM::Explore - wrong m_OptDrive = ");
  }//switch
  //=================================================================================
  //hist_Neff_distrib.Fill( m_LastCe/2.0+0.01, NevEff+0.01);  // 
  //hist_kBest_distrib.Fill( kBest+0.50, 1.0 ); //  debug 
  //hist_xBest_distrib.Fill( xBest+0.01, 1.0 ); //  debug 
  //=================================================================================
  Cell->SetBest(kBest);
  Cell->SetXdiv(xBest);
  Cell->SetIntg(IntTrue);
  Cell->SetDriv(IntDriv);
  Cell->SetPrim(IntPrim);
  // correct/update integrals in all parent cells to the top of the tree
  double  ParIntg, ParDriv;
  for(Parent = Cell->GetPare(); Parent!=NULL; Parent = Parent->GetPare()){
    ParIntg = Parent->GetIntg();  
    ParDriv = Parent->GetDriv();  
    Parent->SetIntg( ParIntg   +IntTrue -IntOld );
    Parent->SetDriv( ParDriv   +IntDriv -DriOld );
  }
  delete [] VolPart;
  delete [] xRand;
  //Cell->Print();
} // TFOAM::Explore
void CVMGenFoam::TFOAM::Varedu(double CeSum[5], int &kBest, double &xBest, double &yBest){
// The case of the optimization of the maximu weight.
// Determine the best edge-candidate for future cell division,
// using results of the MC exploration run within the cell stored in m_HistEdg
  double Nent   = CeSum[2];
  double swAll  = CeSum[0];
  double sswAll = CeSum[1];
  double SSw    = sqrt(sswAll)/sqrt(Nent);
  //
  double SwIn,SwOut,SSwIn,SSwOut,xLo,xUp;
  kBest =-1;
  xBest =0.5;
  yBest =1.0;
  double MaxGain=0.0;
  // Now go over all projections kProj
  for(int kProj=0; kProj<m_nProj; kProj++)
    if( m_MaskDiv[kProj]){
    // initialize search over bins
    double SigmIn =0.0; double SigmOut =0.0;
    double SSwtBest = CVMGenFoam_MAX;
    double Gain =0.0;
    double xMin=0.0; double xMax=0.0; 
    // Double loop over all pairs jLo<jUp
    for(int jLo=1; jLo<=m_nBin; jLo++){
      double swIn=0;  double sswIn=0;
      for(int jUp=jLo; jUp<=m_nBin;jUp++){
		  swIn  +=     (m_HistEdg[kProj])->GetValueOrOverflow(jUp);
		  sswIn += sqr((m_HistEdg[kProj])->GetValueOrOverflow(  jUp));
		  xLo=(jLo-1.0)/m_nBin;
		  xUp=(jUp*1.0)/m_nBin;
		  SwIn  =        swIn/Nent;
		  SwOut = (swAll-swIn)/Nent;
		  SSwIn = sqrt(sswIn)       /sqrt(Nent*(xUp-xLo))     *(xUp-xLo);
		  SSwOut= sqrt(sswAll-sswIn)/sqrt(Nent*(1.0-xUp+xLo)) *(1.0-xUp+xLo);
		  if( (SSwIn+SSwOut) < SSwtBest){
			  SSwtBest = SSwIn+SSwOut;
			  Gain     = SSw-SSwtBest;
			  SigmIn   = SSwIn -SwIn;  // Debug
			  SigmOut  = SSwOut-SwOut; // Debug
			  xMin    = xLo;
			  xMax    = xUp;
		  }
	  }//jUp
	}//jLo
    int iLo = (int) (m_nBin*xMin);
    int iUp = (int) (m_nBin*xMax);
    //----------DEBUG printout
    //std::cout<<"@@@@@  xMin xMax = "<<xMin   <<" "<<xMax<<"  iLo= "<<iLo<<"  iUp= "<<iUp;
    //std::cout<<"  SSwtBest/SSw= "<<SSwtBest/SSw<<"  Gain/SSw= "<< Gain/SSw<<std::endl;
    //----------DEBUG auxilairy Plot
//#ifdef ROOT_DEF
//    if( m_OptDebug == 1) {
//      for(int iBin=1;iBin<=m_nBin;iBin++)
//	if( ((iBin-0.5)/m_nBin > xMin) && ((iBin-0.5)/m_nBin < xMax) ){
//	  ((TH1D *)(*m_HistDbg)[kProj])->SetBinContent(iBin,SigmIn/(xMax-xMin));
//	}else{
//	  ((TH1D *)(*m_HistDbg)[kProj])->SetBinContent(iBin,SigmOut/(1-xMax+xMin));
//	}
//    }
//#endif
    if(Gain>MaxGain){
      MaxGain=Gain;
      kBest=kProj; // <--- !!!!! The best edge
      xBest=xMin;
      yBest=xMax;
      if(iLo == 0   ) xBest=yBest; // The best division point
      if(iUp == m_nBin) yBest=xBest; // this is not realy used
    }
  }
  //----------DEBUG printout
  //std::cout<<"@@@@@@@>>>>> kBest= "<<kBest<<"  MaxGain/SSw= "<< MaxGain/SSw<<std::endl;
  if( (kBest >= m_nProj) || (kBest<0) ) StopM("TFOAM::Varedu: something wrong with kBest");
}          //TFOAM::Varedu

///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::Carver(int &kBest, double &xBest, double &yBest){
// The case of the optimization of the maximu weight.
// Determine the best edge-candidate for future cell division,
// using results of the MC exploration run within the cell stored in m_HistEdg
  int    kProj,iBin;
  double Carve,CarvTot,CarvMax,CarvOne,BinMax,BinTot,PrimTot,PrimMax;
  int    jLow,jUp,iLow,iUp;
  double TheBin;
  int    jDivi; // TEST

  double *Bins  = new double[m_nBin];      // bins of histogram for single  PROJECTION
  if(Bins==NULL)    StopM("TFOAM::Carver: Cannot initialize buffer Bins");
  
  kBest =-1;
  xBest =0.5;
  yBest =1.0;
  CarvMax = CVMGenFoam_MIN;
  PrimMax = CVMGenFoam_MIN;
  for(kProj=0; kProj<m_nProj; kProj++)
    if( m_MaskDiv[kProj] ){
    //if( kProj==1 ){
    //std::cout<<"==================== Carver histogram: kProj ="<<kProj<<"==================="<<std::endl;
    //((TH1D *)(*m_HistEdg)[kProj])->Print("all");
    BinMax = CVMGenFoam_MIN;
    for(iBin=0; iBin<m_nBin;iBin++){
		Bins[iBin]= (m_HistEdg[kProj])->GetValueOrOverflow(iBin+1);      // Unload histogram
		BinMax = dmax( BinMax, Bins[iBin]);       // Maximum content/bin
    }
    if(BinMax < 0 ) {       //case of empty cell
      delete [] Bins;
      return;
    }
    CarvTot = 0.0;
    BinTot  = 0.0;
    for(iBin=0;iBin<m_nBin;iBin++){
      CarvTot = CarvTot + (BinMax-Bins[iBin]);     // Total Carve (more stable)
      BinTot  +=Bins[iBin];
    }
    PrimTot = BinMax*m_nBin;
     //std::cout <<"Carver:  CarvTot "<<CarvTot<< "    PrimTot "<<PrimTot<<std::endl;
    jLow =0;
    jUp  =m_nBin-1;
    CarvOne = CVMGenFoam_MIN;
    double Ylevel = CVMGenFoam_MIN;
    for(iBin=0; iBin<m_nBin;iBin++){
      TheBin = Bins[iBin];
      //-----  walk to the left and find first bin > TheBin
      iLow = iBin;
      for(int j=iBin; j>-1; j-- ) {
	if(TheBin< Bins[j]) break;
	iLow = j;
      }
      //iLow = iBin;
      //if(iLow>0)     while( (TheBin >= Bins[iLow-1])&&(iLow >0) ){iLow--;} // horror!!!
      //------ walk to the right and find first bin > TheBin
      iUp  = iBin;
      for(int j=iBin; j<m_nBin; j++){
	if(TheBin< Bins[j]) break;
	iUp = j;
      }
      //iUp  = iBin;
      //if(iUp<m_nBin-1) while( (TheBin >= Bins[iUp+1])&&( iUp<m_nBin-1 ) ){iUp++;} // horror!!!
      //
      Carve = (iUp-iLow+1)*(BinMax-TheBin);
      if( Carve > CarvOne){
	CarvOne = Carve;
	jLow = iLow;
	jUp  = iUp;
	Ylevel = TheBin;
      }
    }//iBin
    if( CarvTot > CarvMax){
      CarvMax   = CarvTot;
      PrimMax   = PrimTot;
      //std::cout <<"Carver:   PrimMax "<<PrimMax<<std::endl;
      kBest = kProj;    // Best edge
      xBest = ((double)(jLow))/m_nBin;
      yBest = ((double)(jUp+1))/m_nBin;
      if(jLow == 0 )       xBest = yBest;
      if(jUp  == m_nBin-1) yBest = xBest;
      // division ratio in units of 1/m_nBin, testing
      jDivi = jLow;
      if(jLow == 0 )     jDivi=jUp+1;
    }
    //======  extra histograms for debug purposes
    //std::cout<<"kProj= "<<kProj<<" jLow= "<<jLow<<" jUp= "<<jUp<<std::endl;
//#ifdef ROOT_DEF
//    if( m_OptDebug == 1) {
//      for(iBin=0;    iBin<m_nBin;  iBin++)
//	((TH1D *)(*m_HistDbg)[kProj])->SetBinContent(iBin+1,BinMax);
//      for(iBin=jLow; iBin<jUp+1;   iBin++)
//	((TH1D *)(*m_HistDbg)[kProj])->SetBinContent(iBin+1,Ylevel);
//    }
//#endif
  }//kProj
	if( (kBest >= m_nProj) || (kBest<0) ) {
		std::cout<<"kBest="<<kBest<<", m_nProj="<<m_nProj<<std::endl; StopM("TFOAM::Carver: something wrong with kBest");
	}
  delete [] Bins;
}          //TFOAM::Carver
void CVMGenFoam::TFOAM::MakeAlpha(void){
// HYP-CUBICAL SUBSPACE
// Provides random vector Alpha  0< Alpha(i) < 1
  int k;
  if(m_kDim<1) return;

  // simply generate and load kDim uniform random numbers
  m_PseRan->RndArr(m_kDim,m_Rvec);   // kDim random numbers needed
  for(k=0; k<m_kDim; k++) m_Alpha[k] = (double)m_Rvec[k];
} //MakeAlpha
void CVMGenFoam::TFOAM::MakeLambda(void){
// SIMPLICAL SUBSPACE
// Provides random vector Lambda such that Sum Lamba(i) < 1, with uniform
// probability. This  vector is used to populate uniformly the interior
// of a simplex. The method is: generate point inside cube, order components
// (maping into simplex) and take differences of Lambda(i+1) - Lambda(i)
// For higher dimension random-walk method is used instead of ordering.
  int i,k;
  double x, sum;
  if(m_nDim<1) return;

  int nDimMax = 4; // maximum dimension for bubble-sort ordering method

  if (m_nDim>nDimMax){                   // faster random-walk algorithm for high dimensions
    m_PseRan->RndArr(m_nDim+1,m_Rvec);  // nDim+1 random numbers needed
    sum = 0.0;
    for(i=0; i<=m_nDim; i++){
      sum += -log((double)m_Rvec[i]);
      m_Rvec[i]=sum;                     // ordering is here automatic
    }
    for(i=0; i<m_nDim; i++) m_Rvec[i] = m_Rvec[i] /m_Rvec[m_nDim]; // normalize to last one
  }else{                                 // bubble-sort ordering (maping cube->simplex)
    m_PseRan->RndArr(m_nDim,m_Rvec);    // nDim random numbers needed
    for(i=m_nDim-1; i>=0; i--){
      for(k=1; k<=i; k++){
        if ( m_Rvec[k] < m_Rvec[k-1] ){
          x = (double)m_Rvec[k]; m_Rvec[k] = m_Rvec[k-1]; m_Rvec[k-1] = x;} // get ordering
      }
    }
  }
  m_Lambda[0] = (double)m_Rvec[0];
  for(k = 1 ; k < m_nDim ; k++)
    m_Lambda[k] = (double)(m_Rvec[k] - m_Rvec[k-1]); // Sum of lambda's should < 1 !!!
}   //MakeLambda
void CVMGenFoam::TFOAM::Grow(void){
//  Grow new cells by division
  long iCell;
  TFCELL* newCell;

  while ( (m_LastCe+2) < m_nCells ){  // this condition also checked inside Divide
    switch (m_OptPeek){
    case 0:
      iCell   = PeekMax();        // peek up cell with maximum driver integral
      if( (iCell<0) || (iCell>m_LastCe) ) StopM("TFOAM::Grow: wrong iCell");
      newCell = m_Cells[iCell];
      break;
    case 1:
      newCell = PeekRan();        // peek up randomly one cell, tree-wise algorithm
      break;
    default:
      StopM("TFOAM::Grow: wrong value of m_OptPeek");
    }
    if(m_LastCe !=0){
      int kEcho=10;
      if(m_LastCe>=10000) kEcho=100;
      if( (m_LastCe%kEcho)==0){
		  if(m_Chat>0) {
			  if(m_nDim+m_kDim<10) std::cout<<m_nDim+m_kDim<<std::flush;
			  else std::cout<<"."<<std::flush;
			  if( (m_LastCe%(100*kEcho))==0)  std::cout<<"|"<<m_LastCe<<std::endl<<std::flush;
		  }
      }
    }
    //
    if( Divide( newCell )==0) break;  // and divide it into two
  }
  if(m_Chat>0)std::cout<<std::endl<<std::flush;
  CheckAll(0);   // set arg=1 for more info
}// Grow
long  CVMGenFoam::TFOAM::PeekMax(void){
// Initialization, Finds cell with maximal Driver integral
   long  i;
   long iCell = -1;
   double  DrivMax, Driv;

   DrivMax = -1E+150;
   for(i=0; i<=m_LastCe; i++) //without root
   {
      if( m_Cells[i]->GetStat() == 1 )
      {
         Driv =  fabs( m_Cells[i]->GetDriv());
	 //std::cout<<"PeekMax: Driv = "<<Driv<<std::endl;
         if(Driv > DrivMax)
         {
            DrivMax = Driv;
            iCell = i;
         }
      }
   }
//  std::cout << 'TFOAM_PeekMax: iCell=' << iCell <<std::endl;
   if (iCell == -1)
      std::cout << "STOP in TFOAM::PeekMax: not found iCell=" <<  iCell <<std::endl;
   return(iCell);
}                 // TFOAM_PeekMax
CVMGenFoam::TFCELL* CVMGenFoam::TFOAM::PeekRan(void){
// Initialization phase.
//       Tree-wise algorithm
//       Peek up randomly pointer iCell of an active cell
// We walk randomly from top of tree downwards until we find active
// cell m_CeStat=1
// At each step one of daugters cells is choosen randomly according
// to their volume estimates.
   long    iCell,iDau,nDaut;
   double  random,p1,volu1,volu2,sum;
   //
   TFCELL *newCell = m_Cells[0];
   //
   if( (m_nDim>1)&&(m_OptOrd==0)){
     // 0-th cell is special because has nDim! daughters
     double TotDriv = 0.0;
     double TotIntg = 0.0;
     nDaut = silnia(m_nDim);
     for(iCell= 1 ; iCell<=nDaut ; iCell++){ // without root
       //(m_Cells+iCell)->Print();
       TotDriv += m_Cells[iCell]->GetDriv();
       TotIntg += m_Cells[iCell]->GetIntg();
     }
     if( (fabs(TotDriv -m_Cells[0]->GetDriv() ) > 1.0e-5*TotDriv) )
       StopM("STOP in TFOAM::Peek -> something wrong with total driver integral");
     if( (fabs(TotIntg -m_Cells[0]->GetIntg() ) > 1.0e-5*TotDriv) )
       StopM("STOP in TFOAM::Peek -> something wrong with total true integral");
     random=m_PseRan->Rnd();
     iDau  = 0;
     sum   = 0.0;
     for(iCell = 1 ; iCell<=nDaut ; iCell++){     //without root
       iDau = iCell;
       sum += m_Cells[iCell]->GetDriv();
       if( random < sum/TotDriv ) break;
     }
     if (iDau<1 ) StopM("TFOAM::PeekRan: something went wrong, iDau<1 !!!!");
     newCell = m_Cells[iDau];
     if( newCell->GetStat() == 1 ) return newCell;
   }
   // now the other standard cells with 2 daughters
   while (   newCell->GetStat() != 1 ){
     volu1 = newCell->GetDau0()->GetDriv();
     volu2 = newCell->GetDau1()->GetDriv();
     p1 = volu1/(volu1+volu2);
     random=m_PseRan->Rnd();
     if( random < p1 )
       newCell = newCell->GetDau0();
     else
       newCell = newCell->GetDau1();
   }
   return newCell;
}        //Peek
int CVMGenFoam::TFOAM::Divide(CVMGenFoam::TFCELL *Cell){
//  Divide cell iCell into two daughter cells.
//  The iCell is retained and taged as inactive, daughter cells are appended
//  at the end of the buffer.
//  New vertex is added to list of vertices.
//  List of active cells is updated, iCell remooved, two daughters added
//  and their properties set with help of MC sampling (TFOAM_Explore)
//  Return Code RC=-1 of buffer limit is reached,  m_LastCe=m_nBuf
  double Xdiv;
  int  *kVer1 =NULL;
  int  *kVer2 =NULL;
  TFVECT *Posi1  =NULL;
  TFVECT *Size1  =NULL;
  TFVECT *Posi2  =NULL;
  TFVECT *Size2  =NULL;
  TFVECT *NewVert, *OldVe1, *OldVe2;
  int   kBest, j, jv, Old1,Old2;

  if(m_LastCe+1 >= m_nCells) StopM("TFOAM::Divide: buffer limit is reached, m_LastCe=m_nBuf");

  if(m_nDim>0 && m_OptVert==0){
    kVer1 = new int[m_nDim+1];  // array of pointers to vertices for daughter1
    kVer2 = new int[m_nDim+1];  // array of pointers to vertices for daughter2
    if(kVer1==NULL||kVer2==NULL) StopM("TFOAM::Divide: Cannot initialize buffer kVer");
    for(jv=0; jv<m_nDim+1; jv++){
      kVer1[jv] = (*Cell)(jv);   //inherit vertex pointers from parent
      kVer2[jv] = (*Cell)(jv);   //inherit vertex pointers from parent
    }
  }
  if( (m_kDim>0)  && (m_OptMCell==0) ){
    Posi1 = new TFVECT(m_kDim);
    Posi2 = new TFVECT(m_kDim);
    Size1 = new TFVECT(m_kDim);
    Size2 = new TFVECT(m_kDim);
    Cell->GetHcub( *Posi1 , *Size1 );   //inherit from parent
    Cell->GetHcub( *Posi2 , *Size2 );   //inherit from parent
  }

  Cell->SetStat(0); // reset Cell as inactive
  m_NoAct--;

  Xdiv  = Cell->GetXdiv();
  kBest = Cell->GetBest();
  if( kBest<0 || kBest>=m_nProj ) StopM("TFOAM::Divide: wrong kBest");

  int nProj0 = (m_nDim+1)*m_nDim/2;
  if( ( m_P0Si <= kBest ) && ( kBest < m_P0Si+nProj0 ) ){
    if(m_OptVert==0){
    // decoding pair (Old1,Old2) from kBest (a bit tricky method, see $**$)
    Old1 =0; Old2 =0;
    for(j=m_P0Si; j<=kBest; j++){
      Old2++;
      if( Old2 > m_nDim ){ Old1++; Old2 =Old1+1; }
    }
    OldVe1 =(*Cell)[Old1];
    OldVe2 =(*Cell)[Old2];
    if (m_LastVe+1>m_vMax) StopM("STOP in TFOAM::Divide: too short list of vertices ");
    m_LastVe++;                  // add new vertex to the list
    NewVert  =m_VerX[m_LastVe];  // pointer of new vertex
    for(j=0; j<m_nDim; j++)      // define components of new vertex using Xdiv and (Old1,Old2)
      (*NewVert)[j] = Xdiv* (*OldVe1)[j] + (1.0-Xdiv)* (*OldVe2)[j];
    kVer1[Old1] = m_LastVe;     // old pointer is overwriten
    kVer2[Old2] = m_LastVe;     // old pointer is overwriten
    }
  }else{
    if( m_OptMCell==0 ) {
      ////////////////////////////////////////////////////////////////
      //   The best division is in hyp-cubical subspace             //
      ////////////////////////////////////////////////////////////////
      int kDiv = kBest-m_P0Cu;
      if(kDiv<0||kDiv>=m_kDim) StopM("Divide: wrong kDiv");  // kDiv is form zero to m_kDim-1 !!!
      (*Posi1)[kDiv]=                   (*Posi1)[kDiv];      // (1) Position unchanged
      (*Size1)[kDiv]=              Xdiv*(*Size1)[kDiv];      // (1) Size reduced by Xdiv
      (*Posi2)[kDiv]=   (*Posi1)[kDiv] +(*Size1)[kDiv];      // (2) Position shifted
      (*Size2)[kDiv]=        (1.0-Xdiv)*(*Size2)[kDiv];      // (2) Size reduced by (1-Xdiv)
    }
  }
  //////////////////////////////////////////////////////////////////
  //           define two daughter cells (active)                 //
  //////////////////////////////////////////////////////////////////
  //          Status, Parent, Verts,  Posi, Size
  int d1 = CellFill(1,   Cell, kVer1, Posi1, Size1);
  int d2 = CellFill(1,   Cell, kVer2, Posi2, Size2);
  Cell->SetDau0(d1);
  Cell->SetDau1(d2);
  Explore( (m_Cells[d1]) );
  Explore( (m_Cells[d2]) );
  // CLEANUP
  if(kVer1 != NULL)  delete [] kVer1;
  if(kVer2 != NULL)  delete [] kVer2;
  if(Posi1 != NULL)  delete Posi1;
  if(Posi2 != NULL)  delete Posi2;
  if(Size1 != NULL)  delete Size1;
  if(Size2 != NULL)  delete Size2;
  return 1;
} // TFOAM_Divide
void CVMGenFoam::TFOAM::MakeActiveList(void){
  //  Finds out number of active cells m_NoAct,
  //  Creates list of active cell m_CellsAct and primary cumulative m_PrimAcu
  //  They are used during MC to choose randomly an active cell
  long n, iCell;
  double sum;
  // fush previous result
  if(m_PrimAcu  != NULL) delete [] m_PrimAcu;
  if(m_CellsAct != NULL) delete [] m_CellsAct;
  // Count Active cells and find total Primary
  m_Prime = 0.0; n = 0;
  for(iCell=0; iCell<=m_LastCe; iCell++){
    if (m_Cells[iCell]->GetStat()==1){
      m_Prime += m_Cells[iCell]->GetPrim();
      n++;
    }
  }
  // Allocate tables of active cells
  m_NoAct = n;
  m_CellsAct = new TFCELL*[m_NoAct]; // list of pointers
  m_PrimAcu  = new  double[m_NoAct]; // acumulative primary for MC generation
  if( m_CellsAct==NULL || m_PrimAcu==NULL ) StopM("MakeActiveList - Cant allocate m_CellsAct or m_PrimAcu");

  // Fill-in tables of active cells
  n = 0;
  for(iCell=0; iCell<=m_LastCe; iCell++){
    if( m_Cells[iCell]->GetStat() == 1 ){
      m_CellsAct[n] = m_Cells[iCell]; // ?????
      n++;
    }
  }
  sum =0.0;
  for(iCell=0; iCell<m_NoAct; iCell++){
    sum = sum +(m_CellsAct[iCell])->GetPrim()/m_Prime;
    m_PrimAcu[iCell]=sum;
  }
  
} //MakeActiveList
void CVMGenFoam::TFOAM::GenerCell(TFCELL *&pCell){
//  Generation phase.
//  Return randomly chosen active cell with probability equal to its
//  contribution into total driver integral using binary search .
  long  lo=0, hi=m_NoAct-1, hit;
  double Random;

  Random=m_PseRan->Rnd();
  while(lo+1<hi){
    hit = (lo+hi)/2;
    if (m_PrimAcu[hit]>Random)
      hi = hit;
    else
      lo = hit;
  }
  if (m_PrimAcu[lo]>Random)
    pCell = m_CellsAct[lo];
  else
    pCell = m_CellsAct[hi];
}       // TFOAM::GenerCell
void CVMGenFoam::TFOAM::GenerCel2(TFCELL *&pCell){
// Generation phase
// Return randomly chosen active cell with probability equal to its
// contribution into total driver integral using interpolation search
  long  lo, hi, hit;
  double fhit, flo, fhi;
  double Random;

  Random=m_PseRan->Rnd();
  lo  = 0;              hi =m_NoAct-1;
  flo = m_PrimAcu[lo];  fhi=m_PrimAcu[hi];
  while(lo+1<hi){
    hit = lo + (int)( (hi-lo)*(Random-flo)/(fhi-flo)+0.5);
    if (hit<=lo)
      hit = lo+1;
    else if(hit>=hi)
      hit = hi-1;
    fhit=m_PrimAcu[hit];
    if (fhit>Random){
      hi = hit;
      fhi = fhit;
    }else{
      lo = hit;
      flo = fhit;
    }
  }
  if (m_PrimAcu[lo]>Random)
    pCell = m_CellsAct[lo];
  else
    pCell = m_CellsAct[hi];
}       // TFOAM::GenerCel2
void CVMGenFoam::TFOAM::MakeEvent(void){
// Generation phase.
// Generates point/vector Xrand with the weight MCwt.
  int      j,iv;
  double   Wt,Dx,MCwt;
  TFCELL *rCell;
  TFVECT  VRand(m_nDim),Lambda(m_nDim+1), X(m_nDim);

  int OptGener =2;      // local dip-switch for tests!!!
  //
  //********************** MC LOOP STARS HERE **********************
 ee0:
  if(OptGener==1){
    GenerCell(rCell);   // primitive - more effective for small number of cells (?)
  }else{
    GenerCel2(rCell);   // choose randomly one cell
  }
  MakeLambda();
  MakeAlpha();

    if( m_OptVert && m_nDim>0 ){
        for(j=0; j<m_nDim; j++) Lambda[j]=m_Lambda[j];
        Lambda[m_nDim]=0.;
        VRand=0.;
        rCell->GetXSimp(VRand,Lambda,m_nDim);
        for(j=0; j<m_nDim; j++) 
           m_MCvect[m_N0Si+j]=VRand[j];
    }else{
      for(j=0 ;  j<m_nDim ; j++){
      m_MCvect[m_N0Si+j] = (*(*rCell)[m_nDim])[j];
      for(iv=0 ; iv <m_nDim ; iv++)
         m_MCvect[m_N0Si+j] += m_Lambda[iv] *( (*(*rCell)[iv])[j] - (*(*rCell)[m_nDim])[j] );
       }
    }
  TFVECT  Posi(m_kDim); TFVECT  Size(m_kDim);
  rCell->GetHcub(Posi,Size);
  for(j=0; j<m_kDim; j++)
    m_MCvect[m_N0Cu+j]= Posi[j] +m_Alpha[j]*Size[j];
  Dx = rCell->GetVolume();      // Cartesian volume of the Cell
  //  weight average normalised to PRIMARY integral over the cell
  Wt = m_Rho->Density(m_TotDim, m_MCvect )*Dx;
  MCwt = Wt / rCell->GetPrim();  // PRIMARY controls normalization
  m_nCalls++;
  m_MCwt   =  MCwt;
  // accumulation of statistics for the main MC weight
  m_SumWt  += MCwt;           // sum of Wt
  m_SumWt2 += MCwt*MCwt;      // sum of Wt**2
  m_NevGen++;                 // sum of 1d0
  m_WtMax  =  dmax(m_WtMax, MCwt);   // maximum wt
  m_WtMin  =  dmin(m_WtMin, MCwt);   // minimum wt
  m_MCMonit->Fill(MCwt);
  m_HistWt->Fill(MCwt,1.0);          // histogram
  //*******  Optional rejection ******
  if(m_OptRej == 1){
    double Random;
    Random=m_PseRan->Rnd();
    if( m_MaxWtRej*Random > m_MCwt) goto ee0;  // Wt=1 events, internal rejection 
    if( m_MCwt<m_MaxWtRej ){
      m_MCwt = 1.0;                  // normal Wt=1 event
    }else{
      m_MCwt = m_MCwt/m_MaxWtRej;    // weight for overveighted events! kept for debug
      m_SumOve += m_MCwt-m_MaxWtRej; // contribution of overveighted
    }
  }
  //********************** MC LOOP ENDS HERE **********************
} // MakeEvent
void CVMGenFoam::TFOAM::GetMCvect(double *MCvect){
//  Returns generated point/vector 
  for ( int k=0 ; k<m_TotDim ; k++) *(MCvect +k) = m_MCvect[k];
}//GetMCvect
void CVMGenFoam::TFOAM::GetMCvectR(tReal *MCvect){
//  Returns generated point/vector 
  for ( int k=0 ; k<m_TotDim ; k++) *(MCvect +k) = m_MCvect[k];
}//GetMCvect
///////////////////////////////////////////////////////////////////////////////
double CVMGenFoam::TFOAM::GetMCwt(void){
//   Returns point/vector MCvect weight MCwt
  return(m_MCwt);
}
///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::GetMCwt(double &MCwt){
//   Returns point/vector MCvect weight MCwt
  MCwt=m_MCwt;
}
double CVMGenFoam::TFOAM::MCgenerate(tReal *MCvect){
// Generates event and returns weight MCvect
   MakeEvent();
   GetMCvectR(MCvect);
   return(m_MCwt);
}//MCgenerate
void CVMGenFoam::TFOAM::GetIntegMC(double &MCresult, double &MCerror){
// Provides integral resulting calculated from averages of the MC run
// To be called after (or during) MC run
  double MCerelat;
  MCresult = 0.0;
  MCerelat = 1.0;
  if (m_NevGen>0){
    MCresult = m_Prime*m_SumWt/m_NevGen;
    MCerelat = sqrt( m_SumWt2/(m_SumWt*m_SumWt) - 1/m_NevGen);
  }
  MCerror = MCresult *MCerelat;
}//GetIntegMC

///////////////////////////////////////////////////////////////////////////////
void  CVMGenFoam::TFOAM::GetIntNorm(double& IntNorm, double& Errel ){
// Returns NORMALIZATION integral
// To be called after (or during) MC run
  if(m_OptRej == 1){    // Wt=1 events, internal rejection
    double IntMC,ErrMC;
    GetIntegMC(IntMC,ErrMC);
    IntNorm = IntMC;
    Errel   = ErrMC;
  }else{                // Wted events, NO internal rejection
    IntNorm = m_Prime;
    Errel   = 0;
  }
}//GetIntNorm

/////////////////////////////////////////////////////////////////////////////////////////
void  CVMGenFoam::TFOAM::GetWtParams(const double eps, double &AveWt, double &WtMax, double &Sigma){
// Returns parameters of the MC weight for efficiency evaluation
  double MCeff, WtLim;
  m_MCMonit->GetMCeff(eps, MCeff, WtLim);
  WtMax = WtLim;
  AveWt = m_SumWt/m_NevGen;
  Sigma = sqrt( m_SumWt2/m_NevGen -AveWt*AveWt );
}//GetMCeff

/////////////////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::Finalize(double& IntNorm, double& Errel ){
// Finalization phase.
// After MC run is completed it provides normalization and
// also prints some information/statistics on the MC run.
  GetIntNorm(IntNorm,Errel);
  double MCresult,MCerror;
  GetIntegMC(MCresult,MCerror);
  double MCerelat= MCerror/MCresult;
  //
  if(m_Chat>0){
    double eps = 0.0005;
    double MCeff, MCef2, WtMax, AveWt, Sigma;
    GetWtParams(eps, AveWt, WtMax, Sigma);
    MCeff=0;
    if(WtMax>0.0) MCeff=AveWt/WtMax;
    MCef2 = Sigma/AveWt;
    double Driver = m_Cells[0]->GetDriv();
    //
    CVMGenFoam_BXOPE;
    CVMGenFoam_BXTXT("****************************************");
    CVMGenFoam_BXTXT("******      TFOAM::Finalize       ******");
    CVMGenFoam_BXTXT("****************************************");
    CVMGenFoam_BX1I("    NevGen",m_NevGen, "Number of generated events in the MC generation   ");
    CVMGenFoam_BX1I("    LastVe",m_LastVe, "Number of vertices (only for simplical option)    ");
    CVMGenFoam_BX1I("    nCalls",m_nCalls, "Total number of function calls                    ");
    CVMGenFoam_BXTXT("----------------------------------------");
    CVMGenFoam_BX1F("     AveWt",AveWt,    "Average MC weight                      ");
    CVMGenFoam_BX1F("     WtMin",m_WtMin,  "Minimum MC weight (absolute)           ");
    CVMGenFoam_BX1F("     WtMax",m_WtMax,  "Maximum MC weight (absolute)           ");
    CVMGenFoam_BXTXT("----------------------------------------");
    CVMGenFoam_BX1F("    XPrime",m_Prime,  "Primary total integral, R_prime        ");
    CVMGenFoam_BX1F("    XDiver",Driver,   "Driver  total integral, R_loss         ");
    CVMGenFoam_BXTXT("----------------------------------------");
    CVMGenFoam_BX2F("    IntMC", MCresult,  MCerror,      "Result of the MC Integral");
    CVMGenFoam_BX1F(" MCerelat", MCerelat,  "Relative error of the MC intgral       ");
    CVMGenFoam_BX1F(" <w>/WtMax",MCeff,     "MC efficiency, acceptance rate");
    CVMGenFoam_BX1F(" Sigma/<w>",MCef2,     "MC efficiency, variance/ave_wt");
    CVMGenFoam_BX1F("     WtMax",WtMax,     "WtMax(esp= 0.0005)            ");
    CVMGenFoam_BX1F("     Sigma",Sigma,     "variance of MC weight         ");
    if(m_OptRej==1){
       double AvOve=m_SumOve/m_SumWt;
    CVMGenFoam_BX1F("<OveW>/<W>",AvOve,     "Contrib. of events wt>MaxWtRej");
    }
    CVMGenFoam_BXCLO;
  }
}  // Finalize

///////////////////////////////////////////////////////////////////////////////
void  CVMGenFoam::TFOAM::SetInhiDiv(int iDim, int InhiDiv){
// This should be called before Initialize, after setting nDim and kDim
  if(m_kDim==0) StopM("TFOAM::SetInhiDiv: m_kDim=0");
  if(m_InhiDiv == NULL){
    m_InhiDiv = new int[ m_kDim ];
    for(int i=0; i<m_kDim; i++) m_InhiDiv[i]=0;
  }
  //
  if( ( 0<=iDim) && (iDim<m_kDim)){
    m_InhiDiv[iDim] = InhiDiv;
  }else
    StopM("TFOAM::SetInhiDiv: wrong iDim");
}//SetInhiDiv

///////////////////////////////////////////////////////////////////////////////
void  CVMGenFoam::TFOAM::SetXdivPRD(int iDim, int len, double xDiv[]){
// This should be called before Initialize, after setting nDim and kDim
// 0<=iDim<_kDim is in H-cubic subspace!!!
  if(m_kDim<=0)  StopM("TFOAM::SetXdivPRD: m_kDim=0");
  if(   len<1 )  StopM("TFOAM::SetXdivPRD: len<1");
  // allocate list of pointers, if it was not done before
  if(m_XdivPRD == NULL){
    m_XdivPRD = new TFVECT*[m_kDim];
    for(int i=0; i<m_kDim; i++)  m_XdivPRD[i]=NULL; // only for hcubic subspace.
  }
  // set division list for direction iDim in H-cubic subspace!!!
  if( ( 0<=iDim) && (iDim<m_kDim)){
    m_OptPRD =1;      // !!!!
    if( m_XdivPRD[iDim] != NULL)
      StopM("TFOAM::SetXdivPRD: second alocation of XdivPRD not allowed");
    m_XdivPRD[iDim] = new TFVECT(len); // allocate list of division
    for(int i=0; i<len; i++){
      (*m_XdivPRD[iDim])[i]=xDiv[i]; // set list of division points
    }
  }else
    StopM("TFOAM::SetXdivPRD: wrong iDim");
  //[[[[[[[[[[[[[[[[[[[[[
  if(m_Chat>0){
	  std::cout<<" SetXdivPRD, idim= "<<iDim<<"  len= "<<len<<"   "<<std::endl;
	  for(int i=0; i<len; i++){ std::cout<< (*m_XdivPRD[iDim])[i] <<"  "; }
	  std::cout<<std::endl;
	  for(int i=0; i<len; i++)  std::cout<< xDiv[i] <<"   ";
	  std::cout<<std::endl;
  }
  //]]]]]]]]]]]]]]]]]]]]]
}//SetXdivPRD

///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::CheckAll(const int level){
//  Miscelaneous and debug.
//  Checks all pointers, this is useful autodiagnostic.
//  level=0, no printout, failures causes STOP
//  level=1, printout, failures lead to WARNINGS only
  int Errors, Warnings, Ref;
  TFCELL *Cell;
  long iCell,iVert,jCe;
  int k;

  Errors = 0; Warnings = 0;
  if (level==1) std::cout << "///////////////////////////// FOAM_Checks /////////////////////////////////" <<std::endl;
  m_NoAct=0;
  for(iCell=1; iCell<=m_LastCe; iCell++){ // without root
    Cell = m_Cells[iCell];
    if (Cell->GetStat()==1)
      m_NoAct++;
//  checking general rules
    if( ((Cell->GetDau0()==NULL) && (Cell->GetDau1()!=NULL) ) ||
        ((Cell->GetDau1()==NULL) && (Cell->GetDau0()!=NULL) ) ){
      Errors++;
      if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " has only one doughter " <<std::endl;
    }
    if( (Cell->GetDau0()==NULL) && (Cell->GetDau1()==NULL) && (Cell->GetStat()==0) ){
      Errors++;
      if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " has no dougter and is inactive " <<std::endl;
    }
    if( (Cell->GetDau0()!=NULL) && (Cell->GetDau1()!=NULL) && (Cell->GetStat()==1) ){
      Errors++;
      if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " has two dougters and is active " <<std::endl;
    }

// checking parents
    if( (Cell->GetPare())!=m_Cells[0] ){ // not child of the root
      if ( (Cell != Cell->GetPare()->GetDau0()) && (Cell != Cell->GetPare()->GetDau1()) ){
        Errors++;
        if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " parent not pointing to this cell " <<std::endl;
      }
    }

// checking doughters
    if(Cell->GetDau0()!=NULL){
      if(Cell != (Cell->GetDau0())->GetPare()){
        Errors++;
        if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " doughter 0 not pointing to this cell " <<std::endl;
      }
    }
    if(Cell->GetDau1()!=NULL){
      if(Cell != (Cell->GetDau1())->GetPare()){
        Errors++;
        if (level==1) std::cout <<  "ERROR: Cell's no " << iCell << " doughter 1 not pointing to this cell " <<std::endl;
      }
    }
  }// loop after cells;

// check on vertices
  if(m_nDim>0){
    for(iVert=0; iVert<=m_LastVe; iVert++){
      Ref = 0; //False
      for(jCe=0; jCe<=m_LastCe; jCe++){
	Cell = m_Cells[jCe];
	int nVert = Cell->GetnVert();
	for(k=0; k<nVert; k++){
	  if( (*Cell)[k]==m_VerX[iVert] ){
	    Ref = 1; break; // True;
	  }
	}
	if (Ref==1) break;
      }
      if (Ref==0){
	Errors++;
	if (level==1) std::cout << "ERROR: Vertex no. " << iVert << " NOT referenced!" <<std::endl;
      }
    }
  }

// Check for empty cells
  for(iCell=0; iCell<=m_LastCe; iCell++){
    Cell = m_Cells[iCell];
    if( (Cell->GetStat()==1) && (Cell->GetDriv()==0) ){
      Warnings++;
      if(level==1) std::cout << "Warning: Cell no. " << iCell << "is active but empty " <<std::endl;
    }
  }
// summary
  if(level==1){
    std::cout << "TFOAM::Check has found " << Errors << " errors and " << Warnings << " warnings" <<std::endl;
    std::cout << "/////////////////////////////////////////////////////////////////////////" <<std::endl;
  }
  if(Errors>0){
    std::cout << "STOP in TFOAM::Check - found total " << Errors << " errors " <<std::endl;
    exit(0);
  }
} // Check

///////////////////////////////////////////////////////////////////////////////

void CVMGenFoam::TFOAM::PrintVertices(void){
// Prints ALL vertices of the FOAM.
  long iVert;

  if(m_nDim>0){
    for(iVert=0; iVert<=m_LastVe; iVert++){
      std::cout<<"Vert["<<iVert <<"]=  ";
      m_VerX[iVert]->Print();
      //std::cout<<"  {"<<m_VerX[iVert]<<"} ";   // extra DEBUG
      std::cout<<std::endl;
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::PrintCells(void){
// Prints ALL cells of the FOAM
  long iCell;

  for(iCell=0; iCell<=m_LastCe; iCell++){
    std::cout<<"Cell["<<iCell<<"]={ ";
    //std::cout<<"  "<< m_Cells[iCell]<<"  ";  // extra DEBUG
    std::cout<<std::endl;
    m_Cells[iCell]->Print();
    std::cout<<"}"<<std::endl;
  }
}

///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::RootPlot2dim(char *filename){
// Debugging tool which plots cells as triangles or rectangles
// in C++ in format readible for root 
	std::ofstream outfile(filename, std::ios::out);
  double   x1,y1,x2,y2,x3,y3,x,y;
  long    iCell, iVe;
  long    k;
  double offs =0.1;
  double lpag   =1-2*offs;
  outfile<<"{" <<std::endl;
  outfile<<"cMap = new TCanvas(\"Map1\",\" Cell Map \",600,600);"<<std::endl;
  //
  outfile<<"TBox*a=new TBox();"<<std::endl;
  outfile<<"a->SetFillStyle(0);"<<std::endl;  // big frame
  outfile<<"a->SetLineWidth(4);"<<std::endl;
  outfile<<"a->SetLineColor(2);"<<std::endl;
  outfile<<"a->DrawBox("<<offs<<","<<offs<<","<<(offs+lpag)<<","<<(offs+lpag)<<");"<<std::endl;
  //
  outfile<<"TText*t=new TText();"<<std::endl;  // text for numbering
  outfile<<"t->SetTextColor(4);"<<std::endl;
  if(m_LastCe<51)
    outfile<<"t->SetTextSize(0.025);"<<std::endl;  // text for numbering
  else if(m_LastCe<251)
    outfile<<"t->SetTextSize(0.015);"<<std::endl;
  else
    outfile<<"t->SetTextSize(0.008);"<<std::endl;
  //
  outfile<<"TBox*b=new TBox();"<<std::endl;  // single cell
  outfile <<"b->SetFillStyle(0);"<<std::endl;
  //
  if(m_kDim==2 && m_LastCe<=2000){
    TFVECT  Posi(m_kDim); TFVECT  Size(m_kDim);
    outfile << "// =========== Rectangular cells  ==========="<<std::endl;
    for(iCell=1; iCell<=m_LastCe; iCell++){
      if( m_Cells[iCell]->GetStat() == 1){
	m_Cells[iCell]->GetHcub(Posi,Size);
	x1 = offs+lpag*(        Posi[0]); y1 = offs+lpag*(        Posi[1]);
	x2 = offs+lpag*(Posi[0]+Size[0]); y2 = offs+lpag*(Posi[1]+Size[1]);
	//     cell rectangle
	if(m_LastCe<=2000)
	  outfile<<"b->DrawBox("<<x1<<","<<y1<<","<<x2<<","<<y2<<");"<<std::endl;
	//     cell number
	if(m_LastCe<=250){
	  x = offs+lpag*(Posi[0]+0.5*Size[0]); y = offs+lpag*(Posi[1]+0.5*Size[1]);
	  outfile<<"t->DrawText("<<x<<","<<y<<","<<"\""<<iCell<<"\""<<");"<<std::endl;
	}
      }
    }
    outfile<<"// ============== End Rectangles ==========="<<std::endl;
  }//kDim=2
  //
  if(m_nDim==2 && m_LastCe<=2000){
  if(m_OptVert) StopM("RootPlot2dim:: You can only use this method with OptVert=0");
    int  *NoRefsAc = new int[m_vMax];
    outfile<<"// =========== Vertices Vertices ==========="<<std::endl;
    outfile<<"TMarker*mstar=new TMarker(0.0,0.0,30);"<<std::endl;
    outfile<<"TMarker*mdisc=new TMarker(0.0,0.0,20);"<<std::endl;
    outfile<<"mdisc->SetMarkerColor(3);"<<std::endl;
    //Count references of vertices
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++) NoRefsAc[iVe] = 0;
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++)
      for(iCell = 0 ; iCell<m_LastCe ; iCell++){
	TFCELL *Cell = m_Cells[iCell];
	for(k = 0 ; k<m_nDim+1 ; k++){
	  if( Cell->GetStat() == 1)
	    if( (*Cell)[k] == m_VerX[iVe] ) NoRefsAc[iVe]++;
	}
      }
    //Loop over all vertices
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++){
      x = offs+lpag*((*( m_VerX[iVe]))[0]);
      y = offs+lpag*((*( m_VerX[iVe]))[1]);
      if( NoRefsAc[iVe] <= 2 )
	outfile<<"mstar->DrawMarker("<<x<<","<<y<<");"<<std::endl;
      else
	outfile<<"mdisc->DrawMarker("<<x<<","<<y<<");"<<std::endl;
      if(m_LastVe<=250)
	outfile<<"t->DrawText("<<x+0.02<<","<<y<<","<<"\""<<iVe<<"\""<<");"<<std::endl;
    }
    outfile<<"// ============== End Vertices ============="<<std::endl;
    delete [] NoRefsAc;
    //
    outfile<<"// ==========Triangle  Cells  ==============="<<std::endl;
    int iCeStart=1;
    if(m_OptOrd==1) iCeStart=0;
    for(iCell=iCeStart;  iCell<=m_LastCe ; iCell++){
      TFVECT *v1 = (*m_Cells[iCell])[0];
      TFVECT *v2 = (*m_Cells[iCell])[1];
      TFVECT *v3 = (*m_Cells[iCell])[2];
      x1 = offs+lpag*(( *v1 )[0]);
      y1 = offs+lpag*(( *v1 )[1]);
      x2 = offs+lpag*(( *v2 )[0]);
      y2 = offs+lpag*(( *v2 )[1]);
      x3 = offs+lpag*(( *v3 )[0]);
      y3 = offs+lpag*(( *v3 )[1]);
      outfile<<"TGraph *triang = new TGraph(4);"<<std::endl;
      outfile<<"triang->SetPoint(0,"<< x1 <<","<< y1 <<");"<<std::endl;
      outfile<<"triang->SetPoint(1,"<< x2 <<","<< y2 <<");"<<std::endl;
      outfile<<"triang->SetPoint(2,"<< x3 <<","<< y3 <<");"<<std::endl;
      outfile<<"triang->SetPoint(3,"<< x1 <<","<< y1 <<");"<<std::endl;
      outfile<<"triang->Draw(\"L\");"<<std::endl;
    }
    outfile << "// =========== End Triangle  Cells ===========" <<std::endl;
  }//nDim=2
  //
  outfile << "}" <<std::endl;
  outfile.close();
}

///////////////////////////////////////////////////////////////////////////////
void CVMGenFoam::TFOAM::LaTexPlot2dim(char *filename){
// Debugging tool which plots cells as triangles or rectangles
// in the LeTeX picture environment. 
  int     icont = 0;
  long    iCell, iVe;
  long    k;
  int     kx1,ky1,kx2,ky2,kx3,ky3,kx,ky,lx,ly;
  const char star[]    = "\\makebox(0,0){\\Large\\color{red} $\\star$}";
  const char disc[]    = "\\circle*{20}";
  const char dot[]     = "\\circle*{4}";
  std::ofstream outfile(filename, std::ios::out);
  
  icont++;
  if (icont >= 1) outfile << "\\newpage" <<std::endl;
//-------------------------------------------------------------------------
//           Header
//-------------------------------------------------------------------------
  outfile << "\\documentclass[12pt]{article}" <<std::endl;
  outfile << "\\usepackage{color}" <<std::endl;   // <-for colors!!!
  outfile << "\\usepackage{epic}"  <<std::endl;   // <-for extended ploting
  outfile << "\\textwidth  = 16cm" <<std::endl;
  outfile << "\\textheight = 18cm" <<std::endl;
  outfile << "\\pagestyle{empty}"  <<std::endl;
  outfile << "\\begin{document}"   <<std::endl;
  outfile <<std::endl;
//------------------------------!
  outfile << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%" <<std::endl;
  outfile << "\\begin{figure}[!ht]" <<std::endl;
  outfile << "\\centering" <<std::endl;
//------------------------------!
// Frames and labels
//------------------------------!
  outfile << "% =========== big frame, title etc. =======" <<std::endl;
  outfile << "\\setlength{\\unitlength}{0.1mm}" <<std::endl;
  outfile << "\\begin{picture}(1600,1600)" <<std::endl;
  outfile << "\\put(0,0){\\framebox(1600,1600){ }}" <<std::endl;

  if(m_kDim==2 && m_LastCe<=2000){
    TFVECT  Posi(m_kDim); TFVECT  Size(m_kDim);
    outfile << "% =========== Rectangulare cells  ===========" <<std::endl;
    outfile << "\\newcommand{\\VD}[2]{\\put(#1,#2){" << dot << "}}" <<std::endl;
    for(iCell=1; iCell<=m_LastCe; iCell++){
      if( m_Cells[iCell]->GetStat() == 1){
	m_Cells[iCell]->GetHcub(Posi,Size);
	kx = (int)(Posi[0]*1600); ky = (int)(Posi[1]*1600);
	lx = (int)(Size[0]*1600); ly = (int)(Size[1]*1600);
	kx1 = kx+lx/2; ky1 = ky+ly/2;
	//     cell rectangle
	if(m_LastCe<=5000) outfile<<"\\put("<<kx <<","<<ky <<
                "){\\framebox("<<lx<<","<<ly<<"){ }}"<<std::endl;
	//     cell number
	if(m_LastCe<=250) outfile<<"\\put("<<kx1<<","<<ky1<<
	        "){\\makebox(0,0)[b]{\\hbox{\\small\\color{magenta}\\scriptsize "<<iCell<<" }}}"<<std::endl;
      }
    }
    outfile << "% ============== End Rectangles ===========" <<std::endl;
  }//kDim=2
//-----------------------------------------------------------------------------
  if(m_nDim==2 && m_LastCe<=2000){
    if(m_OptVert) StopM("LaTexPlot2dim:: You can only use this method with OptVert=0");
    outfile << "% =========== Vertices Vertices ===========" <<std::endl;
    //Count references of vertices
    int  *NoRefsAc = new int[m_vMax];
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++) NoRefsAc[iVe] = 0;
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++)
      for(iCell = 0 ; iCell<m_LastCe ; iCell++){
	TFCELL *Cell = m_Cells[iCell];
	for(k = 0 ; k<m_nDim+1 ; k++){
	  if( Cell->GetStat() == 1)
	    if( (*Cell)[k] == m_VerX[iVe] ) NoRefsAc[iVe]++;
	}
      }
    //Plotting symbol
    outfile << "\\newcommand{\\VD}[2]{\\put(#1,#2){" << disc << "}}" <<std::endl;
    outfile << "\\newcommand{\\VS}[2]{\\put(#1,#2){" << star << "}}" <<std::endl;
    outfile << "\\newcommand{\\VN}[3]{\\put(#1,#2){\\makebox(0,0)[b]{\\hbox{\\small\\color{red} #3}}}}" <<std::endl;
    //Loop[ over all vertices
    for(iVe = 0 ; iVe<=m_LastVe ; iVe++){
      kx = (int)((*( m_VerX[iVe]))[0]*1600);
      ky = (int)((*( m_VerX[iVe]))[1]*1600);
      if( NoRefsAc[iVe] <= 2 )
	outfile << "\\VD{" << kx << "}{" << ky << "}" <<std::endl;
      else
	outfile << "\\VS{" << kx << "}{" << ky << "}" <<std::endl;
      if(m_LastVe<=250) outfile << "\\VN{" << kx-8 << "}{" << ky+12 << "}{" << iVe << "}" <<std::endl;
    }
    outfile << "% ============== End Vertices ===========" <<std::endl;
    delete [] NoRefsAc;
  }
//-----------------------------------------------------------------------------
  if(m_nDim==2 && m_LastCe<=2000){
    outfile << "% ==========Triangle  Cells  ===============" <<std::endl;
    int iCeStart=1;
    if(m_OptOrd==1) iCeStart=0;
    for(iCell=iCeStart;  iCell<=m_LastCe ; iCell++){
      TFVECT *v1 = (*m_Cells[iCell])[0];
      TFVECT *v2 = (*m_Cells[iCell])[1];
      TFVECT *v3 = (*m_Cells[iCell])[2];
      kx1 = (int)( ( *v1 )[0] *1600 );
      ky1 = (int)( ( *v1 )[1] *1600 );
      kx2 = (int)( ( *v2 )[0] *1600 );
      ky2 = (int)( ( *v2 )[1] *1600 );
      kx3 = (int)( ( *v3 )[0] *1600 );
      ky3 = (int)( ( *v3 )[1] *1600 );
      kx= (kx1+kx2+kx3)/3;
      ky= (ky1+ky2+ky3)/3;
      if( m_Cells[iCell]->GetStat() == 1)
	{
	  outfile << "\\drawline(" << kx1 << "," << ky1 << ")(" << kx2 << "," << ky2 << ")" <<std::endl;
	  outfile << "\\drawline(" << kx2 << "," << ky2 << ")(" << kx3 << "," << ky3 << ")" <<std::endl;
	  outfile << "\\drawline(" << kx3 << "," << ky3 << ")(" << kx1 << "," << ky1 << ")" <<std::endl;
	  if(m_LastCe<=250)
	    outfile << "\\put("      << kx  << "," << ky  
		    << "){\\makebox(0,0)[b]{\\hbox{\\small\\color{magenta} " << iCell << " }}}" <<std::endl;
	}
    }
  }
  // Close frame
  outfile << "% ============== End Cells ===========" <<std::endl;
  
  outfile << "\\end{picture}" <<std::endl;
  outfile << "\\end{figure}" <<std::endl;
  outfile << "\\end{document}" <<std::endl;
  
  outfile.close();
}
///////////////////////////////////////////////// CVMGenFoam::TVMLIB_INTEGRAND
CVMGenFoam::TVMLIB_INTEGRAND::TVMLIB_INTEGRAND(int n, CVMGenFunc* func, tReal* _VarMinRef, tReal* _VarMaxRef) : CVMGenFoam::TFOAM_INTEGRAND() {
	m_n=n; m_pFunc=((func!=NULL)?func->Clone():NULL); m_VarMinRef=_VarMinRef; m_VarMaxRef=_VarMaxRef;
	if ((m_VarMinRef==NULL) || (m_VarMaxRef==NULL) || (m_pFunc==NULL) || (m_n<=0)) { m_n=-1; m_pFunc=NULL; }
}
CVMGenFoam::TVMLIB_INTEGRAND::~TVMLIB_INTEGRAND() { if(m_pFunc!=NULL){ delete m_pFunc; m_pFunc=NULL; } }
double CVMGenFoam::TVMLIB_INTEGRAND::Density(int n, double *var) {
	if ((n!=m_n) || (m_pFunc==NULL) || (var==NULL)) return 0.;
	for(int i=0; i<m_n; i++) { m_LocalVars[i]=m_VarMinRef[i]+(m_VarMaxRef[i]-m_VarMinRef[i])*var[i]; }
	return m_pFunc->Call(m_LocalVars);
}

/////////////////////////////////////////////////////////////////////// CVMGenFoam::CVMGenFoam
CVMGenFoam::CVMGenFoam() : CVMGen() { m_pFoam=NULL; m_nEvtsGen=0; m_FoamFunc=NULL; m_bLog=false; }
CVMGenFoam::~CVMGenFoam() {
	if(m_pFoam!=NULL){ delete m_pFoam; m_pFoam=NULL; }
	if(m_FoamFunc!=NULL){ delete m_FoamFunc; m_FoamFunc=NULL; }
}
const char* CVMGenFoam::ClassName() { return "Foam"; }
void CVMGenFoam::SelfTest() {
	long   loop;
	double MCresult,MCerror,MCwt;
	int    FunType;
	//=========================================================
	long NevTot   =    200000;   // Total MC statistics
	int  nDim     =       3;   // SIMPLICAL   subspace
	int  kDim     =       0;   // HYP-CUBICAL subspace
	int  TotDim   = kDim+ nDim;// total dimension
	int  nCells   =    5000;   // Number of Cells
	int  nSampl   =     500;   // Number of MC events per cell in build-up
	int  nBin     =       8;   // Number of bins in build-up
	int  OptRej   =       0;   // =0, weighted events;  =1, wt=1 events
	int  OptDrive =       2;   // (D=2) Option, type of Drive =0,1,2 for TrueVol,Sigma,WtMax
	int  OptEdge  =       0;   // (D=0) Option, vertices are included in the sampling (1) or not (0)
	int  OptOrd   =       0;   // (D=0) Option, single simplex (1) or nDim! simplices (0)
	int  OptPeek  =       0;   // (D=0) Option, choice of cell in buid-up maximum(0), random(1)
	int  OptMCell =       1;   // (D=1) Option, Mega-Cell = slim memory
	int  OptVert  =       1;   // (D=1) Vertices are not stored
	int  EvPerBin =      25;   // Maximum events (equiv.) per bin in buid-up
	double MaxWtRej =   2.0;   // Maximum wt for rejection, for OptRej=1
	int  Chat     =       1;   // Chat level
	//=========================================================
	FunType =4; //Camel
	TFDISTR *Density1 = new TFDISTR(FunType); // Create integrand function
	TFOAM   *FoamX    = new TFOAM("FoamX");   // Create Simulator
	//=========================================================
	FoamX->SetnDim(        nDim);
	FoamX->SetkDim(        kDim);
	FoamX->SetnCells(      nCells);
	FoamX->SetnSampl(      nSampl);
	FoamX->SetnBin(        nBin);
	FoamX->SetOptRej(      OptRej);
	FoamX->SetOptDrive(    OptDrive);
	FoamX->SetOptPeek(     OptPeek);
	FoamX->SetOptEdge(     OptEdge);
	FoamX->SetOptOrd(      OptOrd);
	FoamX->SetOptMCell(    OptMCell);
	FoamX->SetOptVert(     OptVert);
	FoamX->SetEvPerBin(    EvPerBin);
	FoamX->SetMaxWtRej(    MaxWtRej);
	FoamX->SetChat(        Chat);
	//===============================
	FoamX->Initialize(m_rnd,  Density1 ); // Initialize simulator
	//===============================
	long nCalls=FoamX->GetnCalls();
	if((nDim==2)||(kDim==2)){ FoamX->LaTexPlot2dim("Demo-map.tex"); }
	std::cout << "====== Initialization done, entering MC loop" << std::endl;
	//======================================================================
	double *MCvect =new double[TotDim]; // vector generated in the MC run
	//======================================================================
	CVMHistLite *hst_Wt = new CVMHistLite("hst_Wt",25,0.0,1.25);
	//======================================================================
	for(loop=0; loop<NevTot; loop++){
		FoamX->MakeEvent();           // generate MC event
		FoamX->GetMCvect( MCvect);
		MCwt=FoamX->GetMCwt();
		hst_Wt->Fill(MCwt,1.0);    
		if(loop<15){
			std::cout<<"MCwt= "<<MCwt<<",  ";
			std::cout<<"MCvect= ";
			for ( int k=0 ; k<TotDim ; k++) std::cout<<MCvect[k]<<" "; std::cout<< std::endl;
		}
		if( ((loop)%100000)==0 ){
			std::cout<<"   loop= "<<loop<<std::endl;
		}
	}
	//======================================================================
	std::cout << "====== Events generated, entering Finalize" << std::endl;
	CVMHistLiteView hview(hst_Wt); hview.SaveAsEPS("all.eps");

	double eps = 0.0005;
	double Effic, WtMax, AveWt, Sigma;
	double IntNorm, Errel;
	FoamX->Finalize(   IntNorm, Errel);     // final printout
	FoamX->GetIntegMC( MCresult, MCerror);  // get MC integral
	FoamX->GetWtParams(eps, AveWt, WtMax, Sigma); // get MC wt parameters
	Effic=0; if(WtMax>0) Effic=AveWt/WtMax;
	std::cout << "================================================================" << std::endl;
	std::cout << " MCresult= " << MCresult << " +- " << MCerror << " RelErr= "<< MCerror/MCresult << std::endl;
	std::cout << " Dispersion/<wt>= " << Sigma << std::endl;
	std::cout << "      <wt>/WtMax= " << Effic <<",    for epsilon = "<<eps << std::endl;
	std::cout << " nCalls (initialization only) =   " << nCalls << std::endl;
	std::cout << "================================================================" << std::endl;
	delete [] MCvect;
}
bool CVMGenFoam::InitImpl(int nDim, tReal* _VarMin, tReal* _VarMax, CVMGenFunc* func, bool bFuncOwner, CVMRnd* rndCpy);
	bool ret=CVMGen::InitImpl(nDim, _VarMin, _VarMax, func, bFuncOwner, rndCpy); if(!ret)return false;

	//=========================================================
	if(m_FoamFunc!=NULL){delete m_FoamFunc; m_FoamFunc=NULL;}
	m_FoamFunc=new TVMLIB_INTEGRAND(m_nDim,func,m_VarMin,m_VarMax);
	if(m_pFoam!=NULL){ delete m_pFoam; m_pFoam=NULL; }
	m_pFoam=new TFOAM("FoamX");
	m_pFoam->SetnDim(m_nDim);
	m_pFoam->SetkDim(0); // HYP-CUBICAL subspace
	m_pFoam->SetnCells(200); // No. of cells, can be omitted, default=2000
	//m_pFoam->SetnCells(5000); // Number of Cells
	//m_pFoam->SetnSampl(900); // No. of cells, can be omitted, default=2000
	m_pFoam->SetnSampl(500); // Number of MC events per cell in build-up
	m_pFoam->SetnBin(8); // Number of bins in build-up
	m_pFoam->SetOptRej(0); // =0, weighted events;  =1, wt=1 events
	m_pFoam->SetOptDrive(2); // (D=2) Option, type of Drive =0,1,2 for TrueVol,Sigma,WtMax
	m_pFoam->SetOptPeek(0); // (D=0) Option, choice of cell in buid-up maximum(0), random(1)
	m_pFoam->SetOptEdge(0); // (D=0) Option, vertices are included in the sampling (1) or not (0)	
	m_pFoam->SetOptOrd(0); // (D=0) Option, single simplex (1) or nDim! simplices (0)
	m_pFoam->SetOptMCell(1); // (D=1) Option, Mega-Cell = slim memory
	m_pFoam->SetOptVert(1); // (D=1) Vertices are not stored
	m_pFoam->SetEvPerBin(25); // Maximum events (equiv.) per bin in buid-up
	m_pFoam->SetMaxWtRej(2.0); // Maximum wt for rejection, for OptRej=1
	m_pFoam->SetChat((m_bLog?1:0)); // Chat level
	m_pFoam->Initialize(m_rnd,m_FoamFunc); // Initialize FOAM
	//=========================================================
	long nCalls=m_pFoam->GetnCalls();
	//if((m_bLog)&&(m_nDim==2)){ m_pFoam->LaTexPlot2dim("Demo-map.tex"); }
	if(m_bLog){
		std::cout<<"====== Initialization done ======"<<std::endl;
		std::cout << " nCalls (initialization only) =   " << nCalls << std::endl;
	}

	m_nEvtsGen=0;
	return true;
}
tReal CVMGenFoam::Integrate(double fPrecision) { // To be called after event generation
	long nEvMin = 100000; // minimal no. of events (? required ?) to calculate integral
	//std::cout<<"====== CVMGenFoam::Integrate() started. nEvMin="<<nEvMin<<std::endl;
	if (m_nEvtsGen<nEvMin) {
		for(long i=m_nEvtsGen; i<nEvMin; i++){
			m_pFoam->MakeEvent();
			if(m_bLog){ if( (i%1000)==0 ){ std::cout<<"   loop= "<<i<<std::endl; } }
		}
	}
	if(m_bLog){ std::cout<<"====== Events generated, entering Finalize"<<std::endl; }
	//======================================================================

	double IntNorm, Errel; m_pFoam->Finalize(IntNorm, Errel);			// final printouts
	double MCresult,MCerror; m_pFoam->GetIntegMC(MCresult, MCerror);	// get MC integral
	double eps=0.0005; double WtMax,AveWt,Sigma; m_pFoam->GetWtParams(eps, AveWt, WtMax, Sigma); // get MC wt parameters
	double Effic=0; if(WtMax>0) Effic=AveWt/WtMax;
	if(m_bLog){
		std::cout << "================================================================" << std::endl;
		std::cout << " MCresult= " << MCresult << " +- " << MCerror << " RelErr= "<< MCerror/MCresult << std::endl;
		std::cout << " Dispersion/<wt>= " << Sigma << std::endl;
		std::cout << "      <wt>/WtMax= " << Effic <<",    for epsilon = "<<eps << std::endl;
		std::cout << "================================================================" << std::endl;
	}

	// Rescale from [0..1] to [min..max] volume
	m_Integral=MCresult*m_Volume; m_Error=MCerror*m_Volume;
	m_nFuncCalls=m_pFoam->GetnCalls();
	return m_Integral;
}
bool CVMGenFoam::GenEventWgt() {
	if(m_pFoam==NULL){ m_Weight=0.; m_WeightDebug=0.; return false; }
	m_Weight=m_pFoam->MCgenerate(m_GenValues);
	m_nFuncCalls=m_pFoam->GetnCalls();
	m_nEvtsGen++;

	// rescale points & weight
	for(int i=0; i<m_nDim; i++) { m_GenValues[i]=m_VarMin[i]+(m_VarMax[i]-m_VarMin[i])*m_GenValues[i]; }
	m_Weight*=m_Volume; //Volume=1 --> ActualVolume

	//Normalize
	m_Weight*=m_pFoam->GetPrimary();
	m_WeightDebug=m_Weight;
	return true;
}
#endif // !defined(CVMGen_NO_FOAM)

///////////////////////////////////// CVMGenTest
CVMGenTest::CIntegrateCircleImpl::CIntegrateCircleImpl(int nDim) : CVMGen::CVMGenFunc() { m_nDim=nDim; }
tReal CVMGenTest::CIntegrateCircleImpl::Call(tReal* params) {
	tReal r2=0.; for(int i=0; i<m_nDim; i++) { r2+=params[i]*params[i]; } return (r2<=1.);
}
CVMGen::CVMGenFunc* CVMGenTest::CIntegrateCircleImpl::Clone() const { return new CIntegrateCircleImpl(m_nDim); }
bool CVMGenTest::IntegrateCircle(int nDim, tReal prec, CVMGen* gen) {
	bool bDelGen=false; if(gen==NULL){ gen=new CVMGenLite(); bDelGen=true; }
	if(nDim<2){ nDim=2; std::cout<<"[CVMGenTest::IntegrateCircle] 'nDim' set to "<<nDim<<std::endl; }
	if((prec<0.) || (prec>1.)) { prec=0.01; std::cout<<"[CVMGenTest::IntegrateCircle] 'precision' set to "<<prec<<std::endl; }
	long i;

	//
	tReal pi=atan((tReal)1.)*((tReal)4.);
	//V_n=R^n*C_n, C_n=pi^{n/2}/Gamma{n/2+1}, C_2n=pi^n/n!, C_{2n+1}=2^{n+1}pi^n/(2n+1)!!, 2n!!=2*4*6...*2n=2^n*n!, (2n+1)!!=1*3*5...*(2n+1)=(2n+1)!/(2n)!
	tReal exp=1.;
	if(nDim%2==0) {
		int k=nDim/2;
		tReal c_2k=pow(pi,(tReal)k); for(i=1;i<=k;i++){ c_2k=c_2k/i; } //C_2k=pi^k/k!
		exp*=c_2k;
	}else{
		int k=(nDim-1)/2;
		tReal c_2kp1=2.*pow(2.*pi,(tReal)k); for(i=1;i<=nDim;i+=2){ c_2kp1=c_2kp1/i; } //C_{2k+1}=2*(2pi)^k/(2k+1)!!
		exp*=c_2kp1;
	}

	//
	tReal* varMin=new tReal[nDim]; tReal* varMax=new tReal[nDim]; for(i=0;i<nDim;i++){ varMin[i]=-1.; varMax[i]=1.; }
	CIntegrateCircleImpl impl(nDim);
	gen->Init(nDim,varMin,varMax,&impl);
	//gen->SetExpectedIntegral(exp*0.9);
	tReal res = gen->Integrate(prec);
	if(gen->ClassName() == CVMGenLite::CLASS_NAME) { ((CVMGenLite*)gen)->ViewGrid(); }

	tReal fDiff = ((res-exp)/exp);
	bool bFit = (fDiff <= prec*10.);
	int dig=(int)(log(1.1/prec)/log(10.)); if(dig<3)dig=3; int digOld=std::cout.precision(dig);
	std::cout<<"[CVMGenTest::IntegrateCircle] result:   "<<res<<std::endl;
	std::cout<<"[CVMGenTest::IntegrateCircle] expected: "<<exp<<", diff: "<<fDiff<<std::endl;
	std::cout.precision(digOld);
	std::cout<<"[CVMGenTest::IntegrateCircle] test "<<(bFit?"passed successfully":"failed")<<std::endl;

	//Test event generation efficiency
	for(i=0;i<1000;i++) { if(!gen->GenEvent())break; }
	std::cout<<"[CVMGenTest::IntegrateCircle] generation efficiency: "<<gen->GetGenEventEfficiency()<<" (plain generator: "<<(exp/pow(2.,nDim))<<")"<<std::endl;

	if(bDelGen)delete gen;
	delete[] varMin; delete[] varMax;
	return bFit;
}
